package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.ApiClassifyVo;
import com.yizhi.course.application.vo.domain.ClassifyEntityVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;


@FeignClient(name = "course", contextId = "ClassifyClient")
public interface ClassifyClient {

    @GetMapping("/manage/classify/find/id")
    ClassifyEntityVo getById(@RequestParam("id") Long id);

    @GetMapping("/manage/classify/check/name")
    boolean checkExtisClassifyName(@RequestParam("name") String name,
                                  @RequestParam(value = "parentId" ,required = false) Long parentId,
                                  @RequestParam("siteId") Long siteId,
                                  @RequestParam("type") Integer type,
                                  @RequestParam(value = "classifyId" ,required = false) Long classifyId);
    /**
     * 添加分类
     *
     * @param classify
     * @return
     */
    @PostMapping("/manage/classify/save")
    int saveClassify(@RequestBody ClassifyEntityVo classify);

    /**
     * 删除分类
     *
     * @param id
     * @return
     */
    @PostMapping("/manage/classify/delete")
    int deleteClassify(@RequestBody ClassifyEntityVo classify);

    /**
     * 修改分类
     *
     * @param classify
     * @return
     */
    @PostMapping("/manage/classify/update")
    int updateClassify(@RequestBody ClassifyEntityVo classify);

    /**
     * 查询分类
     *
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/manage/classify/list")
    Page<ClassifyEntityVo> listClassify(
            @RequestParam(required = false,name = "isStartHQ") Boolean isStartHQ,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "companyId") Long companyId,
            @RequestParam(value = "siteId") Long siteId,
            @RequestParam(name = "orgIds", required = false) List<Long> orgIds
    );

    /**
     * 学员端查询分类
     *
     * @return
     */
    @GetMapping("/api/classify/list")
    List<ClassifyEntityVo> listStuClassify(
            @RequestParam(value = "companyId") Long companyId,
            @RequestParam(value = "siteId") Long siteId
    );

    /**
     * 分类排序
     *
     * @param list
     * @return
     */
    @PostMapping("/manage/classify/sort")
    int sortClassify(@RequestBody List<ClassifyEntityVo> list);


    @GetMapping(value = "/manage/classify/page/list")
    Page<ApiClassifyVo> pageList(
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "orgIds", required = false) List<Long> orgIds
    );

    /**
     * 验证同站点下 同级的分类不能重名
     *
     * @param name      分类名
     * @param layer     分类层级
     * @param companyId 企业名
     * @param siteId    站点id
     * @param id        分类id （修改的时候传入）
     * @return Boolean
     */
    @GetMapping("/manage/classify/vail/name")
    Boolean vailName(
            @RequestParam(name = "name") String name,
            @RequestParam(name = "layer") Integer layer,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "id",required = false) Long id

    );
    
    @GetMapping("/manage/classify/getClassifyName")
    public String getClassifyName(@RequestParam("classifyId")Long classifyId);
}
