package com.yizhi.course.application.feign;

import com.yizhi.course.application.vo.ClassifyGroupInfoVO;
import com.yizhi.course.application.vo.ClassifyGroupStudentVO;
import com.yizhi.course.application.vo.ClassifyGroupVO;
import com.yizhi.course.application.vo.GroupClassifyItemVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

@FeignClient(name = "course", contextId = "ClassifyGroupClient")
public interface ClassifyGroupClient {

    /**
     * 勾选分类之后进行保存
     * @param classifyGroupInfoVO
     * @return
     */
    @PostMapping("/classify/group/save")
    List<ClassifyGroupVO> save(@RequestBody ClassifyGroupInfoVO classifyGroupInfoVO);

    /**
     * 全量保存
     * @param classifyGroupInfoVO
     * @return
     */
    @PostMapping("/classify/group/save/all")
    List<ClassifyGroupVO> saveAll(@RequestBody ClassifyGroupInfoVO classifyGroupInfoVO);

    /**
     * 获取 每一级分类信息和上次保存的分类信息
     * @param id 创建的分类组id（主信息ID）
     * @param classifyId 当前选择的分类id（首次进来时候不要传）
     * @param layer 选择的分类层级（1表示一级......3表示三级）,没有选择不要传
     * @return （classifyData存放分类列表信息； classifyGroupData存放选择分类的选择过的）
     */
    @GetMapping("/classify/group/get")
    List<GroupClassifyItemVO> get(@RequestParam("id") Long id,
                                  @RequestParam(value = "classifyId" ,required = false) Long classifyId,
                                  @RequestParam(value = "layer",required = false) Integer layer);

    /**
     * 获取被选择的分类id
     * @param id
     * @return
     */
    @GetMapping("/classify/group/get/select/classifyIds")
    List<Long> getSelectIds(@RequestParam("id") Long id);

    @GetMapping("/classify/group/join/status")
    Boolean joinStatus(@RequestParam("cid") Long cid);

    /**
     * 前端显示分类组下的全部关联信息
     * @param id
     * @return
     */
    @GetMapping("/classify/group/view")
    ClassifyGroupStudentVO view(@RequestParam(name = "id") Long id);

    @GetMapping("/classify/group/manage/view")
    ClassifyGroupStudentVO manageView(@RequestParam(name = "id") Long id,
                                      @RequestParam(name = "keyName",required = false) String keyName);


}
