package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.ClassifyGroupStudentVO;
import com.yizhi.course.application.vo.domain.ClassifyGroupInfoEntityVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

@FeignClient(name = "course", contextId = "ClassifyGroupInfoClient")
public interface ClassifyGroupInfoClient {

    /**
     * 保存分类组的基本信息
     * @param classifyGroupInfo
     * @return
     */
    @PostMapping("/classify/group/info/save")
    ClassifyGroupInfoEntityVo save(@RequestBody ClassifyGroupInfoEntityVo classifyGroupInfo);

    /**
     * 获取分类组的列表信息
     * @param pageNo 页码
     * @param pageSize 每页大小
     * @param status 状态（可以不传，查询全部状态，传了就查询对应状态的
     * @return
     */
    @GetMapping("/classify/group/info/list")
    Page<ClassifyGroupInfoEntityVo> list(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                 @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize,
                                 @RequestParam(name="status", required = false ) Integer status);

    /**
     * 提供给外部模块使用，根据企业和站点查询该站点下的分类组
     * @param companyId 登录的用户企业id
     * @param siteId 用户当前访问的站点id
     * @return
     */
    @GetMapping("/classify/group/info/list/all")
    List<ClassifyGroupInfoEntityVo> listAll(@RequestParam("companyId") Long companyId,
                                    @RequestParam("siteId") Long siteId);

    /**
     * 选择一条分组信息进行查看
     * @param id 分类组基本信息的唯一id
     * @return
     */
    @GetMapping("/classify/group/info/get")
    ClassifyGroupInfoEntityVo get(@RequestParam(name = "id") Long id);

    /**
     * 删除一条分类组信息
     * @param id  分类组基本信息的唯一id
     * @return
     */
    @GetMapping("/classify/group/info/delete")
    boolean delete(@RequestParam(name = "id") Long id);

    /**
     * 学员端查看（包含分类组基本信息和分类列表信息（层级平铺前端处理）
     * @param id 分类组基本信息的唯一id
     * @return
     */
    @GetMapping("/classify/group/info/view")
    ClassifyGroupStudentVO view(@RequestParam(name = "id") Long id);
}
