package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 13:44 2018/12/19
 * @Modified By
 */
@FeignClient(name = "course", contextId = "CoursePcClient")
public interface CoursePcClient {

    String COURSE_STATUS_STUDYING = "studying";// 学习中的状态
    String COURSE_STATUS_COMPLETE = "complete";// 完成中的状态

    @GetMapping("/pc/recommend")
    List<HomePageCoursePageVo> recommend();

    /**
     * 查询推荐课程列表 版本v2
     */
    @GetMapping("/pc/v2/recommend")
    public List<HomePageCoursePageVoV2> recommendV2();

    @PostMapping("/pc/status/ids/get")
    List<CourseStudentStatusVO> getStatusByIds(@RequestBody List<Long> ids);

    @GetMapping("/pc/status/init")
    void studeyStatusInit(@RequestParam(value = "siteId",required = false) Long siteId);

    @GetMapping("/pc/list")
    public Page<HomePageCoursePageVo> listCourseInfo(
            @RequestParam(value="classifyId",required = false) Long classifyId,
            @RequestParam(value = "pageNo",required = false ) Integer pageNo,
            @RequestParam(value = "pageSize",required = false ) Integer pageSize,
            @RequestParam(value = "isQueryChildren",required = false) Boolean isQueryChildren);

    @GetMapping("/pc/v2/list")
    public Page<HomePageCoursePageVoV2> listCourseInfoV2(
            @RequestParam(value="classifyId",required = false) Long classifyId,
            @RequestParam(value = "pageNo",required = false ) Integer pageNo,
            @RequestParam(value = "pageSize",required = false ) Integer pageSize,
            @RequestParam(value = "isQueryChildren",required = false) Boolean isQueryChildren,
            @RequestParam(value = "hotDesc",required = false) Boolean hotDesc,
            @RequestParam(value = "gradeDesc",required = false) Boolean gradeDesc);

    @GetMapping("/pc/classify/list")
    public List<ClassifyVo> listClassifyInfo(
            @RequestParam(value="classifyId",required = false) Long classifyId);
    
    
    
    @GetMapping("/pc/list/course/grade")
    public List<ModuleAllVoCourse> listCoursePCgrade(@RequestParam(name="ids")List<Long> ids);
    
    @PostMapping("/pc/list/course/get/material/time")
    public Map<Long,Long> getMaterialTimeByCourseIds(@RequestBody List<Long> ids);

    @GetMapping("/pc/list/course/portal/pc/api")
    public List<ModuleAllVoCourse> listCourseScope(@RequestParam(name="relationIds",required=false)List<Long> relationIds, @RequestParam(name="num")Integer num,@RequestParam(name="listIds",required=false)List<Long> listIds);

    /**
     * 章节学习列表，最后一次在最上面
     * @param testId
     * @param accountId
     * @return
     */
    @GetMapping("/pc/chapter/test/list")
    List<TestCaseRecordVO> chapterTestList(@RequestParam(name="testId") Long testId,
                                           @RequestParam(name="accountId") Long accountId);

    @GetMapping("/pc/my/study/count")
    MyCourseStudyCountVo getMyCourseStudyCount(@RequestParam("siteId") Long siteId,
                                                      @RequestParam("accountId") Long accountId);

    @GetMapping("/pc/my/study/list")
    Page<MyCourseRecordeVo> getMyCourseStudyList(
            @RequestParam("siteId") Long siteId,
            @RequestParam("accountId") Long accountId,
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "type", required = false) Integer type, //课程是否完成 0 为进行中 1 为已完成
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize,
            @RequestParam(name = "startDate", required=false) Long startDate,
            @RequestParam(name = "endDate", required=false) Long endDate
    );

    /**
     * 查询课程学时
     * @param ids
     * @return
     */
    @PostMapping("/pc/course/study/hour/get")
    Map<Long,Float> getStudyHourByCourseIds(@RequestBody List<Long> ids);

}
