package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import java.util.Date;
import java.util.List;
import java.util.Map;

@FeignClient(name = "course", contextId = "CourseReportClient")
public interface CourseReportClient {
	
	  /**
	   * 课程----图形报表
	   * @param startDate
	   * @param endDate
	   * @param companyId
	   * @param orgId
	   * @param siteId
	   * @return
	   */
	  @GetMapping("/statisticsCourse/chart/group")
	  public List<ReportStudyCourseChartsVo> chartGroup(
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam("companyId")Long companyId,
			  @RequestParam("orgId")Long orgId,
			  @RequestParam("siteId")Long siteId);
	  
	  
	  
	  /**
	   * 课程---按课程统计
	   * @param kwd
	   * @param startDate
	   * @param endDate
	   * @param companyId
	   * @param orgId
	   * @param siteId
	   * @return
	   */
	  @GetMapping("/statisticsCourse/group")
	  public Page<ReportStudyCourseVo> courseGroup(
			  @RequestParam(value="kwd",required=false)String kwd,
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam("companyId")Long companyId,
			  @RequestParam("orgId")Long orgId,
			  @RequestParam("siteId")Long siteId,
			  @RequestParam(value="pageNo",required=false)Integer pageNo,
			  @RequestParam(value="pageSize",required=false)Integer pageSize);
	  
	  
	  
	  
	  /**
	   * 课程----按照课程统计----详情
	   * @param accountKwd
	   * @param orgKwd
	   * @param courseId
	   * @param startDate
	   * @param endDate
	   * @param pageNo
	   * @param pageSize
	   * @return
	   */
	  @GetMapping("/statisticsCourse/group/view")
	  public Map<String,Object> courseGroupView(
			  @RequestParam(value="accountKwd",required=false)String accountKwd,
			  @RequestParam(value="orgKwd",required=false)String orgKwd,
			  @RequestParam("courseId")Long courseId,
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam(value="pageNo",required=false)Integer pageNo,
			  @RequestParam(value="pageSize",required=false)Integer pageSize);
	  
	  
	  
	  
	  
	  
	  /**
	   * 按照用户统计
	   * @param accountKwd
	   * @param orgKwd
	   * @param startDate
	   * @param endDate
	   * @param companyId
	   * @param orgId
	   * @param siteId
	   * @return
	   */
	  @GetMapping("/statisticsCourse/account/group")
	  public Page<ReportStudyCourseAccountVo> courseAccountGroup(
			  @RequestParam(value="accountKwd",required=false)String accountKwd,
			  @RequestParam(value="orgKwd",required=false)String orgKwd,
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam("companyId")Long companyId,
			  @RequestParam("orgId")Long orgId,
			  @RequestParam("siteId")Long siteId,
			  @RequestParam(value="pageNo",required=false)Integer pageNo,
			  @RequestParam(value="pageSize",required=false)Integer pageSize);
	  
	  
	  /**
	   * 按照用户统计下载  因为上边的那个如果应用到下载执行速度比较慢，因此进行优化了。
	   * @param accountKwd
	   * @param orgKwd
	   * @param startDate
	   * @param endDate
	   * @param companyId
	   * @param orgId
	   * @param siteId
	   * @param pageNo
	   * @param pageSize
	   * @return
	   */
	  @GetMapping("/statisticsCourse/account/group/export")
	  public Page<ReportStudyCourseAccountVo> courseAccountGroupExport(
			  @RequestParam(value="accountKwd",required=false)String accountKwd,
			  @RequestParam(value="orgKwd",required=false)String orgKwd,
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam("companyId")Long companyId,
			  @RequestParam("orgId")Long orgId,
			  @RequestParam("siteId")Long siteId,
			  @RequestParam(value="pageNo",required=false)Integer pageNo,
			  @RequestParam(value="pageSize",required=false)Integer pageSize);
	  
	  
	  
	  
	  
	  @GetMapping("/statisticsCourse/account/details")
	  public List<ReportStudyCourseAccountDetailsVo> courseAccountDetails(
			  @RequestParam(value="accountKwd",required=false)String accountKwd,
			  @RequestParam(value="orgKwd",required=false)String orgKwd,
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam("companyId")Long companyId,
			  @RequestParam("orgId")Long orgId,
			  @RequestParam("siteId")Long siteId,
			  @RequestParam("pageNo")Integer pageNo,
			  @RequestParam("pageSize")Integer pageSize);
	  
	  @GetMapping("/statisticsCourse/account/details/dataTotal")
	  public Integer courseAccountDetailsDataTotal(
			  @RequestParam(value="accountKwd",required=false)String accountKwd,
			  @RequestParam(value="orgKwd",required=false)String orgKwd,
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam("companyId")Long companyId,
			  @RequestParam("orgId")Long orgId,
			  @RequestParam("siteId")Long siteId);
	  
	  
	  
	  
	  
	  
	  /**
	   * 按照用户统计----详情
	   * @param accountId
	   * @param startDate
	   * @param endDate
	   * @param pageNo
	   * @param pageSize
	   * @return
	   */
	  @GetMapping("/statisticsCourse/account/group/view")
	  public  Map<String,Object> courseAccountGroupView(
			  @RequestParam("accountId")Long accountId,
			  @RequestParam(value="kwd",required=false)String kwd,
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam(value="pageNo",required=false)Integer pageNo,
			  @RequestParam(value="pageSize",required=false)Integer pageSize);
	  
	  
	  
	  
	  
	  
	  /**
	   * 按照部门统计
	   * @param kwd
	   * @param startDate
	   * @param endDate
	   * @param companyId
	   * @param orgId
	   * @param siteId
	   * @return
	   */
	  @GetMapping("/statisticsCourse/org/group")
	  public Page<ReportStudyCourseOrgVo> courseOrgGroup(
			  @RequestParam(value="kwd",required=false)String kwd,
			  @RequestParam("startDate")String startDate,
			  @RequestParam("endDate")String endDate,
			  @RequestParam("companyId")Long companyId,
			  @RequestParam("orgId")Long orgId,
			  @RequestParam("siteId")Long siteId,
			  @RequestParam(value="pageNo",required=false)Integer pageNo,
			  @RequestParam(value="pageSize",required=false)Integer pageSize);
	  
	  
	  
	  /**
	   * 异步加载报表
	   * @param startDate
	   * @return
	   */
	  @GetMapping("/statisticsCourse/asynchronous/load/date")
	  public String AsynchronousCourse(@RequestParam(value="startDate",required=false) String startDate,
			                           @RequestParam(value="endDate",required=false) String endDate);
	  
	  
	  
	  
	  
	  
	  /**
	   * 报表服务专用接口
	   */
	  @GetMapping("/statisticsCourse/get/statis/course/message")
	  public StatisCourse getStatisCourseMessage(@RequestParam(value="companyIdParam",required=false)Long companyIdParam,@RequestParam(value="siteIdParam",required=false)Long siteIdParam);
	  
	  
	  @GetMapping("/statisticsCourse/get/all/siteId")
	  public List<Long> getAllSiteId();
	  
	  
	  @GetMapping("/statisticsCourse/get/statis/course/relationId/by/courseId")
	  public CourseAccountReport listCourseAccountByCourseId(@RequestParam(value="id")Long id);
	  
	  @GetMapping("/statisticsCourse/get/statis/course/materialtime/by/courseId")
	  public String getCourseDuration(@RequestParam(value="cId")Long cId);
	                          
	  @GetMapping("/statisticsCourse/get/statis/course/recorde/by/courseId")
	  public List<StatisticsCourseRecorde> getCourseRecordeByCourseId(@RequestParam(value="cId")Long cId,@RequestParam("startDate")String startDate,@RequestParam("endDate")String endDate,@RequestParam("type")Integer type);

	  
	  @GetMapping("/statisticsCourse/get/statis/course/message/by/courseId")
	  public  List<StatisticsCourseMessage> getCourseMessageByCourseId(@RequestParam("cId")Long cId,@RequestParam("startDate")String startDate,@RequestParam("endDate")String endDate,@RequestParam("siteId")Long siteId);

	  @GetMapping("/statisticsCourse/get/statis/course/mintime/by/courseId")
	  public Date getCourseMinTimeByCourseId(@RequestParam(value="id")Long id);
	  
	  @GetMapping("/statisticsCourse/get/statis/course/grade")
	  public List<ScormRecordVO> getCourseGrade();
}
