package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.ProjectsDurationParamVO;
import com.yizhi.course.application.vo.StudyHourReportVO;
import com.yizhi.course.application.vo.domain.RecordeEntityVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 13:41 2018/11/14
 * @Modified By
 */
@FeignClient(name = "course", contextId = "CourseStudyClient")
public interface CourseStudyClient {

    @PostMapping("/study/status")
    Boolean completionStatus(@RequestBody RecordeEntityVo studyRecd);

    @GetMapping("/study/point/init")
    Boolean coursePointInit(@RequestParam(value = "siteId", required = false) Long siteId);

    /**
     * 获取学时统计列表
     *
     * @param userName
     * @param orgNameLike
     * @param pageNo
     * @param pageSize
     * @return
     * @author ding
     */
    @GetMapping("/study/report/list")
    Page<StudyHourReportVO> getStudyHourList(@RequestParam(value = "userName", required = false) String userName,
                                             @RequestParam(value = "orgNameLike", required = false) String orgNameLike,
                                             @RequestParam("pageNo") Integer pageNo,
                                             @RequestParam("pageSize") Integer pageSize);

    /**
     * 获取学时统计列表下载
     *
     * @param userName
     * @param orgNameLike
     * @return
     * @author ding
     */
    @GetMapping("/study/report/listDownload")
    String getStudyHourListDownload(@RequestParam(value = "userName", required = false) String userName,
                                    @RequestParam(value = "orgNameLike", required = false) String orgNameLike);

    /**
     * 学时明细下载
     *
     * @param userName
     * @param orgNameLike
     * @return
     */
    @GetMapping("/study/details/download")
    public String studyHourDetailsDownload(
            @RequestParam(value = "userName", required = false) String userName,
            @RequestParam(value = "orgNameLike", required = false) String orgNameLike);

    /**
     * 复旦mini mba项目获取多个培训项目总学习时长
     *
     * @param paramVO
     * @return
     */
    @PostMapping("/study/projects/totalDuration")
    public Integer getProjectsTotalDuration(@RequestBody ProjectsDurationParamVO paramVO);

    /**
     * 复旦mini mba项目获取多个个课程的学习总时长
     *
     * @param paramVO
     * @return
     */
    @PostMapping("/study/course/totalDuration")
    Map<Long, Map<String, Object>> getCourseTotalDuration(@RequestBody ProjectsDurationParamVO paramVO);

    @GetMapping("/study/course/mq/record/open")
    Boolean openRedis(@RequestParam("key") String key);
}
