package com.yizhi.course.application.feign;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.domain.CourseEntityVo;
import com.yizhi.course.application.vo.*;
import com.yizhi.util.application.page.PageInfo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * <p>
 * 营销课程
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@FeignClient(name = "course", contextId = "MarketingCourseClient")
public interface MarketingCourseClient {
    @GetMapping("/marketing/course/extis/chapte")
    boolean extisChapte(@RequestParam("courseId") Long courseId);
    /*************************  营销课程接口 *********************************/
    @PostMapping("/marketing/course/save")
    public MarketingCourseVo save(@RequestBody MarketingCourseVo marketingCourseVo);

    @PostMapping("/marketing/course/update")
    public boolean update(@RequestBody MarketingCourseVo marketingCourseVo);

    @GetMapping("/marketing/course/delete")
    public boolean delete(@RequestParam(value = "courseIds") String courseIds);

    @GetMapping("/marketing/course/up")
    public Boolean up(@RequestParam(value = "id") Long id);

    @GetMapping("/marketing/course/down")
    public Boolean down(@RequestParam(value = "id") Long id);

    @PostMapping("/marketing/course/list")
    public Page<CourseEntityVo> list(@RequestBody QueryCourseRequest queryCourseRequest);

    @GetMapping("/marketing/course/view")
    public MarketingCourseVo view(@RequestParam(value = "courseId") Long courseId,
                                  @RequestParam(required = false,value = "shareId") Long shareId,
                                  @RequestParam(required = false,value = "accountId") Long accountId,
                                  @RequestParam(required = false,value = "openId") String openId,
                                  @RequestParam(required = false,value = "manage") Boolean manage,
                                  @RequestParam(required = false,value = "childShareOpenId") String childShareOpenId);

    @PostMapping("/marketing/course/share")
    public CourseShareVo share(@RequestBody CourseShareVo vo);

    @GetMapping("/marketing/course/activity")
    public boolean activity(@RequestParam(value = "browseRecordId") Long browseRecordId );

    @GetMapping("/marketing/course/name/get")
    public MarketingCourseVo getByName(@RequestParam("courseName") String courseName,@RequestParam("siteId") Long siteId,
                                       @RequestParam(value = "courseId",required = false) Long courseId);

    @GetMapping("/marketing/course/code/get")
    MarketingCourseVo existByCode(@RequestParam("courseCode") String courseCode,@RequestParam("siteId") Long siteId,
                                         @RequestParam(value = "courseId",required = false) Long courseId);

    @GetMapping("/marketing/course/classify/id/get")
    public Integer getByClassifyId(@RequestParam(value = "classifyId") Long classifyId);

    @GetMapping("/marketing/course/share/list")
    public PageInfo<CourseShareVo> shareList(@RequestParam(value = "pageNo") int pageNo
            , @RequestParam(value = "pageSize" ,required = false) Integer pageSize
            , @RequestParam(value = "searchFiled" ,required = false) String searchFiled);

    @GetMapping("/marketing/course/share/browse/records")
    public Page<BrowseRecordsVo> shareBrowseRecords(@RequestParam(value = "courseId") Long courseId
            , @RequestParam(value = "pageNo") int pageNo
            , @RequestParam(value = "pageSize" ,required = false) Integer pageSize
            , @RequestParam(value = "searchFiled" ,required = false) String searchFiled);

    @GetMapping("/marketing/course/share/count")
    public int shareCount();

    @GetMapping("/marketing/course/name/find")
    public List<CourseVo> courseFindByName(@RequestParam(value = "name") String name,
                                           @RequestParam(value = "companyId") Long companyId,
                                           @RequestParam(value = "siteId") Long siteId,
                                           @RequestParam(value = "orgId") Long orgId);

    @GetMapping("/marketing/course/report/view")
    public Page<ReportMarketingCourseDataChartsVo> chartGroup(
            @RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,
            @RequestParam(name = "type", required = true, defaultValue = "1") Integer type,//value = "类型：1按日 2按周 3按月（默认按日）
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo);


    @GetMapping("/marketing/course/report/course/group")
    public Page<ReportMarketingCourseVo> marketingCourseGroup(
            @RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,
            @RequestParam(name = "kwd", required = false) String kwd,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo);

    @GetMapping("/marketing/course/report/course/group/view/list")
    Page<MarketingCourseViewVo> marketingCourseViewList(
            @RequestParam(name = "courseId") Long courseId,
            @RequestParam(name = "nikeName",required = false) String nikeName,
            @RequestParam(name = "pageSize", required = false) Integer pageSize,
            @RequestParam(name = "pageNo", required = false) Integer pageNo);

    @GetMapping("/marketing/course/report/account/group")
    public Page<ReportMarketingCourseAccountVo> marketingCourseAccountGroup(
            @RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,
            @RequestParam(name = "orgKwd", required = false) String orgKwd,
            @RequestParam(name = "accountKwd", required = false) String accountKwd,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo);

    @PostMapping("/marketing/course/report/account/group/export")
    public Page<ReportMarketingCourseAccountVo> marketingCourseAccountGroupExp(
            @RequestBody MkCourseExportRepVO vo);

    @PostMapping("/marketing/course/report/course/group/export")
    public Page<ReportMarketingCourseVo> marketingCourseGroupExp(
            @RequestBody MkCourseExportRepVO vo);


    @GetMapping("/marketing/course/share/spread/list")
    PageInfo<CourseShareVo> shareSpreadList(
            @RequestParam(value = "courseId" ) Long courseId,
            @RequestParam(value = "pageNo") int pageNo,
            @RequestParam(value = "pageSize" ,required = false) Integer pageSize,
            @RequestParam(value = "firstShareId" ,required = false) String firstShareId);

    @GetMapping("/marketing/course/wechat/openid/get")
    WechatInfoVO wachatOpenIdGet(@RequestParam("openId") String openId
    );


    @GetMapping("/marketing/course/sms/pre/check")
    Integer smsSendPreCheck(@RequestParam("openId") String openId, @RequestParam("tel") String tel);

    @GetMapping("/marketing/course/sms/captcha/save")
    Boolean saveCaptcha(@RequestParam("openId") String openId,
                                   @RequestParam("tel") String tel,
                                   @RequestParam("captcha") String captcha);

    @GetMapping("/marketing/course/sms/captcha/validate")
    Integer smsCaptchaValidate(@RequestParam("openId") String openId,
                               @RequestParam("tel") String tel,
                               @RequestParam("captcha") String captcha);
}

