package com.yizhi.course.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.vo.MyCourseRecordeVo;
import com.yizhi.course.application.vo.RecordeDurationVO;
import com.yizhi.course.application.vo.RecordeListVo;
import com.yizhi.course.application.vo.StudyDurationVO;
import com.yizhi.course.application.vo.domain.ChapterEntityVo;
import com.yizhi.course.application.vo.domain.CourseEntityVo;
import com.yizhi.course.application.vo.domain.MaterialEntityVo;
import com.yizhi.course.application.vo.domain.RecordeEntityVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Set;


@FeignClient(name = "course", contextId = "RecordeClient")
public interface RecordeClient {

    @PostMapping("/recorde/save")
    Boolean save(@RequestBody RecordeEntityVo recorde);

    @PostMapping("/recorde/completed/save")
    public Boolean completedSave(@RequestBody RecordeEntityVo recorde);

    @PostMapping("/recorde/last/study/save")
    Boolean lastStudySave(@RequestBody RecordeEntityVo recorde);

    @PostMapping("/recorde/update")
    Boolean update(@RequestBody RecordeEntityVo recorde);

    @PostMapping("/recorde/update/list")
    Boolean updateList(@RequestBody List<RecordeEntityVo> recordes);

    @GetMapping("/recorde/get")
    RecordeEntityVo get(
            @RequestParam("accountId") Long accountId,
            @RequestParam("chapterId") Long chapterId,
            @RequestParam("sessionId") String sessionId
    );

    @PostMapping("/recorde/list/save")
    List<RecordeEntityVo> saveList(@RequestBody List<RecordeEntityVo> recordes);

    @PostMapping("/recorde/list/update")
    public boolean batchUpdate(@RequestBody List<RecordeEntityVo> recordes);

    /**
     * 删除学习记录list
     *
     * @param recordes
     * @return
     */
    @PostMapping("/list/delete")
    Boolean deleteList(@RequestBody List<RecordeEntityVo> recordes);

    @GetMapping("/recorde/get/completed")
    Integer getCompleted(
            @RequestParam("chapterId") Long chapterId,
            @RequestParam("accountId") Long accountId,
            @RequestParam("duration") Integer duration
    );


    @GetMapping("/recorde/list")
    List<RecordeListVo> getMyRecorde(
            @RequestParam(name = "courseId") Long courseId,
            @RequestParam(name = "accountId") Long accountId
    );

    @GetMapping("/recorde/my/list")
    Page<MyCourseRecordeVo> getMyCourseRecordeList(
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "accountId") Long accountId,
            @RequestParam(name = "type", required = false) Integer type,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize
    );

    /**
     * 获得我的课程 进行中或完成的数量
     *
     * @param accountId
     * @param type      课程是否完成 0 为进行中 1 为已完成
     * @return
     */
    @GetMapping("/recorde/my/list/count")
    Integer getMyCourseRecordeCount(
            @RequestParam(name = "accountId") Long accountId,
            @RequestParam(name = "type", required = false) Integer type
    );

    /**
     * 查询所有课程
     *
     * @return
     */
    @GetMapping("/recorde/course/list")
    List<CourseEntityVo> getCourseList();

    @PostMapping("/recorde/exist/course/list")
    List<CourseEntityVo> selectExistCourse(@RequestBody Set<Long> courseIds);

    /**
     * 查询所有章节
     *
     * @return
     */
    @GetMapping("/recorde/chapter/list")
    List<ChapterEntityVo> getChapterList();

    @PostMapping("/recorde/exist/chapter/list")
    List<ChapterEntityVo> selectExistChapter(@RequestBody List<Long> chapterIds);

    /**
     * 查询所有素材
     *
     * @return
     */
    @GetMapping("/recorde/material/list")
    List<MaterialEntityVo> getMaterialList();

    @PostMapping("/recorde/exist/material/list")
    List<MaterialEntityVo> selectExistMaterial(@RequestBody List<Long> materialIds);

    /**
     * 查询所有学习记录
     *
     * @return
     */
    @GetMapping("/recorde/all/list")
    List<RecordeEntityVo> getAllRecordeList();

    @PostMapping("/recorde/exist/recorde/list")
    List<RecordeEntityVo> getExistRecorde(@RequestBody List<String> sessionIds);


    /**
     * 查询所有的学员每个章节的总学习时长
     *
     * @return
     */
    @GetMapping("/recorde/duration/list")
    List<RecordeDurationVO> getRecordeDuration();

    /**
     * 查询课程是否完成
     *
     * @param chapterId
     * @return
     */
    @GetMapping("recorde/course/isFinish")
    Long getCourseFinished(@RequestParam("accountId") Long accountId, @RequestParam("chapterId") Long chapterId);

    /**
     * 查询课程完成的学员
     *
     * @param courseId
     *
     * @return
     */
    @GetMapping("recorde/course/finishAccountIds")
    List<Long> getFinishedAccountIds(@RequestParam("courseId") Long courseId);

    /**
     * 专为培训统计时长所用
     * @param courseId
     * @return
     */
    @GetMapping("recorde/course/getStudyTime")
    Integer getStudyTime(@RequestParam(value = "courseId",required = false) List<Long> courseId);

    /**
     * 课程章节完成的数量
     * @param accountId
     * @param chapterId
     * @return
     */
    @GetMapping("/recorde/course/chapter/isFinish")
    public int getCourseChapterFinished(@RequestParam("accountId") Long accountId,
                                        @RequestParam("chapterId") Long chapterId);

    /**
     * 统计学习时长。chapterId不是必须。如果chapterId指定，回统计该用户对于章节的学习时长
     * @param accountId
     * @param SiteId
     * @param chapterId
     * @return
     */
    @GetMapping("/recorde/count/study/duration")
    Long countStudyDuration(@RequestParam("accountId") Long accountId,
                                   @RequestParam("SiteId") Long SiteId,
                                   @RequestParam(value = "chapterId" ,required = false) Long chapterId);

    /**
     * 按课程统计学习总时长
     * @param accountId
     * @param SiteId
     * @param courseIds
     * @return
     */
    @GetMapping("/recorde/count/study/duration/cids")
    Long countStudyDurationByIds(@RequestParam("accountId") Long accountId,
                                        @RequestParam("SiteId") Long SiteId,
                                        @RequestParam(value = "courseIds") List<Long> courseIds);
    /**
     * 平台报告统计
     * @param accountId
     * @param siteIds
     * @param startDate
     * @param endDate
     * @param accounts
     * @return
     */
    @GetMapping("/recorde/studyDuration")
    StudyDurationVO studyDuration(@RequestParam("accountId") Long accountId, @RequestParam("siteIds") List<Long> siteIds,
                                  @RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate , @RequestParam("accounts") List<Long> accounts);
}
