package com.yizhi.course.application.controller;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.service.IAccreditService;
import com.yizhi.course.application.vo.AccreditCourseVO;
import com.yizhi.core.application.vo.AccreditVO;
import com.yizhi.course.application.vo.AuthorizeCourseVO;
import com.yizhi.course.application.vo.PageAuthorizeVO;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 课程资源授权管理 前端控制器
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Api(tags = "课程授权")
@RestController
@RequestMapping("/accredit")
public class AccreditController {

    @Autowired
    private IAccreditService iAccreditService;


    @Autowired
    IdGenerator idGenerator;


    /**
     * 根据课程关键字（课程名称 name或者标签tags）、授权站点id,站点id 查询未选择课程（分页并显示总条数)
     */
    @GetMapping("/course/noCourseList")
    public Page<Course> noCourseList(@RequestBody AccreditCourseVO accreditCourseVO) {

        System.out.println(accreditCourseVO.getCompanyId() + "~~~~~~~~~~~~~~~~~~~~~~");
        Page<Course> page = iAccreditService.noCourseList(accreditCourseVO.getName(), accreditCourseVO.getCompanyId(),
                accreditCourseVO.getSiteId(), accreditCourseVO.getPageNo(), accreditCourseVO.getPageSize());

        return page;
    }


    /**
     * 根据课程关键字（课程名称 name或者标签tags）授权站点id、 站点id 查询已选择课程（分页并显示总条数)
     */
    @GetMapping("/courseList")
    public Page<Course> courseList(@RequestBody AccreditCourseVO accreditCourseVO) {
        System.out.println(accreditCourseVO.getCompanyId() + "~~~~~~~~~~~~~~~~~~~~~~");
        Page<Course> page = iAccreditService.courseList(accreditCourseVO.getName(), accreditCourseVO.getCompanyId(),
                accreditCourseVO.getSiteId(), accreditCourseVO.getPageNo(), accreditCourseVO.getPageSize());

        return page;
    }


    /**
     * 根据课程关键字（课程名称 name或者标签tags）授权站点id、 站点id 查询已选择课程（不用分页）
     * 作用：供 保存、 授权接口调用  与传进来的已选课程对象集合相比较
     */
    @GetMapping("/XYcourseList")
    public List<Course> YXcourseList(@RequestBody AccreditCourseVO accreditCourseVO) {

        List<Course> CourseList = iAccreditService.YXcourseList(accreditCourseVO.getName(), accreditCourseVO.getCompanyId(), accreditCourseVO.getSiteId());
        return CourseList;
    }


    /**
     * 获取授权列表
     *
     * @param siteName
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/get/list")
    public PageAuthorizeVO queryAuthorizeCourse(@RequestParam(name = "type", required = true) String type,
                                                @RequestParam(name = "siteName", required = false) String siteName,
                                                @RequestParam(name = "companyName", required = false) String companyName,
                                                @RequestParam(name = "pageNo", defaultValue = "0") Integer pageNo,
                                                @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize) {

        return iAccreditService.queryAuthorizeCourse(type, siteName, companyName, pageNo, pageSize);
    }

    @GetMapping(value = "/authorize/list")
    public Page<AuthorizeCourseVO> queryAuthorizedCourseList(@RequestParam(name = "type", required = true) String type,
                                                             @RequestParam(name = "siteName", required = false) String siteName,
                                                             @RequestParam(name = "companyName", required = false) String companyName,
                                                             @RequestParam(name = "pageNo", defaultValue = "0") Integer pageNo,
                                                             @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize) {

        return iAccreditService.queryAuthorizedCourseList(type, siteName, companyName, pageNo, pageSize);
    }

    /**
     * 授权编辑
     *
     * @param accreditVO
     * @return
     */
    @PostMapping(value = "/update")
    public Boolean updateAuthorize(@RequestBody AccreditVO accreditVO) {
        return iAccreditService.updateAuthorize(accreditVO);
    }

    /**
     * 此授权课程当前授权有效课程
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/courseCount")
    public Integer getAccreditByCourseId(@RequestParam(value = "id") Long id) {
        return iAccreditService.getAccreditByCourseId(id);
    }
}
