package com.yizhi.course.application.controller;


import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.course.application.domain.CardConfig;
import com.yizhi.course.application.service.ICardConfigService;
import com.yizhi.course.application.vo.CardConfigVO;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.system.remote.StudentEditPermissionClient;
import com.yizhi.system.application.vo.SiteVO;
import com.yizhi.system.application.vo.StudentEditPermissionVO;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 营销课程名片配置 前端控制器
 * </p>
 *
 * @author 谢海军123
 * @since 2019-03-07
 */
@RestController
@RequestMapping("/cardConfig")
public class CardConfigController {

    private Logger logger = LoggerFactory.getLogger(CardConfigController.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ICardConfigService cardConfigService;

    @Autowired
    private StudentEditPermissionClient permissionClient;

    @Autowired
    private SiteClient siteClient;

    @GetMapping(value = "/init")
    public boolean initConfig(@RequestParam("siteId") Long siteId){
        try {
            List<SiteVO> siteVOS =  siteClient.findByCompanyId(null);
            List<CardConfig> data = cardConfigService.selectList(QueryUtil.condition(new CardConfig()));
            List<Long> siteids = data.stream().parallel().map(obj->obj.getSiteId()).collect(Collectors.toList());
            boolean isExtis = false;
            if(CollectionUtils.isNotEmpty(siteids)){
                isExtis = true;
            }
            List<CardConfig> insertData = new ArrayList<>();
            for(SiteVO item : siteVOS){
                Long itemSiteId = item.getId();
                if((isExtis && siteids.contains(itemSiteId)) || itemSiteId==null){
                    continue;
                }
                CardConfig insertItem = new CardConfig();
                insertItem.setId(idGenerator.generate());
                insertItem.setSiteId(itemSiteId);
                insertItem.setCompanyId(item.getCompanyId());

                insertItem.setCardTopDisplay(true);
                insertItem.setTopBarText("联系我了解更多");
                insertItem.setTopBarButton(0);
                insertItem.setTopBarBgcolor("#666666");

                insertItem.setCardBodyDisplay(true);
                insertItem.setBodyField1(true);
                insertItem.setBodyField2(1);
                insertItem.setBodyField3(2);
                insertItem.setBodyField4(3);
                insertItem.setBodyField5(9);
                insertItem.setBodyBgimgRadio(0);
                insertItem.setBodyBgimg("https://cloud-wmy.oss-cn-shanghai.aliyuncs.com/wmy/grop.jpg");

                insertItem.setCardFootDisplay(true);

                StudentEditPermissionVO savePermission = new StudentEditPermissionVO();
                savePermission.setSiteId(itemSiteId);
                savePermission.setCompanyId(item.getCompanyId());
                savePermission.setEmailEditPermission(true);
                savePermission.setMobileEditPermission(true);
                savePermission.setNameEditPermission(true);
                savePermission.setSynopsisEditPermission(true);
                savePermission.setPositionEditPermission(true);
                permissionClient.saveOrUpdate(savePermission);

                insertData.add(insertItem);
            }

            if(CollectionUtils.isNotEmpty(insertData)){
                cardConfigService.insertBatch(insertData);
            }

            return true;
        }catch (Exception e){
            return false;
        }

    }
    /**
     * 新增或者修改名片配置信息
     * @param vo
     * @return
     */
    @PostMapping(value = "/saveOrUpdate")
    public CardConfigVO addConfig(@RequestBody CardConfigVO vo){
        Long id = vo.getId();
        if(Objects.nonNull(id)){
            CardConfig cardConfigUp = new CardConfig();
            cardConfigUp.setUpdateTime(new Date());
            BeanUtils.copyProperties(vo,cardConfigUp);
            cardConfigService.updateById(cardConfigUp);
        }else{// 新增
            CardConfig cardConfigAdd = new CardConfig();
            vo.setCreateTime(new Date());
            vo.setId(idGenerator.generate());
            BeanUtils.copyProperties(vo,cardConfigAdd);
            cardConfigService.insert(cardConfigAdd);
        }
        return vo;
    }

    /**
     * 获取站点的名片配置信息
     * @param siteId
     * @return
     */
    @GetMapping(value = "/get")
    public CardConfigVO getBySiteId(@RequestParam("siteId") Long siteId){
        CardConfig cardConfig = new CardConfig();
        cardConfig.setSiteId(siteId);
        CardConfig old = cardConfigService.selectOne(QueryUtil.condition(cardConfig));
        CardConfigVO retData = null;
        if(old != null){
            retData = new CardConfigVO();
            BeanUtils.copyProperties(old,retData);
        }
        return retData;
    }
}

