package com.yizhi.course.application.controller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.service.IClassifyService;
import com.yizhi.course.application.vo.ApiClassifyVo;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 课程分类 前端控制器
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@RestController
@RequestMapping("/manage/classify")
public class ClassifyController {
    @Autowired
    IdGenerator idGenerator;
    private Logger logger = LoggerFactory.getLogger(ClassifyController.class);
    @Autowired
    private IClassifyService iClassifyService;

    @GetMapping("/find/id")
    public Classify getById(@RequestParam("id") Long id){

        return iClassifyService.selectById(id);
    }

    @ApiOperation("添加分类")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public int saveClassify(
            @RequestBody Classify classify
    ) {
        return iClassifyService.saveClassify(classify);
    }

    /**
     * 检查是否存在分类名称
     * @param name 分类名称
     * @param parentId 分类的上级id（一级分类为0）
     * @param siteId 站点id
     * @param type 检查类型（1：新增，2：修改(需要传classifyId)）
     * @param classifyId 分类id，修改检查时用到
     * @return 不存在返回true，存放相同名返回false
     */
    @GetMapping("/check/name")
    public boolean checkExtisClassifyName(@RequestParam("name") String name,
                                          @RequestParam(value = "parentId" ,required = false) Long parentId,
                                          @RequestParam("siteId") Long siteId,
                                          @RequestParam("type") Integer type,
                                          @RequestParam(value = "classifyId" ,required = false) Long classifyId){
        return  iClassifyService.checkExtisClassifyName(name,parentId,siteId,type,classifyId);
    }

    @ApiOperation("分类删除")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public int deleteClassify(
            @RequestBody Classify classify
    ) {
        return iClassifyService.deleteClassify(classify.getId());
    }

    @ApiOperation("分类名称修改")
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public int updateClassify(
            @RequestBody Classify classify
    ) {
        return iClassifyService.updateClassify(classify);
    }

    @ApiOperation("分类查询")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Page<Classify> listClassify(
            @RequestParam(required = false,name = "isStartHQ") Boolean isStartHQ,
            @RequestParam(required = false) String name,
            @RequestParam Integer pageNo,
            @RequestParam Integer pageSize,
            @RequestParam Long companyId,
            @RequestParam Long siteId,
            @RequestParam(name = "orgIds", required = false) List<Long> orgIds
    ) {
        return iClassifyService.listClassify(name, pageNo, pageSize, companyId, orgIds, siteId, isStartHQ);
    }

    @ApiOperation(value = "分类导入")
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    public Response<Integer> importClassify(
            @ApiParam(required = true, value = "分类导入url") @RequestParam(value = "url", required = true) String url
    ) {
        return Response.ok(1);
    }

    @ApiOperation("分类排序")
    @RequestMapping(value = "/sort", method = RequestMethod.POST)
    public int sortClassify(
            @RequestBody List<Classify> list
    ) {
        logger.info("Course端传入要排序的ID列表==========================:"+list);
        return iClassifyService.updateClassiyfsort(list);
    }


    @ApiOperation("分类分页查询")
    @RequestMapping(value = "/page/list", method = RequestMethod.GET)
    public Page<ApiClassifyVo> pageList(
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "orgIds", required = false) List<Long> orgIds
    ) {
        logger.info("分类分页查询orgIds=", orgIds==null?0:orgIds.size());
        Page<ApiClassifyVo> page = iClassifyService.selectListByCompanyId(companyId, name, orgIds, siteId, pageNo, pageSize);

        List<Classify> classifies2 = iClassifyService.selectClassifyByLayer(companyId,siteId,2);

        List<Classify> classifies3 = iClassifyService.selectClassifyByLayer(companyId,siteId,3);
        Map<Long, List<ApiClassifyVo>> map3 = new HashMap<Long, List<ApiClassifyVo>>();
        for (Classify c3 : classifies3) {
            if (map3.containsKey(c3.getParentId())) {
                List<ApiClassifyVo> apiClassifyVos = map3.get(c3.getParentId());
                ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
                apiClassifyVo = copyBean(apiClassifyVo, c3);
                apiClassifyVos.add(apiClassifyVo);
            } else {
                List<ApiClassifyVo> apiClassifyVos = new ArrayList<ApiClassifyVo>();
                ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
                apiClassifyVo = copyBean(apiClassifyVo, c3);
                apiClassifyVos.add(apiClassifyVo);
                map3.put(c3.getParentId(), apiClassifyVos);
            }
        }

        Map<Long, List<ApiClassifyVo>> map2 = new HashMap<Long, List<ApiClassifyVo>>();
        for (Classify c2 : classifies2) {
            if (map2.containsKey(c2.getParentId())) {
                List<ApiClassifyVo> apiClassifyVos = map2.get(c2.getParentId());
                ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
                apiClassifyVo = copyBean(apiClassifyVo, c2);
                apiClassifyVo.setChildren(map3.get(c2.getId()));
                apiClassifyVos.add(apiClassifyVo);
            } else {
                List<ApiClassifyVo> apiClassifyVos = new ArrayList<ApiClassifyVo>();
                ApiClassifyVo apiClassifyVo = new ApiClassifyVo();
                apiClassifyVo = copyBean(apiClassifyVo, c2);
                apiClassifyVo.setChildren(map3.get(c2.getId()));
                apiClassifyVos.add(apiClassifyVo);
                map2.put(c2.getParentId(), apiClassifyVos);
            }
        }
        List<ApiClassifyVo> classifies = page.getRecords();
        int size = classifies.size();
        ApiClassifyVo c1 = null;


        for(int i = 0; i < size; i++){
        	c1 = classifies.get(i);
        	c1.setChildren(map2.get(c1.getId()));
            logger.info("这是 分页 接口 返回的 page 里的 sort值为：====="+c1.getName()+"======================="+c1.getSort());
        	//设置上一个分类主键ID
        	if(i > 0){
        		c1.setMoveUpId(classifies.get(i - 1).getId());
        	}
        	//设置下一个分类主键ID
        	if(i < size - 1){
        		c1.setMoveDownId(classifies.get(i + 1).getId());
        	}
        }
        
        //设置上一个和下一个ID
        Page<ApiClassifyVo> pageTow = null;
        if(pageNo > 1){
        	pageTow = iClassifyService.selectListByCompanyId(companyId, name, orgIds, siteId, pageNo - 1, pageSize);
        	classifies.get(0).setMoveUpId(pageTow.getRecords().get(pageTow.getRecords().size() - 1).getId());
        }
        pageTow = iClassifyService.selectListByCompanyId(companyId, name, orgIds, siteId, pageNo + 1, pageSize);
        if(pageTow != null && pageTow.getRecords().size() > 0){
        	classifies.get(classifies.size() - 1).setMoveDownId(pageTow.getRecords().get(0).getId());
        }
        return page;
    }


    private ApiClassifyVo copyBean(ApiClassifyVo apiClassifyVo, Classify classify) {
        apiClassifyVo.setCode(classify.getCode());
        apiClassifyVo.setCompanyId(classify.getCompanyId());
        apiClassifyVo.setCreateById(classify.getCreateById());
        apiClassifyVo.setCreateByName(classify.getCreateByName());
        apiClassifyVo.setCreateTime(classify.getCreateTime());
        apiClassifyVo.setDescription(classify.getDescription());
        apiClassifyVo.setId(classify.getId());
        apiClassifyVo.setLayer(classify.getLayer());
        apiClassifyVo.setName(classify.getName());
        apiClassifyVo.setOrgId(classify.getOrgId());
        apiClassifyVo.setParentId(classify.getParentId());
//        apiClassifyVo.setParentId(classify.getParentId());
        apiClassifyVo.setPath(classify.getPath());
        apiClassifyVo.setRootId(classify.getRootId());
        apiClassifyVo.setSiteId(classify.getSiteId());
        apiClassifyVo.setSort(classify.getSort());
        apiClassifyVo.setSource(classify.getSource());
        apiClassifyVo.setStatus(classify.getStatus());
        apiClassifyVo.setUpdateById(classify.getUpdateById());
        apiClassifyVo.setUpdateByName(classify.getUpdateByName());
        apiClassifyVo.setUpdateTime(classify.getUpdateTime());
        return apiClassifyVo;
    }

    /**
     * 验证同站点下 同级的分类不能重名
     *
     * @param name      分类名
     * @param layer     分类层级
     * @param companyId 企业名
     * @param siteId    站点id
     * @param id        分类id （修改的时候传入）
     * @return Boolean
     */
    @GetMapping("/vail/name")
    public Boolean vailName(
            @RequestParam(name = "name") String name,
            @RequestParam(name = "layer") Integer layer,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "id",required = false) Long id

    ) {
        Classify classify = new Classify();
        classify.setName(name);
        classify.setLayer(layer);
        classify.setCompanyId(companyId);
        classify.setSiteId(siteId);
        EntityWrapper<Classify> entityWrapper = new EntityWrapper<>(classify);
        if (null != id) {
            entityWrapper.ne("id", id);
        }
        List list = iClassifyService.selectList(entityWrapper);
        if (null != list && list.size() > 0) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @GetMapping("/getClassifyName")
    public String getClassifyName(@RequestParam("classifyId")Long classifyId) {
        //课程分类回写
    	String classifyName = "";
        Classify classify = iClassifyService.selectById(classifyId);
        if (classify != null && !StringUtils.isEmpty(classify.getPath())) {

            Set<Long> idsList = new HashSet<>();
            if (classify.getRootId() != 0) {
                idsList.add(classify.getRootId());
            }
            if (classify.getParentId() != 0) {
                idsList.add(classify.getParentId());
            }
            idsList.add(classify.getId());
            List<Classify> classifies = iClassifyService.selectClassifyByPath(new ArrayList(idsList));
            for (Classify classify1 : classifies) {
                if ("".equals(classifyName.toString())) {
                    classifyName+=classify1.getId();
                } else {
                    classifyName+=",";
                    classifyName+=classify1.getId();
                }
            }
        }
        return classifyName;
    }
}

