package com.yizhi.course.application.controller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.domain.Comment;
import com.yizhi.course.application.domain.Reply;
import com.yizhi.course.application.service.ICommentService;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.service.IReplyService;
import com.yizhi.course.application.service.IThumbsUpService;
import com.yizhi.course.application.task.CommentListExportAsync;
import com.yizhi.course.application.vo.ApiCommnentVo;
import com.yizhi.course.application.vo.CourseCommentVo;
import com.yizhi.course.application.vo.ListCommentVo;
import com.yizhi.course.application.vo.PageCommentVo;
import com.yizhi.course.application.vo.domain.ClassifyEntityVo;
import com.yizhi.course.application.vo.domain.ReplyEntityVo;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountIdsByKeyReqVO;
import com.yizhi.system.application.vo.AccountVO;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiParam;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/comm")
public class CommController<E, V> {

	@Autowired
	private ICommentService commentService;

	@Autowired
	private IReplyService replyService;

	@Autowired
	private ICourseService courseService;

	@Autowired
	private IThumbsUpService thumbsUpService;

	@Autowired
	private AccountClient accountClient;

	@Autowired
	private CommentListExportAsync commentListExportAsync;

	/**
	 * 课程评论管理
	 * 
	 * @param courseId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/list")
	public PageCommentVo list(@RequestParam("courseId") Long courseId,
							  @RequestParam(name = "commentator", required = false) String commentator,
							  @RequestParam(name = "status", required = false) Integer status, @RequestParam("pageNo") Integer pageNo,
							  @RequestParam("pageSize") Integer pageSize) {
		RequestContext requestContext = ContextHolder.get();
		List<Long> accountIds = new ArrayList<Long>();
		Page<ListCommentVo> page = new Page<ListCommentVo>();
		PageCommentVo pageCommentVo = new PageCommentVo();
		if (StringUtils.isNotBlank(commentator)) {
			AccountIdsByKeyReqVO vo = new AccountIdsByKeyReqVO();
			vo.setKey(commentator);
			vo.setCompanyId(requestContext.getCompanyId());
			vo.setSiteId(requestContext.getSiteId());
			accountIds = accountClient.findAIdsByKeywordByVo(vo);
			if (accountIds==null) {
				return pageCommentVo;
			}
		}
		page = commentService.getCommentPage(courseId, pageNo, pageSize, accountIds, status);
		pageCommentVo.setPage(page);
		Double level = courseService.getAvgLevel(courseId);
		pageCommentVo.setAvgLevel(level);

//        Reply reply = new Reply();
//        reply.setCourseId(courseId);
//        reply.setDeleted(0);
//        EntityWrapper<Reply> wrapper = new EntityWrapper<Reply>(reply);
//        List<String> list = new ArrayList<String>();
//        list.add("createTime");
//        wrapper.orderDesc(list);
//        List<Reply> replies = replyService.selectList(wrapper);
//
//        Map<Long, List<Reply>> listMap = new HashMap<>();
//        for (Reply reply1 : replies) {
//            if (listMap.containsKey(reply1.getCommentId())) {
//                listMap.get(reply1.getCommentId()).add(reply1);
//            } else {
//                List<Reply> replies1 = new ArrayList<>();
//                replies1.add(reply);
//                listMap.put(reply1.getCommentId(), replies1);
//            }
//        }
		String fullName;
		String name;
		for (ListCommentVo listCommentVo : page.getRecords()) {
			AccountVO accountVO = accountClient.findById(listCommentVo.getAccountId());
			name = accountVO.getName();
			fullName = accountVO.getFullName();
			listCommentVo.setType(listCommentVo.getContent()!=null?0:1);
			listCommentVo.setCommentator(name);
			listCommentVo.setContent(StringEscapeUtils.unescapeJava(listCommentVo.getContent()));
			listCommentVo.setReplyName("--");
			listCommentVo.setCommentatorName(null == fullName||""==fullName ? name : fullName);
//            if (listMap.containsKey(listCommentVo.getId())) {
//                listCommentVo.setReplies(listMap.get(listCommentVo.getId()));
//            }

		}
		return pageCommentVo;
	}

	/**
	 * 评论下的评论回复列表
	 * 
	 * @param commmentId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/reply/list")
	public Page<Reply> replyList(@RequestParam("commmentId") Long commmentId, @RequestParam("pageNo") Integer pageNo,
								 @RequestParam("pageSize") Integer pageSize) {
		Page<Reply> page = new Page<Reply>(pageNo, pageSize);
		Reply reply = new Reply();
		reply.setCommentId(commmentId);
		reply.setDeleted(0);
		EntityWrapper<Reply> wrapper = new EntityWrapper<Reply>(reply);
		List<String> list = new ArrayList<String>();
		list.add("createTime");
		wrapper.orderDesc(list);
		replyService.selectPage(page, wrapper);
		Comment comment = commentService.selectById(commmentId);
		String userAvatar = null;
		if (comment != null) {
			userAvatar = comment.getUserAvatar();
		}
		String fullName;
		String name;
		for (Reply listCommentVo : page.getRecords()) {
			if (null!=listCommentVo.getReplyParentId()&&0!=listCommentVo.getReplyParentId()) {
				Long accountId = replyService.selectById(listCommentVo.getReplyParentId()).getAccountId();
				AccountVO findById = accountClient.findById(accountId);
				listCommentVo.setParentAccountFullName(null==findById.getFullName()||""==findById.getFullName()?findById.getName():findById.getFullName());
			}else {
				AccountVO findById = accountClient.findById(commentService.selectById(listCommentVo.getCommentId()).getAccountId());
				listCommentVo.setParentAccountFullName(null==findById.getFullName()||""==findById.getFullName()?findById.getName():findById.getFullName());
			}
			AccountVO accountVO = accountClient.findById(listCommentVo.getAccountId());
			name = accountVO.getName();
			fullName = accountVO.getFullName();
			listCommentVo.setReplier(name);
			listCommentVo.setContent(StringEscapeUtils.unescapeJava(listCommentVo.getContent()));
			listCommentVo.setUserAvatar(userAvatar);
			listCommentVo.setReplierName(null == fullName||""==fullName ? name : fullName);
		}
		return page;
	}

	/**
	 * 学员端 评论列表
	 * 
	 * @param courseId
	 * @param accountId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/api/list")
	public Page<ApiCommnentVo> apiList(@RequestParam("courseId") Long courseId,
									   @RequestParam("accountId") Long accountId, @RequestParam("pageNo") Integer pageNo,
									   @RequestParam("pageSize") Integer pageSize) {
		// 优化查询，（sql已优化，暂时去掉）
		/*
		 * RequestContext rt = ContextHolder.get(); Page<ApiCommnentVo> ret = new
		 * Page<>();
		 * 
		 * Page<Comment> page = new Page<>(pageNo,pageSize); Comment wheComment = new
		 * Comment(); wheComment.setCourseId(courseId); wheComment.setDeleted(0);
		 * EntityWrapper<Comment> wheEw = QueryUtil.condition(wheComment);
		 * wheEw.orderBy("create_time",false);//DESC Page<Comment> data =
		 * commentService.selectPage(page,wheEw); List<Comment> commentData =
		 * data.getRecords(); if(CollectionUtils.isNotEmpty(commentData)){
		 * List<ApiCommnentVo> all = new ArrayList<>();
		 * BeanUtils.copyProperties(data,ret,"records"); List<Long> commentIds =
		 * commentData.stream().map(Comment::getId).collect(Collectors.toList()); //
		 * 查询点赞数 Map<Long,ThumbsUp> thumbData =
		 * thumbsUpService.getThumbsInfo(rt.getSiteId(),commentIds,accountId); // 回复信息查询
		 * EntityWrapper<Reply> replyEw = QueryUtil.condition(new Reply());
		 * replyEw.in("comment_id",commentIds).eq("deleted",0);
		 * replyEw.orderBy("create_time"); List<Reply> replyData =
		 * replyService.selectList(replyEw); Map<Long,List<Reply>> replyMap = null;
		 * if(CollectionUtils.isNotEmpty(replyData)){ replyMap =
		 * replyData.stream().collect(Collectors.groupingBy(key->key.getCommentId())); }
		 * for(Comment itemObj:commentData){ ApiCommnentVo vo = new ApiCommnentVo();
		 * BeanUtils.copyProperties(itemObj,vo); Long ccid = itemObj.getId();
		 * if(thumbData != null && thumbData.containsKey(ccid)){ ThumbsUp up =
		 * thumbData.get(ccid); vo.setThumbsUp(up.getThumbsUpNum());
		 * vo.setStatus(up.getHasThumbsUp()?1:0); } if(replyMap != null &&
		 * replyMap.containsKey(ccid)){ List<Reply> reTemp = replyMap.get(ccid);
		 * vo.setReplys(reTemp.size()); vo.setReplies(reTemp); } }
		 * 
		 * ret.setRecords(all); } return ret;
		 */

		Page<ApiCommnentVo> page = commentService.apiCourseCommentPage(courseId, accountId, pageNo, pageSize);

		Reply reply = new Reply();
		reply.setCourseId(courseId);
		reply.setDeleted(0);
		reply.setStatus(0);
		EntityWrapper<Reply> wrapper = new EntityWrapper<Reply>(reply);
		List<String> list = new ArrayList<String>();
		list.add("createTime");
		wrapper.orderDesc(list);
		List<Reply> replies = replyService.selectList(wrapper);

		Map<Long, List<Reply>> listMap = new HashMap<>();
		for (Reply reply1 : replies) {
			if (listMap.containsKey(reply1.getCommentId())) {
				listMap.get(reply1.getCommentId()).add(reply1);
			} else {
				List<Reply> replies1 = new ArrayList<>();
				replies1.add(reply1);
				listMap.put(reply1.getCommentId(), replies1);
			}
		}

		for (ApiCommnentVo listCommentVo : page.getRecords()) {
			listCommentVo.setContent(StringEscapeUtils.unescapeJava(listCommentVo.getContent()));
			if (listMap.containsKey(listCommentVo.getId())) {
				List<Reply> listReply = listMap.get(listCommentVo.getId());
				List<ReplyEntityVo> vos = new ArrayList<>();
				for (Reply it : listReply) {
					ReplyEntityVo vo = new ReplyEntityVo();
					BeanUtils.copyProperties(it,vo);
					vos.add(vo);
				}
				listCommentVo.setReplies(vos);
			}
		}
		return page;
	}

	/**
	 * 根据id 获得评论实体
	 * 
	 * @param id
	 * @return
	 */
	@GetMapping("/get")
	public Comment get(@RequestParam("id") Long id) {
		Comment comment = commentService.selectById(id);
		comment.setContent(StringEscapeUtils.unescapeJava(comment.getContent()));
		return comment;
	}

	/**
	 * PC端课程评论列表
	 * 
	 * @param commentVo
	 * @return
	 */
	@PostMapping("/pc/list")
	public PageCommentVo pcList(@RequestBody CourseCommentVo commentVo) {
		Page<ListCommentVo> page = commentService.getCommentPage(commentVo.getCourseId(), commentVo.getPageNo(),
				commentVo.getPageSize(), null, 0);
		PageCommentVo pageCommentVo = new PageCommentVo();
		Double level = courseService.getAvgLevel(commentVo.getCourseId());
		pageCommentVo.setAvgLevel(level);
		List<ListCommentVo> list = new ArrayList<ListCommentVo>();
		for (ListCommentVo listCommentVo : page.getRecords()) {
			listCommentVo.setContent(StringEscapeUtils.unescapeJava(listCommentVo.getContent()));
			Page<Reply> replyPage = this.replyList2(listCommentVo.getId(), commentVo.getReplyPageNo(),
					commentVo.getReplyPageSize());
			if (null != replyPage.getRecords() && replyPage.getSize() > 0) {

				List<Reply> listReply = replyPage.getRecords();
				List<ReplyEntityVo> vos = new ArrayList<>();
				for (Reply it : listReply) {
					ReplyEntityVo vo = new ReplyEntityVo();
					BeanUtils.copyProperties(it,vo);
					vos.add(vo);
				}
				listCommentVo.setReplies(vos);
			}
			Boolean hasThumbsUp = thumbsUpService.judgeHasThumbsUp(commentVo.getCourseId(), listCommentVo.getId());
			listCommentVo.setHasThumbsUp(hasThumbsUp);
			if (listCommentVo.getContent()!=null) {
				list.add(listCommentVo);
			}
		}
		page.setRecords(list);
		page.setTotal(page.getTotal());
		pageCommentVo.setPage(page);
		return pageCommentVo;
	}

	/**
	 * 课程评论（回复）上架
	 * 
	 * @param id
	 * @param type
	 * @return boolean
	 */
	@GetMapping("/up")
	public Boolean up(@RequestParam("id") Long id, @RequestParam("type") Integer type) {
		if (type == 0) {
			Comment comment = commentService.selectById(id);
			if (comment != null) {
				comment.setStatus(0);
				commentService.updateById(comment);
				return true;
			} else {
				return false;
			}
		} else {
			Reply reply = replyService.selectById(id);
			if (reply != null) {
				reply.setStatus(0);
				replyService.updateById(reply);
				return true;
			} else {
				return false;
			}
		}
	}

	/**
	 * 课程评论（回复）下架
	 * 
	 * @param id
	 * @param type
	 * @return
	 */
	@GetMapping("/down")
	public Boolean down(@RequestParam("id") Long id, @RequestParam("type") Integer type) {
		if (type == 0) {
			Comment comment = commentService.selectById(id);
			if (comment != null) {
				comment.setStatus(1);
				commentService.updateById(comment);
				return true;
			} else {
				return false;
			}
		} else {
			Reply reply = replyService.selectById(id);
			if (reply != null) {
				reply.setStatus(1);
				replyService.updateById(reply);
				return true;
			} else {
				return false;
			}
		}
	}

	@GetMapping("/list/export")
	public Map<String, Object> export(
			@ApiParam(name = "courseId", value = "课程id") @RequestParam(name = "courseId") Long courseId,
			@ApiParam(name = "courseName", value = "课程名称") @RequestParam(name = "courseName") String courseName,
			@RequestParam(name = "commentator", required = false) String commentator,
			@RequestParam(name = "status", required = false) Integer status) {
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = "导出评论信息-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("courseId", courseId);
		map.put("courseName", courseName);
		map.put("status", status);
		map.put("result", result);
		map.put("commentator", commentator);
		commentListExportAsync.execute(map, true);
		return map;
	}
	
	/**
	 * 用于学员PC端回复列表查询
	 * @param commmentId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/reply/list2")
	public Page<Reply> replyList2(@RequestParam("commmentId") Long commmentId, @RequestParam("pageNo") Integer pageNo,
			@RequestParam("pageSize") Integer pageSize) {
		Page<Reply> page = new Page<Reply>(pageNo, pageSize);
		Reply reply = new Reply();
		reply.setCommentId(commmentId);
		reply.setDeleted(0);
		reply.setStatus(0);
		EntityWrapper<Reply> wrapper = new EntityWrapper<Reply>(reply);
		List<String> list = new ArrayList<String>();
		list.add("createTime");
		wrapper.orderDesc(list);
		replyService.selectPage(page, wrapper);
		Comment comment = commentService.selectById(commmentId);
		String userAvatar = null;
		if (comment != null) {
			userAvatar = comment.getUserAvatar();
		}
		String fullName;
		String name;
		for (Reply listCommentVo : page.getRecords()) {
			if (null!=listCommentVo.getReplyParentId()&&0!=listCommentVo.getReplyParentId()) {
				Long accountId = replyService.selectById(listCommentVo.getReplyParentId()).getAccountId();
				AccountVO findById = accountClient.findById(accountId);
				listCommentVo.setParentAccountFullName(null==findById.getFullName()||""==findById.getFullName()?findById.getName():findById.getFullName());
			}else {
				AccountVO findById = accountClient.findById(commentService.selectById(listCommentVo.getCommentId()).getAccountId());
				listCommentVo.setParentAccountFullName(null==findById.getFullName()||""==findById.getFullName()?findById.getName():findById.getFullName());
			}
			AccountVO accountVO = accountClient.findById(listCommentVo.getAccountId());
			name = accountVO.getName();
			fullName = accountVO.getFullName();
			listCommentVo.setReplier(name);
			listCommentVo.setContent(StringEscapeUtils.unescapeJava(listCommentVo.getContent()));
			listCommentVo.setUserAvatar(userAvatar);
			listCommentVo.setReplierName(null == fullName||""==fullName ? name : fullName);
		}
		return page;
	}
	
	@GetMapping("/getCount")
	public List<Integer> countComm(@RequestParam("courseIds") List<Long> courseIds) {
		List<Integer> list = new ArrayList<Integer>();
		for(Long courseId:courseIds) {
			Comment comment = new Comment();
			comment.setCourseId(courseId);
			comment.setDeleted(0);
			comment.setStatus(0);
			EntityWrapper<Comment> wrapper = new EntityWrapper<Comment>(comment);
			int count = commentService.selectCount(wrapper);
			list.add(count);
		}
		return list;
	}
}
