package com.yizhi.course.application.controller;


import com.yizhi.course.application.service.ICourseCountService;
import com.yizhi.course.application.service.ICourseStudentStatusService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.vo.CourseDashboardParam;
import com.yizhi.course.application.vo.CourseDashboardVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author fulan123
 * @since 2018-03-30
 */
@RestController
@RequestMapping("/dashboard")
public class CourseDashboardController {

    private Logger LOGGER = LoggerFactory.getLogger(CourseDashboardController.class);

    @Autowired
    private ICourseStudentStatusService courseStudentStatusService;
    @Autowired
    private IRecordeService recordeService;
    @Autowired
    private ICourseCountService courseReadRecordService;



    @PostMapping("/getCourseDashboardData")
    public Map<Long, CourseDashboardVO> getCourseDashboardData(@RequestBody CourseDashboardParam param) {
        if (param.getSiteId() == null || param.getSiteId() == 0L) {
            LOGGER.error("站点id为空,dashboard获取数据失败!!");
            return null;
        }
        return courseStudentStatusService.getCourseDashboardData(param);
    }

    /**
     * 根据时间站点  获取《用户id,时间内的累计学习时长》
     * @param param
     * @return
     */
    @PostMapping("/getCourseStudyTime")
    public Map<Long, Long> getCourseStudyTime(@RequestBody CourseDashboardParam param) {
        if (param.getSiteId() == null || param.getSiteId() == 0L) {
            LOGGER.error("站点id为空,dashboard获取数据失败!!");
            return null;
        }
        return recordeService.getCourseStudyTime(param);
    }

    /**
     * 根据时间实时获取当前用户课程的累计学习时间
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/todayStudyTime")
    public Long todayStudyTime(@RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate) {
        return recordeService.todayStudyTime(startDate, endDate);

    }

    /**
     * 实时获取浏览课程数
     * @return
     */
    @GetMapping("/viewRecords/getCount")
    public Integer getViewRecordsCount() {
        return courseReadRecordService.getCount();
    }


}