package com.yizhi.course.application.controller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.feign.CoursePcClient;
import com.yizhi.course.application.mapper.StatisticsCourseMapper;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.task.CourseStudyInitAsync;
import com.yizhi.course.application.vo.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: XieHaijun
 * @Description: pc端课程相关
 * @Date: Created in 10:13 2018/12/19
 * @Modified By
 */
@RestController
@RequestMapping("/pc")
public class CoursePcController {

    @Autowired
    private ICourseService courseService;

    @Autowired
    private ICourseAccountService courseAccountService;

    @Autowired
    private IClassifyService classifyService;

    @Autowired
    private ITestCaseService testCaseService;
    
    @Autowired
    private StatisticsCourseMapper statisticsCourseMapper;

    @Autowired
    private CourseStudyInitAsync courseStudyInitAsync;

    @Autowired
    private ICourseStudentStatusService studentStatusService;

    @Autowired
    IdGenerator idGenerator;

    private Logger LOGGER = LoggerFactory.getLogger(CoursePcController.class);
    
    

    /**
     * 课程列表
     * @param classifyId 课程分类id
     * @return
     */
    @GetMapping("/list")
    public Page<HomePageCoursePageVo> listCourseInfo(
                                    @RequestParam(value="classifyId",required = false) Long classifyId,
                                    @RequestParam(value = "pageNo",required = false ) Integer pageNo,
                                    @RequestParam(value = "pageSize",required = false ) Integer pageSize,
                                    @RequestParam(value = "isQueryChildren",required = false) Boolean isQueryChildren) {
        Page<HomePageCoursePageVo> retPage = new Page<>(pageNo,pageSize);

        if(Objects.isNull(isQueryChildren)){
            isQueryChildren = Boolean.TRUE;
        }

        List<Long> courseIds = null;
        Page<Long> idsPage = courseAccountService.selectLoginAccountCourseIds(classifyId,null,
                null,isQueryChildren,null);
        if(idsPage!=null){
            courseIds = idsPage.getRecords();
            if(CollectionUtils.isEmpty(courseIds)){
                return retPage;
            }
            retPage = getHomePageCoursePageVoPage(pageNo, pageSize, courseIds);
        }

        return retPage;
    }
    /**
     * 课程列表v2版本
     * @param classifyId 课程分类id
     * 新增课程排序 默认最后一次修改时间倒叙
     * @param hotDesc 人气最高 排序 默认null  人气相同按照最后一次修改时间倒叙
     * @param gradeDesc 好评优先 好评相同；按照最后一次修改时间倒叙
     */
    @GetMapping("/v2/list")
    public Page<HomePageCoursePageVoV2> listCourseInfoV2(
            @RequestParam(value="classifyId",required = false) Long classifyId,
            @RequestParam(value = "pageNo",required = false ) Integer pageNo,
            @RequestParam(value = "pageSize",required = false ) Integer pageSize,
            @RequestParam(value = "isQueryChildren",required = false) Boolean isQueryChildren,
            @RequestParam(value = "hotDesc",required = false) Boolean hotDesc,
            @RequestParam(value = "gradeDesc",required = false) Boolean gradeDesc) {
        Page<HomePageCoursePageVoV2> retPage = new Page<>(pageNo,pageSize);

        if(Objects.isNull(isQueryChildren)){
            isQueryChildren = Boolean.TRUE;
        }

        List<Long> courseIds = null;
        Page<Long> idsPage = courseAccountService.selectLoginAccountCourseIds(classifyId,null,
                null,isQueryChildren,null);
        if(idsPage!=null){
            courseIds = idsPage.getRecords();
            if(CollectionUtils.isEmpty(courseIds)){
                return retPage;
            }
            retPage = getHomePageCoursePageVoPageV2(pageNo, pageSize, courseIds,hotDesc,gradeDesc);
        }

        return retPage;
    }

    private Page<HomePageCoursePageVo> getHomePageCoursePageVoPage(Integer pageNo, Integer pageSize, List<Long> courseIds) {
        Page<HomePageCoursePageVo> retPage;
        retPage = courseService.findCourseList(courseIds,pageNo,pageSize);

        List<HomePageCoursePageVo> data = retPage.getRecords();
        // 查询课程的总学习时长
        Map<Long, Long> courseDuration = getCourseDuration(data.stream().map(stuCourse->stuCourse.getCourseId())
                .collect(Collectors.toList()));
        for(HomePageCoursePageVo obj : data){
            Long currCourseId = obj.getCourseId();
            if(Objects.nonNull(currCourseId)){
                Long durationTem = courseDuration.get(currCourseId);
                obj.setDuration(durationTem==null?0:Integer.valueOf(durationTem.toString()));
            }
        }
        return retPage;
    }

    /**
     * v2版本
     @param hotDesc 人气最高 排序 默认null  人气相同按照最后一次修改时间倒叙
      ** @param gradeDesc 好评优先 好评相同；按照最后一次修改时间倒叙
     */
    private Page<HomePageCoursePageVoV2> getHomePageCoursePageVoPageV2(Integer pageNo, Integer pageSize, List<Long> courseIds, Boolean hotDesc, Boolean gradeDesc) {
        Page<HomePageCoursePageVoV2> retPageV2 = new Page<>();
        List<HomePageCoursePageVoV2> coursePageVoV2List = new ArrayList<>();
        Page<HomePageCoursePageVo> retPage = courseService.findCourseListV2(courseIds,pageNo,pageSize,hotDesc,gradeDesc);

        List<HomePageCoursePageVo> data = retPage.getRecords();
        // 查询课程的总学习时长
        Map<Long, Long> courseDuration = getCourseDuration(data.stream().map(stuCourse->stuCourse.getCourseId())
                .collect(Collectors.toList()));
        for(HomePageCoursePageVo obj : data){
            //v2版本修改评分为字符串
            HomePageCoursePageVoV2 pageVoV2 = new HomePageCoursePageVoV2();
            BeanUtils.copyProperties(obj,pageVoV2);
            pageVoV2.setLevel(String.valueOf(obj.getLevel()));
            Long currCourseId = obj.getCourseId();
            if(Objects.nonNull(currCourseId)){
                Long durationTem = courseDuration.get(currCourseId);
                obj.setDuration(durationTem==null?0:Integer.valueOf(durationTem.toString()));
                pageVoV2.setDuration(durationTem==null?0:Integer.valueOf(durationTem.toString()));
            }
            coursePageVoV2List.add(pageVoV2);
        }
        BeanUtils.copyProperties(retPage,retPageV2);
        retPageV2.setRecords(coursePageVoV2List);
        return retPageV2;
    }

    @GetMapping("/classify/list")
    public List<ClassifyVo> listClassifyInfo(
            @RequestParam(value="classifyId",required = false) Long classifyId){
        RequestContext rc = ContextHolder.get();
        Classify classify = new Classify();
        classify.setCompanyId(rc.getCompanyId());
        classify.setSiteId(rc.getSiteId());
        classify.setStatus(1);
        if(Objects.isNull(classifyId)){
            classify.setLayer(1);
        }else {
            classify.setParentId(classifyId);
        }

        List<Classify> retData = classifyService.selectList(QueryUtil.condition(classify).ne("source",4).
                orderBy("sort",false));
        if(!CollectionUtils.isEmpty(retData)){
            Set<Long> cids = retData.stream().map(item->item.getId()).collect(Collectors.toSet());
            List<ClassifyVo> existVos = classifyService.findExistChildNodes(new ArrayList<>(cids));
            Map<Long,Integer> exitsMap = null;
            boolean isExits = false;
            if(!CollectionUtils.isEmpty(existVos)){
                isExits = true;
                exitsMap = existVos.stream().collect(Collectors.toMap(key->key.getId(),val->val.getTotal()));
            }
            List<ClassifyVo> data = new ArrayList<>();
            for(Classify item : retData){
                ClassifyVo vo = new ClassifyVo();
                BeanUtils.copyProperties(item,vo);
                if(isExits && exitsMap.containsKey(vo.getId())){
                    vo.setExistChildNode(true);
                }
                data.add(vo);
            }
            return data;
        }
        return  null;

    }
    
    
    
    @GetMapping("/list/course/grade")
    public List<ModuleAllVoCourse> listCoursePCgrade(@RequestParam(name="ids")List<Long> ids){
    	return courseService.listCoursePCgrade(ids);
    	
    }
    
    
    @PostMapping("/list/course/get/material/time")
    public Map<Long,Long> getMaterialTimeByCourseIds(@RequestBody List<Long> ids){
        return getCourseDuration(ids);
    }


    /**
     * 查询课程学时
     * @param ids
     * @return
     */
    @PostMapping("/course/study/hour/get")
    public Map<Long,Float> getStudyHourByCourseIds(@RequestBody List<Long> ids){
        List<Course> data = courseService.selectBatchIds(ids);
        if(CollectionUtils.isEmpty(data)){
            return null;
        }
        Map<Long,Float> ret = new HashMap<>();
        for(Course item : data){
            ret.put(item.getId(),item.getTotalDuration());
        }
        return ret;
    }

    /**
     * 获取课程的总学习时长
     * @param ids
     * @return
     */
    private Map<Long, Long> getCourseDuration(List<Long> ids) {
        long startDate = System.currentTimeMillis();
        List<Material> listMaterial=statisticsCourseMapper.getCourseDurationById(ids);
        Map<Long,Long> map=new HashMap<>();
        if(!CollectionUtils.isEmpty(listMaterial)) {
            map=new HashMap<>();
            for (Material material:listMaterial) {
                long ms = 0;
                Integer hour = material.getTextHour();
                if(hour != null && hour.intValue()>0){
                    ms+=(hour.intValue()*3600);
                }
                Integer minute = material.getTextMinute();
                if(minute != null && minute.intValue()>0){
                    ms+=(minute.intValue()*60);
                }
                Integer second = material.getTextSecond();
                if(second != null && second.intValue()>0){
                    ms+=second.intValue();
                }
                map.put(material.getId(), ms);
            }
        }
        LOGGER.info("*************************************************课程时长查询耗时{}毫秒",(System.currentTimeMillis()-startDate));
        return map;
    }


    /**
     * 根据relation_id查询课程
     * @param relationIds
     * @return
     */
    @GetMapping("/list/course/portal/pc/api")
    public List<ModuleAllVoCourse> listCourseScope(@RequestParam(name="relationIds",required=false)List<Long> relationIds,@RequestParam(name="num")Integer num,@RequestParam(name="listIds",required=false)List<Long> listIds){
        return courseService.listCourseRelationId(relationIds,num,listIds);
    }

    @GetMapping("/chapter/test/list")
    public List<TestCaseRecordVO> chapterTestList(@RequestParam(name="testId") Long testId,
                                                  @RequestParam(name="accountId") Long accountId){
        return testCaseService.chapterTestList(testId,accountId);
    }


    @GetMapping("/my/study/count")
    public MyCourseStudyCountVo getMyCourseStudyCount(@RequestParam("siteId") Long siteId,
                                                      @RequestParam("accountId") Long accountId) {
        /*Map<String, Map<Long,List<Chapter>>> dataMap = courseService.getMyStudyCourseIds(siteId,accountId);
        MyCourseStudyCountVo vo = setMyCourseStudyCount(dataMap);*/
        // 使用新的课程完成状态统计完成数据
        Map<String,List<CourseStudentStatus>> dataMap = courseService.getMyStudyCourseStatus(siteId,accountId,null,null);
        MyCourseStudyCountVo vo = setMyCourseStatusCount(dataMap);
        return vo;
    }

    /**
     * 计算课程完成的数量
     * @param dataMap
     * @return
     */
    private MyCourseStudyCountVo setMyCourseStudyCount(Map<String, Map<Long, List<Chapter>>> dataMap) {
        int sumCountNum = 0;
        int studyingNum = 0;
        int completeNum = 0;
        if(dataMap != null){
            studyingNum = dataMap.get(CoursePcClient.COURSE_STATUS_STUDYING).size();
            completeNum = dataMap.get(CoursePcClient.COURSE_STATUS_COMPLETE).size();
            sumCountNum = studyingNum+completeNum;
        }
        MyCourseStudyCountVo vo = new MyCourseStudyCountVo();
        vo.setSumCountNum(sumCountNum);
        vo.setStudyingCountNum(studyingNum);
        vo.setCompleteCountNum(completeNum);
        return vo;
    }

    private MyCourseStudyCountVo setMyCourseStatusCount(Map<String, List<CourseStudentStatus>> dataMap) {
        int sumCountNum = 0;
        int studyingNum = 0;
        int completeNum = 0;
        if(dataMap != null){
            List<CourseStudentStatus> studying = dataMap.get(CoursePcClient.COURSE_STATUS_STUDYING);
            if(!CollectionUtils.isEmpty(studying)){
                studyingNum = studying.size();
            }
            List<CourseStudentStatus> complete = dataMap.get(CoursePcClient.COURSE_STATUS_COMPLETE);
            if(!CollectionUtils.isEmpty(complete)){
                completeNum = complete.size();
            }
        }
        MyCourseStudyCountVo vo = new MyCourseStudyCountVo();
        vo.setSumCountNum(sumCountNum);
        vo.setStudyingCountNum(studyingNum);
        vo.setCompleteCountNum(completeNum);
        return vo;
    }

    @GetMapping("/my/study/list")
    public Page<MyCourseRecordeVo> getMyCourseStudyList(
            @RequestParam("siteId") Long siteId,
            @RequestParam("accountId") Long accountId,
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "type", required = false) Integer type, //课程是否完成 0 为进行中 1 为已完成
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize,
            @RequestParam(name = "startDate", required=false) Long startDate,
            @RequestParam(name = "endDate", required=false) Long endDate
    ) {
        LOGGER.info("请求开始时间{}和结束时间{}",startDate,endDate);
        Page<MyCourseRecordeVo> retData = new Page<>();
        retData.setCurrent(pageNo);
        retData.setSize(pageSize);
        int typeTemp = type==null?-1:type.intValue();
        /*Map<Long,List<Chapter>> studyCourseList = new HashMap<>();
        Map<String, Map<Long,List<Chapter>>> dataMap = courseService.getMyStudyCourseIds(siteId,accountId);*/
        List<CourseStudentStatus> studyCourseList = new ArrayList<>();
        Date startDateCovert = null;
        Date endDateCovert = null;
        if(startDate!=null && endDate!=null){
            startDateCovert = new Date(startDate.longValue());
            endDateCovert = new Date(endDate.longValue());
        }
        Map<String,List<CourseStudentStatus>> dataMap = courseService.getMyStudyCourseStatus(siteId,accountId,startDateCovert,endDateCovert);

        List<CourseStudentStatus> studying = dataMap.get(CoursePcClient.COURSE_STATUS_STUDYING);
        List<CourseStudentStatus> complete = dataMap.get(CoursePcClient.COURSE_STATUS_COMPLETE);
        if(dataMap != null){
            switch (typeTemp){
                case 0:
                    if(!CollectionUtils.isEmpty(studying)){
                        studyCourseList.addAll(studying);
                    }
                    break;
                case 1:
                    if(!CollectionUtils.isEmpty(complete)){
                        studyCourseList.addAll(complete);
                    }
                    break;
                default:
                    if(!CollectionUtils.isEmpty(studying)){
                        studyCourseList.addAll(studying);
                    }
                    if(!CollectionUtils.isEmpty(complete)){
                        studyCourseList.addAll(complete);
                    }
            }
        }
        if(studyCourseList.size()>0){
            // todo 根据名称查找业务暂时不考虑，后期有需要再加

            // 获取我的学习统计数据
            //MyCourseStudyCountVo vo = setMyCourseStudyCount(dataMap);
            MyCourseStudyCountVo vo = setMyCourseStatusCount(dataMap);

            // 查询课程列表详情
            Page<MyCourseRecordeVo> data = new Page<>();
            // 学习的课程列表
            //Set<Long> findMyCourseId = studyCourseList.keySet();
            Map<Long,CourseStudentStatus> findMyCourseIdMap = studyCourseList.stream().parallel().collect(Collectors.toMap(key->key.getCourseId(),val->val
                    ,(e1, e2) -> e1));// 添加,(e1, e2) -> e1，也就是有重复值的时候，我取先进来的值，也就是e1。
            Set<Long> findMyCourseId = findMyCourseIdMap.keySet();

            // 查询课程信息
            data = courseService.findCourseBaseInfoByIds(findMyCourseId,siteId,accountId,pageNo,pageSize);
            // 获取主题数据
            List<MyCourseRecordeVo> coreData = data.getRecords();

            if(! CollectionUtils.isEmpty(coreData)){
                // 查询课程的总学习时长
                Map<Long, Long> courseDuration = getCourseDuration(coreData.stream().map(stuCourse->stuCourse.getId())
                        .collect(Collectors.toList()));

                List<ApiListCourseVo> levels = courseService.findWechatCourseLevelList(new ArrayList<>(findMyCourseId));
                boolean exitsLevel = false;
                Map<Long,ApiListCourseVo> exitsLevels = null;
                if(!CollectionUtils.isEmpty(levels)){
                    exitsLevel = true;
                    exitsLevels = levels.stream().collect(Collectors.toMap(key->key.getId(),val->val));
                }
                for(MyCourseRecordeVo itemObj : coreData){
                    Long currCourseId = itemObj.getId();
                    Long durationTem = courseDuration.get(currCourseId);
                    itemObj.setDuration(durationTem==null?0:Integer.valueOf(durationTem.toString()));
                    // 计算课程星级
                    if(exitsLevel){
                        if(exitsLevels.containsKey(currCourseId)){
                            ApiListCourseVo levelVo = exitsLevels.get(currCourseId);
                            itemObj.setLevel(levelVo.getLevel());
                        }else {
                            itemObj.setLevel(5.0);//默认为5星
                        }
                    }else {
                        itemObj.setLevel(5.0);//默认为5星
                    }
                    // 计算课程的完成百分比
                    /*List<Chapter> studyChapterListNum =  studyCourseList.get(currCourseId);
                    int sumNum = studyChapterListNum.size();
                    long completionNumber = studyChapterListNum.stream()
                            .filter(studyNum->Objects.nonNull(studyNum.getOvered())).count();
                    //LOGGER.info("课程id={},课程章节信息={}",currCourseId, JSON.toJSONString(studyChapterListNum));
                    //LOGGER.info("完成的课程数量={}",completionNumber);
                    if(completionNumber>0){
                        *//*double percent = new BigDecimal((float)completionNumber.intValue()/sumNum)
                                .setScale(3, BigDecimal.ROUND_HALF_UP).doubleValue();*//*

                        double percent = BigDecimal.valueOf(completionNumber).
                                divide(BigDecimal.valueOf(sumNum),3,BigDecimal.ROUND_HALF_UP).doubleValue();
                        //LOGGER.info("完成百分比={}",percent);
                        itemObj.setPercent((int)(percent*100));
                    }else {
                        itemObj.setPercent(0);
                    }*/
                    CourseStudentStatus currStudentStatus = findMyCourseIdMap.get(currCourseId);
                    if(currStudentStatus != null){
                        itemObj.setPercent(currStudentStatus.getPercent());
                    }

                }
                data.setRecords(coreData);
                List<String> studyNumCountObj = new ArrayList<>();
                studyNumCountObj.add(vo.getSumCountNum().toString());// 全部学习的课程
                studyNumCountObj.add(vo.getCompleteCountNum().toString());// 学习完成的课程
                studyNumCountObj.add(vo.getStudyingCountNum().toString());// 学习中的课程
                data.setDescs(studyNumCountObj);// 存放到Descs对象
                return data;
            }
        }
        return retData;
    }

    @GetMapping("/status/init")
    public void studeyStatusInit(@RequestParam(value = "siteId",required = false) Long siteId){
        Map<String, Object> param = new HashMap<>();
        param.put("siteId",siteId);
        courseStudyInitAsync.execute(param,true);
        //return true;
    }

    @PostMapping("/status/ids/get")
    public List<CourseStudentStatus> getStatusByIds(@RequestBody List<Long> ids){
        RequestContext requestContext = ContextHolder.get();
        EntityWrapper<CourseStudentStatus> ew = QueryUtil.condition(new CourseStudentStatus());
        ew.in("course_id", ids);
        ew.eq("site_id", requestContext.getSiteId());
        ew.eq("account_id", requestContext.getAccountId());
        List<CourseStudentStatus> data = studentStatusService.selectList(ew);

        if(!CollectionUtils.isEmpty(data)){
            List<CourseStudentStatus> dataNew = new ArrayList<>();
            Map<Long,List<CourseStudentStatus>> tmp = data.stream().collect(Collectors.groupingBy(key->key.getCourseId()));
            for(List<CourseStudentStatus> item :tmp.values()){
                dataNew.add(item.get(0));
                /*if(item.size() > 1){
                    item.stream().filter()
                }else{
                    dataNew.add(item.get(0));
                }*/
            }
            return dataNew;
        }

        return data;
    }

    @GetMapping("/recommend")
    public List<HomePageCoursePageVo> recommend(){
        List<HomePageCoursePageVo> data = null;

        // 查询可见范围内能见的课程列表（上架的）
        List<Long> courseIds = null;
        Page<Long> idsPage = courseAccountService.selectLoginAccountCourseIds(null,null,
                null,true,null);
        if(idsPage!=null){
            courseIds = idsPage.getRecords();
            data = new ArrayList<>();
            if(CollectionUtils.isEmpty(courseIds)){
                return null;
            }
            // 查询站点下热门课程的排行榜
            RequestContext rc = ContextHolder.get();
            List<HomePageCoursePageVo> tmpData = courseService.findRecommendCourseList(courseIds,rc.getCompanyId()
                    ,rc.getSiteId(),rc.getAccountId());
            if(CollectionUtils.isEmpty(tmpData)){
                return null;
            }
            // 获取排行榜的课程列表(最大取20条)
            List<Long> ids = tmpData.stream().limit(20).map(itemObj->itemObj.getCourseId()).collect(Collectors.toList());
            Page<HomePageCoursePageVo> retPage = getHomePageCoursePageVoPage(1, 20, ids);
            if(retPage!=null){
                List<HomePageCoursePageVo> dataNoSort = retPage.getRecords();
                Map<Long,HomePageCoursePageVo> dataMap = dataNoSort.stream().
                        collect(Collectors.toMap(key->key.getCourseId(),val->val));
                for(Long id : ids){
                    if(dataMap.containsKey(id)){
                        data.add(dataMap.get(id)) ;
                    }
                }
            }
        }

        return data;
    }

    /**
     * 推荐课程列表  版本2
     * 课程评分修改为string类型;返回类型由 HomePageCoursePageVo -> HomePageCoursePageVoV2
     */
    @GetMapping("/v2/recommend")
    public List<HomePageCoursePageVoV2> recommendV2(){
        List<HomePageCoursePageVoV2> dataV2 = new ArrayList<>();

        // 查询可见范围内能见的课程列表（上架的）
        List<Long> courseIds = null;
        Page<Long> idsPage = courseAccountService.selectLoginAccountCourseIds(null,null,
                null,true,null);
        if(idsPage!=null){
            courseIds = idsPage.getRecords();
            if(CollectionUtils.isEmpty(courseIds)){
                return null;
            }
            // 查询站点下热门课程的排行榜
            RequestContext rc = ContextHolder.get();
            List<HomePageCoursePageVo> tmpData = courseService.findRecommendCourseList(courseIds,rc.getCompanyId()
                    ,rc.getSiteId(),rc.getAccountId());
            if(CollectionUtils.isEmpty(tmpData)){
                return null;
            }
            // 获取排行榜的课程列表(最大取20条)
            List<Long> ids = tmpData.stream().limit(20).map(itemObj->itemObj.getCourseId()).collect(Collectors.toList());
            Page<HomePageCoursePageVoV2> retPage = getHomePageCoursePageVoPageV2(1, 20, ids,false,false);
            if(retPage!=null){
                List<HomePageCoursePageVoV2> dataNoSort = retPage.getRecords();
                Map<Long,HomePageCoursePageVoV2> dataMap = dataNoSort.stream().
                        collect(Collectors.toMap(key->key.getCourseId(),val->val));
                for(Long id : ids){
                    if(dataMap.containsKey(id)){
                        dataV2.add(dataMap.get(id)) ;
                    }
                }
            }
        }
        return dataV2;
    }
}
