package com.yizhi.course.application.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.studyHourReportExport.StudyHourDetailsExport;
import com.yizhi.course.application.studyHourReportExport.StudyHourReportExport;
import com.yizhi.course.application.task.CoursePointInitAsync;
import com.yizhi.course.application.task.CourseStudyAnalysisAsync;
import com.yizhi.course.application.vo.ProjectsDurationParamVO;
import com.yizhi.course.application.vo.StudyHourReportVO;
import com.yizhi.course.application.vo.StudyLog;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.point.application.vo.PointParamVO;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: XieHaijun
 * @Description: 课程学习相关处理
 * @Date: Created in 16:09 2018/11/13
 * @Modified By
 */
@RestController
@RequestMapping("/study")
public class CourseStudyController {

    @Autowired
    private TaskExecutor taskExecutor;

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    PointRedisFeignClients pointRedisFeignClients;
/*
    @Autowired
    AmqpTemplate amqpTemplate;*/

    @Autowired
    private ICourseService courseService;

    @Autowired
    private IChapterService chapterService;

    @Autowired
    private CourseStudyAnalysisAsync analysisAsync;

    @Autowired
    private CoursePointInitAsync coursePointInitAsync;

    @Autowired
    private ICourseStudentStatusService studentStatusService;

    @Autowired
    private ICourseStudyHourDetailedService studyHourDetailedService;

    @Autowired
    private ICourseStudyHourService studyHourService;

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    private StudyHourReportExport studyHourReportExport;

    @Autowired
    private StudyHourDetailsExport studyHourDetailsExport;

    @Autowired
    private IScormService scormService;

    @Autowired
    private ICourseMqRecordService courseMqRecordService;

    @Autowired
    private RedisCache redisCache;

    public final static String IS_OPEN_COURSE_MQ_RECODE = "course:is.open.course.mq.recode";

    private Logger LOGGER = LoggerFactory.getLogger(CourseStudyController.class);

    // 课程积分发放的事件名称
    private static final String EVENT_NAME = "pointCourse";

    // 课程积分发放MQ队列名称
    private static final String COURSE_MQ_SEND_SCORE = "course";

    private static final String EXPERIENCE_COURSE = "experienceCourse";

    @Autowired
    private IRecordeService recordeService;

    @Autowired
    private ILastViewRecordService lastViewRecordService;

    /**
     * 初始化积分发放
     *
     * @param siteId 站点id
     * @return
     */
    @GetMapping("/point/init")
    public Boolean coursePointInit(@RequestParam(value = "siteId", required = false) Long siteId) {
        Map<String, Object> param = new HashMap<>();
        if (siteId != null) {
            param.put("siteId", siteId);
        }
        LOGGER.info("初始化积分发放开始");
        coursePointInitAsync.execute(param, true);
        LOGGER.info("初始化积分发放异步开始调用，返回接口数据");
        return true;
    }

    /**
     * 学习日志分析
     *
     * @param studyLogList(按课程和学习账户id)
     * @return
     */
    @RequestMapping("/studyLog/analysis")
    public Boolean studyLogAnalysis(@RequestBody List<StudyLog> studyLogList) {
        if (!CollectionUtils.isEmpty(studyLogList)) {
            Map<String, Object> param = new HashMap<>();
            param.put("courseStudyRecd", studyLogList);
            analysisAsync.execute(param, true);
        }
        return true;
    }

    /**
     * 根据学习记录发放积分
     *
     * @param studyRecd 学习记录
     * @return
     */
    @PostMapping("/status")
    public Boolean completionStatus(@RequestBody Recorde studyRecd) {
        //return calculateStudyStatus(studyRecd);
        return calculateStudyStatusNew(studyRecd);
    }

    public Boolean save(LastViewRecord vo){

        try {
            LastViewRecord lastViewRecordEw = new LastViewRecord();
            lastViewRecordEw.setAccountId(vo.getAccountId());
            lastViewRecordEw.setChapterId(vo.getChapterId());
            LastViewRecord dbInfo = lastViewRecordService.selectOne(QueryUtil.condition(lastViewRecordEw));
            if(dbInfo == null){
                vo.setCreateTime(new Date());
                lastViewRecordService.insert(vo);
            }else{
                lastViewRecordService.updateByProgress(vo.getAccountId(),vo.getChapterId(),vo.getViewPonit());
            }
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    public Boolean calculateStudyStatusNew(Recorde studyRecd){
        LOGGER.info("计算学习记录的接口 请求的数据={}",JSON.toJSONString(studyRecd));

        if(studyRecd.getDurationTmp() == null){
            studyRecd.setDurationTmp(0);
        }
        // 课程id
        Long courseId = studyRecd.getCourseId();
        // 查询课程信息
        Course course = courseService.selectById(courseId);
        if(course == null){
            return false;
        }
        // 当次学习的记录
        String currSessionId = studyRecd.getSessionId();
        // 如果存在sessionId，查询该次sessionId的学习完成状态（true表示已经学习完成）
        boolean sessionIdStudyStatus = false;
        boolean extisflag = false;
        Recorde sessionIdRecode = null;
        Integer progress = studyRecd.getProgress();
        if(progress != null && progress.intValue()>=0){
            LastViewRecord vo = new LastViewRecord();
            vo.setAccountId(studyRecd.getAccountId());
            vo.setChapterId(studyRecd.getChapterId());
            vo.setCompanyId(studyRecd.getCompanyId());
            vo.setCourseId(studyRecd.getCourseId());
            vo.setOrgId(studyRecd.getOrgId());
            vo.setSiteId(studyRecd.getSiteId());
            vo.setViewPonit(progress.longValue());
            save(vo);
        }
        if(Objects.nonNull(currSessionId)){
            Recorde sessionIdRecodeEw = new Recorde();
            sessionIdRecodeEw.setAccountId(studyRecd.getAccountId());
            sessionIdRecodeEw.setChapterId(studyRecd.getChapterId());
            sessionIdRecodeEw.setSessionId(currSessionId);

            sessionIdRecode = recordeService.selectOne(QueryUtil.condition(sessionIdRecodeEw));
            if(sessionIdRecode != null ){
                extisflag = true;
                if(Integer.valueOf(1).equals(sessionIdRecode.getOvered())){
                    sessionIdStudyStatus = true;
                }
            }
        }
        // 判斷是否需要發放積分
        Integer currOvered = studyRecd.getOvered();

        LOGGER.info("当前sessionId是否存在{},当前完成状态={}",sessionIdStudyStatus,currOvered);

        // 积分发放规则  1按课程  2 按章 3 按节
        int portRule = course.getPortRule();
        // 发放积分，如果该次学习没有完成过，并且本次学习记录是是已完成
        if(!sessionIdStudyStatus && Integer.valueOf(1).equals(currOvered)){
            switch (portRule){
                case 2:
                    boolean finishStatus = false;
                    Chapter currStudyChapter = chapterService.selectById(studyRecd.getChapterId());
                    LOGGER.info("按章发放积分开始 。。。。。。。。。。。。");
                    LOGGER.info("当前的章信息={}",JSON.toJSONString(currStudyChapter));
                    Long parentId = currStudyChapter.getParentId();
                    Chapter studyChapterGp = null;
                    int sumScore = 0;
                    if(parentId != null && parentId.longValue()>0){
                        LOGGER.info("章下的节学习，其父亲id={}",parentId.longValue());
                        EntityWrapper<Chapter> ewParentId = QueryUtil.condition(new Chapter());
                        ewParentId.eq("course_id", studyRecd.getCourseId());
                        ewParentId.eq("parent_id", parentId);
                        List<Chapter> data = chapterService.selectList(ewParentId);
                        LOGGER.info("查询当前父id的同级别的节列表={}",JSON.toJSONString(data));
                        if(!CollectionUtils.isEmpty(data) && data.size() == 1){ // 单节的章
                            finishStatus = true;
                            Integer score = currStudyChapter.getScore();
                            if(score != null && score.intValue() >0){
                                sumScore+=score.intValue();
                            }
                        }else { // 多节的章
                            // 判断章是否全部完成
                            List<Long> chapterIds = data.stream().map(chapter -> chapter.getId()).collect(Collectors.toList());
                            chapterIds.remove(studyRecd.getChapterId()); // 删除自己的
                            Collections.sort(chapterIds); //demo_list.stream().sorted(Comparator.comparing(bean::getXxx));
                            List<Long> finishChapterIds = chapterService.getFinishChapterIds(studyRecd.getAccountId(),chapterIds);
                            if(chapterIds.size() == finishChapterIds.size()){
                                finishStatus = true;
                                for(Chapter c : data){
                                    Integer score = c.getScore();
                                    if(score != null && score.intValue() >0){
                                        sumScore+=score.intValue();
                                    }
                                }
                            }
                        }

                    }else { // 直接添加内容，然后按章发放积分的
                        studyChapterGp = currStudyChapter;
                        Integer score = currStudyChapter.getScore();
                        if(score != null && score.intValue() >0){
                            sumScore = score.intValue();
                        }
                        finishStatus = true;
                    }

                    // 章的完成状态
                    if(finishStatus){
                        if(sumScore > 0){
                            LOGGER.info("按章完成，准备按章发放积分");
                            if(studyChapterGp == null){
                                studyChapterGp = chapterService.selectById(parentId);
                            }
                            String mqSendStatu = sendPoint(course, sumScore, studyRecd, studyChapterGp, COURSE_MQ_SEND_SCORE);
                            saveSendRQRecode("一般课程按章完成发放积分" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());
                        }
                        // 给经验发放发送消息
                        LOGGER.info("课程完成状态处理完成，准备配合积分发送消息");
                        String mqSendStatu = sendPoint(course, 0, studyRecd, currStudyChapter, EXPERIENCE_COURSE);
                        saveSendRQRecode("一般课程按章完成发放经验" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());
                    }
                    break;
                case 3:
                    // 按节发放积分，如果该次学习没有完成过，并且本次学习记录是是已完成
                    Chapter currChapter = chapterService.selectById(studyRecd.getChapterId());
                    if(currChapter == null){
                        return false;
                    }
                    Integer score = currChapter.getScore(); // 发放的积分数
                    if (score != null && score.intValue() > 0) {
                        // 按节直接发放积分
                        LOGGER.info("按节完成，准备按节发放积分");
                        String mqSendStatuOneA = sendPoint(course, score, studyRecd, currChapter, COURSE_MQ_SEND_SCORE);
                        saveSendRQRecode("一般课程按节完成发放积分"+"-"+mqSendStatuOneA, studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());
                    }
                    // 给经验发放发送消息
                    LOGGER.info("课程完成状态处理完成，准备配合积分发送消息");
                    String mqSendStatuOneB = sendPoint(course, 0, studyRecd, currChapter, EXPERIENCE_COURSE);
                    saveSendRQRecode("一般课程按节完成发放经验" +"-"+mqSendStatuOneB, studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());

                    break;
            }
        }
        // 判断课程完成状态
        double percentDouble = 0;
        boolean finishStatusAll = false;
        int sumScoreAll = 0;
        EntityWrapper<Chapter> ewParentIdP = QueryUtil.condition(new Chapter());
        ewParentIdP.eq("course_id", studyRecd.getCourseId());
        ewParentIdP.isNotNull("material_id");
        List<Chapter> dataP = chapterService.selectList(ewParentIdP);
        LOGGER.info("查询全部需要学习的章节信息={}",JSON.toJSONString(dataP));
        List<Long> ids = dataP.parallelStream().map(obj -> obj.getId()).collect(Collectors.toList());
        if(ids.size()==1 && ids.get(0).equals(studyRecd.getChapterId())){
            LOGGER.info("只有一个章节的");
            if(!sessionIdStudyStatus && Integer.valueOf(1).equals(currOvered)){
                finishStatusAll = true;
                Integer score = dataP.get(0).getScore();
                if(score != null && score.intValue() >0){
                    sumScoreAll+=score.intValue();
                }
            }

        }else{
            // 排除自己
            int allSize = ids.size();
            ids.remove(studyRecd.getChapterId());
            LOGGER.info("排除自己的id={}",studyRecd.getChapterId());
            Collections.sort(ids); //demo_list.stream().sorted(Comparator.comparing(bean::getXxx));
            List<Long> finishChapterIds = chapterService.getFinishChapterIds(studyRecd.getAccountId(),ids);
            LOGGER.info("需要学习的课程ids={},已经学习完成的章节ids={}",JSON.toJSONString(ids),JSON.toJSONString(finishChapterIds));
            BigDecimal finishNum = BigDecimal.valueOf(finishChapterIds.size());
            if(!sessionIdStudyStatus && Integer.valueOf(1).equals(currOvered)){
                finishNum = BigDecimal.valueOf(finishChapterIds.size()+1);
            }

            percentDouble = finishNum.divide(BigDecimal.valueOf(allSize), 3, BigDecimal.ROUND_HALF_UP).doubleValue();
            LOGGER.info("多个章节的学习完成状态={}",percentDouble);
            if(percentDouble == 1.0d){
                finishStatusAll = true;
                for(Chapter c : dataP){
                    Integer score = c.getScore();
                    if(score != null && score.intValue() >0){
                        sumScoreAll+=score.intValue();
                    }
                }
            }
        }

        // 查询课程学习状态
        CourseStudentStatus studentStatusEw = new CourseStudentStatus();
        studentStatusEw.setAccountId(studyRecd.getAccountId());
        studentStatusEw.setCourseId(studyRecd.getCourseId());
        CourseStudentStatus studentStatus = studentStatusService.selectOne(QueryUtil.condition(studentStatusEw));
        if (studentStatus == null) {
            studentStatus = new CourseStudentStatus();
            studentStatus.setId(idGenerator.generate());
            studentStatus.setCompanyId(studyRecd.getCompanyId());
            studentStatus.setSiteId(studyRecd.getSiteId());
            studentStatus.setOrgId(studyRecd.getOrgId());
            studentStatus.setAccountId(studyRecd.getAccountId());
            studentStatus.setCourseId(studyRecd.getCourseId());
        }
        studentStatus.setLastStudentTime(studyRecd.getEndTime()); // 最后学习时间
        // 表示全部完成的课程
        Chapter currStudyChapter = dataP.parallelStream().filter(filterObj -> filterObj.getId().equals(studyRecd.getChapterId())).findFirst().get();
        if(finishStatusAll){
            if(studentStatus.getFirstFinishTime() == null){
                studentStatus.setFirstFinishTime(studyRecd.getEndTime());
            }
            studentStatus.setStudentStatus(1);
            studentStatus.setPercent(100);
            // 如果按课程发放积分
            if(portRule == 1){
                if(sumScoreAll > 0){
                    LOGGER.info("按课程完成，准备按课程发放积分");
                    String mqSendStatuAll =  sendPoint(course, sumScoreAll, studyRecd, currStudyChapter, COURSE_MQ_SEND_SCORE);
                    saveSendRQRecode("一般课程按课程完成发放积分" +"-"+mqSendStatuAll, studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());
                }
                // 给经验发放发送消息
                LOGGER.info("课程完成状态处理完成，准备配合积分发送消息");
                String mqSendStatuAll =  sendPoint(course, 0, studyRecd, currStudyChapter, EXPERIENCE_COURSE);
                saveSendRQRecode("一般课程按课程完成发放经验" +"-"+mqSendStatuAll, studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());
            }
            // 可以发送项目完成状态
            Integer durationTmp = studyRecd.getDurationTmp();
            sendFinishStatus(studyRecd.getCourseId(), studyRecd.getAccountId(), durationTmp==null?0:durationTmp.longValue(), true, studyRecd.getSiteId());
            saveSendRQRecode("一般课程完成同步课程完成状态", studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());

            // 可以发送项目完成状态
            /*sendFinishStatus(studyRecd.getCourseId(), studyRecd.getAccountId(), studyRecd.getDurationTmp().longValue(), true, studyRecd.getSiteId());
            saveSendRQRecode("一般课程完成同步课程完成状态", studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());*/

            LOGGER.info("证书颁发状态={}",course.getGrantCertificate());
            if(course.getGrantCertificate()){ // 如果需要颁发证书
                // 发放课程证书
                LOGGER.info("开始发送证书，证书类表={}",course.getCertificates());
                sendCertificate(course.getId(),course.getName(),studyRecd.getAccountId(),studyRecd.getSiteId(),course.getCertificates());
                saveSendRQRecode("一般课程发放证书", studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());
                //String[] certificates = course.getCertificates().split(",");
                /*for(String certificateId : certificates){
                    sendCertificate(course.getId(),course.getName(),studyRecd.getAccountId(),studyRecd.getSiteId(),Long.valueOf(certificateId));
                }*/
            }

            // 发送课程学时
            Float totalDuration = course.getTotalDuration();
            // 学时发放学时标识逻辑为发送，并设在的学时数据不等于null以及后台配置的课程学时发放开关为打开
            if (Objects.nonNull(totalDuration) && Boolean.TRUE.equals(course.getTotalDurationGrant())) {
                // 判断是否是已经已经发放学时的
                String keyWord = studyRecd.getSessionId();
                keyWord = keyWord==null?"":keyWord;
                EntityWrapper<CourseStudyHourDetailed> ewDetai = QueryUtil.condition(new CourseStudyHourDetailed());
                ewDetai.eq("account_id",studyRecd.getAccountId()).eq("course_id",course.getId())
                        .eq("key_word",keyWord);
                int countNum = studyHourDetailedService.selectCount(ewDetai);
                if(countNum == 0){
                    CourseStudyHour hourEntiy = new CourseStudyHour();
                    hourEntiy.setTotalDuration(course.getTotalDuration());
                    hourEntiy.setAccountId(studyRecd.getAccountId());
                    hourEntiy.setCompanyId(studyRecd.getCompanyId());
                    hourEntiy.setCourseId(studyRecd.getCourseId());
                    hourEntiy.setSiteId(studyRecd.getSiteId());
                    hourEntiy.setOrgId(studyRecd.getOrgId());
                    LOGGER.info("满足学时发放要求，准备发放学时，课程学时对象={}", JSON.toJSONString(hourEntiy));
                    this.addCourseStudyHour(hourEntiy);
                }
            }
        }else {

            if(studentStatus.getStudentStatus() == null || Integer.valueOf(0).equals(studentStatus.getStudentStatus())){
                studentStatus.setStudentStatus(0);
                studentStatus.setPercent((int)(percentDouble*100));
            }
            if(!extisflag){
                // 学习未完成发送培训项目状态
                Integer durationTmp = studyRecd.getDurationTmp();
                sendFinishStatus(studyRecd.getCourseId(), studyRecd.getAccountId(),
                        durationTmp==null?0:durationTmp.longValue(), false, studyRecd.getSiteId());
                saveSendRQRecode("一般课程未完成同步课程完成状态,课程完成状态"+false, studyRecd.getCourseId(),studyRecd.getAccountId(), studyRecd.getChapterId());
            }
        }

        // 保存课程学习状态
        studentStatusService.insertOrUpdate(studentStatus);

        return true;
    }


    public Boolean calculateStudyStatus(Recorde studyRecd) {
        LOGGER.info("课程学习状态处理开始。发放积分处理开始。请求参数={}", JSON.toJSONString(studyRecd));

        // 课程学习记录不存在的直接返回
        if (Objects.isNull(studyRecd)) {
            return true;
        }

        // 获取sessionId 关联的数据
        String sessionIdQuery = studyRecd.getSessionId();
        boolean sessionIdDBOvered = false; // 如果存在sessionId，查询该次sessionId的学习完成状态（true表示已经学习完成）
        if(StringUtils.isNotBlank(sessionIdQuery)){
            Recorde recordeSession = new Recorde();
            recordeSession.setAccountId(studyRecd.getAccountId());
            recordeSession.setChapterId(studyRecd.getChapterId());
            recordeSession.setSessionId(sessionIdQuery);
            recordeSession = recordeService.selectOne(QueryUtil.condition(recordeSession));
            if(recordeSession != null && Integer.valueOf(1).equals(recordeSession.getOvered())){
                sessionIdDBOvered = true;
            }
        }
        if(sessionIdDBOvered){
            return true;
        }

        Integer durationTmp = studyRecd.getDurationTmp();
        if (Objects.isNull(durationTmp)) {
            studyRecd.setDurationTmp(0);
        }
        // 章节id
        Long chapterId = studyRecd.getChapterId();
        // 课程id
        Long courseId = studyRecd.getCourseId();
        // 查询课程信息
        Course course = courseService.selectById(courseId);

        LOGGER.info("课程学习状态处理查询的课程信息= {}", JSON.toJSONString(course));
        // 积分发放规则  1按课程  2 按章 3 按节
        int portRule = course.getPortRule();
        // 学习通过，准备计算是否需要发放积分和项目课程的完成状态
        Integer overed = studyRecd.getOvered(); // 当前学习状态
        if (overed == null) {
            overed = 0;
        }

        // 查询本人学习的章节状态
        List<Chapter> chapterStudyStatus = chapterService.selectCourseFinishChapter(courseId, studyRecd.getAccountId(), studyRecd.getSiteId());
        Map<Long, Chapter> chapterMap = chapterStudyStatus.stream().collect(Collectors.toMap(key -> key.getId(), val -> val));

        // 当前章节学习在数据库的数据
        Chapter currStudyChapter = chapterMap.get(chapterId);

        if (portRule == 3 || portRule == 2) {
            Integer score = currStudyChapter.getScore(); // 发放的积分数
            LOGGER.info("当前学习的完成状态overed={},课程章节设定的积分={},当前学习章节数据的历史完成状态{}", overed, score);
            if (overed.intValue() == 1) {// 当前完成的状态为已完成 并且不属于当次学习段的完成
                if (portRule == 3) {// 按章节发放积分
                    // 如果当前课程db不存在完成的记录并且积分大于0设置，就发放积分
                    //if(Objects.isNull(currOveredDb) && score != null && score.intValue() > 0){// 增加历史是否完成发放过积分的判断
                    if (score != null && score.intValue() > 0) {
                        // 按节直接发放积分
                        LOGGER.info("按节完成，准备按节发放积分");
                        String mqSendStatu = sendPoint(course, score, studyRecd, currStudyChapter, COURSE_MQ_SEND_SCORE);
                        saveSendRQRecode("一般课程按节完成发放积分"+"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
                    }
                    // 给经验发放发送消息
                    LOGGER.info("课程完成状态处理完成，准备配合积分发送消息");
                    String mqSendStatu = sendPoint(course, 0, studyRecd, currStudyChapter, EXPERIENCE_COURSE);
                    saveSendRQRecode("一般课程按节完成发放经验" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
                } else {// 安装章发放
                    Long parentId = currStudyChapter.getParentId();
                    // 如果当前学习的对象是章并且积分大于0
                    if (Objects.isNull(parentId) || parentId.longValue() == 0 ) {
                        if(score != null && score.intValue() > 0){
                            // 对于章直接放内容
                            LOGGER.info("按章完成，准备按章发放积分");
                            String mqSendStatu = sendPoint(course, score, studyRecd, currStudyChapter, COURSE_MQ_SEND_SCORE);
                            saveSendRQRecode("一般课程按章完成发放积分" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
                        }

                        // 给经验发放发送消息
                        LOGGER.info("课程完成状态处理完成，准备配合积分发送消息");
                        String mqSendStatu = sendPoint(course, 0, studyRecd, currStudyChapter, EXPERIENCE_COURSE);
                        saveSendRQRecode("一般课程按章完成发放经验" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);

                    } else {
                        List<Chapter> chapterG = chapterStudyStatus.stream().
                                filter(item -> item.getParentId().equals(parentId)).collect(Collectors.toList());
                        int sumScore = 0;
                        boolean finishFlg = true;
                        for (Chapter studyItem : chapterG) {
                            Long chapterIdDB = studyItem.getId();
                            Integer overedDB = studyItem.getOvered();
                            Integer dbScore = studyItem.getScore();
                            if (Objects.nonNull(dbScore)) {
                                sumScore += dbScore.intValue();
                            }
                            // 如果该章下有未完成的节（并且和当次的节的学习的id不一致，说明该章该学员的学习状态就算未全部完成
                            if (Objects.isNull(overedDB) || overedDB.intValue() != 1) {
                                if (!chapterIdDB.equals(chapterId)) {
                                    finishFlg = false;
                                    break;
                                }
                            }
                        }
                        // 按章发放积分
                        //if(finishFlg && sumScore>0 && Objects.isNull(currOveredDb)){ // 增加历史是否完成发放过积分的判断
                        if (finishFlg ) {
                            if(sumScore > 0){
                                LOGGER.info("按章完成，准备按章发放积分");
                                Chapter studyChapterGp = chapterMap.get(parentId);
                                String mqSendStatu = sendPoint(course, sumScore, studyRecd, studyChapterGp, COURSE_MQ_SEND_SCORE);
                                saveSendRQRecode("一般课程按章完成发放积分" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
                            }
                            // 给经验发放发送消息
                            LOGGER.info("课程完成状态处理完成，准备配合积分发送消息");
                            String mqSendStatu = sendPoint(course, 0, studyRecd, currStudyChapter, EXPERIENCE_COURSE);
                            saveSendRQRecode("一般课程按章完成发放经验" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
                        }
                    }
                }
            }
        }
        // 课程全部的积分
        int sumScoreAll = 0;
        // 是否全部完成
        boolean finishFlgAll = false;
        // 要学习的章节id
        List<Long> stuChapters = new ArrayList<>();
        // 全部完成的章节id
        Set<Long> finishChapters = new HashSet<>();
        // 当前章节是否完成
        if (overed == 1) {
            finishChapters.add(chapterId);
        }
        // 查看课程是否全部学习完成
        for (Chapter allIitem : chapterStudyStatus) {
            Long chapterIdDB = allIitem.getId();
            Integer overedDB = allIitem.getOvered();
            Integer dbScore = allIitem.getScore();
            Long materialIdDb = allIitem.getMaterialId();
            // 只找素材id存在的章节（这些章节的内容才需要学习）
            if (Objects.nonNull(materialIdDb) && materialIdDb.longValue()>0) {
                stuChapters.add(chapterIdDB); // 要学习的章节id
                // 累加课程积分
                if (Objects.nonNull(dbScore)) {
                    sumScoreAll += dbScore.intValue();
                }
                // 已完成的课程章节
                if (Integer.valueOf(1).equals(overedDB)) {
                    finishChapters.add(chapterIdDB);
                }
            }
        }
        List<Long> finishChaptersList = new ArrayList<>(finishChapters);
        double percent = 0;
        LOGGER.info("课程的要学习章节和完成的章节信息，stuChapters={},finishChapters={}", stuChapters, finishChaptersList);
        if (!CollectionUtils.isEmpty(finishChaptersList)) {
            //Collections.sort(finishChaptersList);
            int finishChaptersListSize = finishChaptersList.size();
            int stuChaptersSize = stuChapters.size();
            // 如果学习的章节数和完成数一致，说明学习完成
            if (finishChaptersListSize == stuChaptersSize) {
                finishFlgAll = true;
                percent = 1;
            } else {
                percent = BigDecimal.valueOf(finishChaptersListSize).
                        divide(BigDecimal.valueOf(stuChaptersSize), 3, BigDecimal.ROUND_HALF_UP)
                        .doubleValue();
            }
            LOGGER.info("percent={}",percent);
        }
        // 标识课程是否已经学习完成(标记课程学习状态表是否已完成状态（true 表示该课程该学员已经学完）)
        // 查询课程学习状态
        CourseStudentStatus studentStatusEw = new CourseStudentStatus();
        studentStatusEw.setAccountId(studyRecd.getAccountId());
        studentStatusEw.setCourseId(studyRecd.getCourseId());
        boolean updateFlg = true;
        CourseStudentStatus studentStatus = studentStatusService.selectOne(QueryUtil.condition(studentStatusEw));
        if (studentStatus == null) {
            updateFlg = false;
            studentStatus = new CourseStudentStatus();
            studentStatus.setId(idGenerator.generate());
            studentStatus.setCourseId(studyRecd.getCourseId());
            studentStatus.setAccountId(studyRecd.getAccountId());
            studentStatus.setCompanyId(studyRecd.getCompanyId());
            studentStatus.setSiteId(studyRecd.getSiteId());
            studentStatus.setOrgId(studyRecd.getOrgId());
        }
        studentStatus.setLastStudentTime(studyRecd.getEndTime());

        // 课程是否全部学完
        if (finishFlgAll) {
            LOGGER.info("课程是否全部学习完成？finishFlgAll={}", finishFlgAll);
            if(Objects.isNull(studentStatus.getFirstFinishTime())){
                studentStatus.setFirstFinishTime(new Date());
            }
            studentStatus.setStudentStatus(1);
            studentStatus.setPercent(100);
//                if(!sessionIdDBOvered && studentStatusFlg){
//                    //课程完成可以发送消息(完成课程，触发通知）
//
//                    taskExecutor.asynExecute(new AbstractTaskHandler() {
//                        @Override
//                        public void handle() {
//                            courseEvenSendMessage.evenSendMessage(course,studyRecd.getAccountId(),EvenType.COURSE_FINISH);
//                        }
//                    });
//                }

            // 可以发送项目完成状态
            sendFinishStatus(studyRecd.getCourseId(), studyRecd.getAccountId(), studyRecd.getDurationTmp().longValue(), true, studyRecd.getSiteId());
            saveSendRQRecode("一般课程完成同步课程完成状态", studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
            LOGGER.info("证书颁发状态={}",course.getGrantCertificate());
            if(course.getGrantCertificate()){ // 如果需要颁发证书
                // 发放课程证书
                LOGGER.info("开始发送证书，证书类表={}",course.getCertificates());
                sendCertificate(course.getId(),course.getName(),studyRecd.getAccountId(),studyRecd.getSiteId(),course.getCertificates());
                saveSendRQRecode("一般课程发放证书", studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
                //String[] certificates = course.getCertificates().split(",");
                /*for(String certificateId : certificates){
                    sendCertificate(course.getId(),course.getName(),studyRecd.getAccountId(),studyRecd.getSiteId(),Long.valueOf(certificateId));
                }*/
            }
            // 发送课程学时
            Float totalDuration = course.getTotalDuration();
            // 学时发放学时标识逻辑为发送，并设在的学时数据不等于null以及后台配置的课程学时发放开关为打开
            if (Objects.nonNull(totalDuration) && Boolean.TRUE.equals(course.getTotalDurationGrant())) {
                CourseStudyHour hourEntiy = new CourseStudyHour();
                hourEntiy.setTotalDuration(course.getTotalDuration());
                hourEntiy.setAccountId(studyRecd.getAccountId());
                hourEntiy.setCompanyId(studyRecd.getCompanyId());
                hourEntiy.setCourseId(studyRecd.getCourseId());
                hourEntiy.setSiteId(studyRecd.getSiteId());
                hourEntiy.setOrgId(studyRecd.getOrgId());
                LOGGER.info("满足学时发放要求，准备发放学时，课程学时对象={}", JSON.toJSONString(hourEntiy));
                this.addCourseStudyHour(hourEntiy);
            }

            if (portRule == 1 ) { // 增加历史是否完成发放过积分的判断
                if(sumScoreAll > 0){
                    LOGGER.info("按课程完成，准备按课程发放积分");
                    String mqSendStatu =  sendPoint(course, sumScoreAll, studyRecd, currStudyChapter, COURSE_MQ_SEND_SCORE);
                    saveSendRQRecode("一般课程按课程完成发放积分" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
                }
                // 给经验发放发送消息
                LOGGER.info("课程完成状态处理完成，准备配合积分发送消息");
                String mqSendStatu =  sendPoint(course, 0, studyRecd, currStudyChapter, EXPERIENCE_COURSE);
                saveSendRQRecode("一般课程按课程完成发放经验" +"-"+mqSendStatu, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
            }

        } else {
            boolean finishStatus = false;
            Integer studentStatusOld = studentStatus.getStudentStatus();
            if(Integer.valueOf(1).equals(studentStatusOld)){
                finishStatus = true;
            }
            if(studentStatusOld == null || studentStatusOld.intValue()==0){
                studentStatus.setStudentStatus(0);
                studentStatus.setPercent((int) (percent * 100));
            }
            // 学习未完成发送培训项目状态
            sendFinishStatus(studyRecd.getCourseId(), studyRecd.getAccountId(),
                    studyRecd.getDurationTmp().longValue(), finishStatus, studyRecd.getSiteId());
            saveSendRQRecode("一般课程未完成同步课程完成状态,课程完成状态"+finishStatus, studyRecd.getCourseId(),studyRecd.getAccountId(), chapterId);
        }

        // 保存课程学习状态
        if(updateFlg){
            studentStatusService.updateById(studentStatus);
        }else{
            studentStatusService.insert(studentStatus);
        }

        return true;
    }

    /**
     * 发送课程学时
     * sql查询重复学时：select * FROM course_study_hour GROUP BY account_id,course_id,create_time HAVING COUNT(0)>1
     * 增加重复限制：ALTER TABLE course_study_hour ADD CONSTRAINT uq_dataUnique UNIQUE (account_id,course_id,create_time)
     * @param entiy
     * @return
     */
    public Boolean addCourseStudyHour(CourseStudyHour entiy) {
        Float totalDuration = entiy.getTotalDuration();

        if (Objects.nonNull(totalDuration)) {
            Float hour = null;

            if (totalDuration > 0f) {
                hour = studyHourService.studyHourSum(entiy.getSiteId(),
                        entiy.getAccountId(), entiy.getCourseId());
                LOGGER.info("课程学时大于0，去数据库查询之前发放的课程总学时={}", hour);
            }
            try {
                if (hour == null) {
                    entiy.setId(idGenerator.generate());
                    entiy.setCreateTime(new Date());
                    studyHourService.insert(entiy);
                    LOGGER.info("课程学时大于0，还不存在课程总学时，进行插入新的学时={}", JSON.toJSONString(entiy));
                } else if (totalDuration > hour) {
                    entiy.setId(idGenerator.generate());
                    entiy.setTotalDuration(totalDuration - hour);
                    entiy.setCreateTime(new Date());
                    studyHourService.insert(entiy);
                    LOGGER.info("课程学时大于0，发放的总学时小于新发放的学时，进行插入新的学时={}", JSON.toJSONString(entiy));
                }
            } catch (Exception e) {
                LOGGER.info("课程插入失败，忽略插入{}", e);
            }
            // 插入学时明细
            CourseStudyHourDetailed entiyDetailed = new CourseStudyHourDetailed();
            BeanUtils.copyProperties(entiy, entiyDetailed, "id", "createTime");
            entiyDetailed.setId(idGenerator.generate());
            entiyDetailed.setCreateTime(new Date());
            LOGGER.info("准备插入新的学时明细信息={}", entiyDetailed.getId());
            studyHourDetailedService.insert(entiyDetailed);
        }

        return true;
    }

    /**
     * 发送证书
     * @param courseId 课程id
     * @param courseName 课程名
     * @param accountId 用户id
     * @param siteId 站点id
     * @param certificateId 证书id
     */
    private void sendCertificate(Long courseId, String courseName, Long accountId, Long siteId, String certificateId) {

        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                Map<String, Object> map = new HashMap<>();
                map.put("siteId", siteId);
                map.put("accountId", accountId);
                map.put("courseId", courseId);
                map.put("courseName", courseName);
                map.put("bizdType", 1);
                try {
                    String[] certificates = certificateId.split(",");
                    for(String certificateIdItem : certificates){
                        //sendCertificate(courseId,courseName,accountId,siteId,Long.valueOf(certificateId));
                        long bid =  Long.valueOf(certificateIdItem);
                        map.put("certificateId", bid);
                        cloudEventPublisher.publish("myCertificate", new EventWrapper<Map>(bid, map));
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }
    /**
     * 给项目发送课程完成消息
     */
    private void sendFinishStatus(Long courseId, Long accountId, Long duration, boolean finishStatus, Long siteId) {
        Date sendDate = new Date();
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                try {
                    cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE, new EventWrapper<>(
                            courseId, // 课程id
                            TrainingProjectEvent.getInstanceCourse(
                                    courseId, // 课程id
                                    accountId, // 账号id
                                    sendDate,
                                    duration,//学习时长
                                    finishStatus,//完成状态
                                    siteId//站点id
                            )
                    ));
                    cloudEventPublisher.publish(QueueConstant.ALBUM_EVENT_QUEUE, new EventWrapper<>(
                            courseId, // 课程id
                            TrainingProjectEvent.getInstanceCourse(
                                    courseId, // 课程id
                                    accountId, // 账号id
                                    sendDate,
                                    duration,//学习时长
                                    finishStatus,//完成状态
                                    siteId//站点id
                            )
                    ));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * 发放积分
     *
     * @param course  课程信息
     * @param score   发放的积分数
     * @param recorde 学习记录
     * @param chapter 章节信息
     */
    private String sendPoint(Course course, int score, Recorde recorde, Chapter chapter, String mqKey) {
        if (score == 0 && !EXPERIENCE_COURSE.equals(mqKey)) {
            LOGGER.info("正常学习和心跳处理操作准备发放积分时，发现积分为0，积分发放准备停止");
            return "发放的积分为空";
        }
        //学习活动名
        StringBuilder activityName = new StringBuilder();
        try {
            PointParamVO pointParamVO = new PointParamVO();
            pointParamVO.setPortRule(course.getPortRule());
            //事件名称
            pointParamVO.setEventName(EVENT_NAME);//PointEventEnum.POINTCOURSE
            //事件类型
            pointParamVO.setActivityType("课程学习完成积分发放");
            //学习活动来源
            pointParamVO.setActivitySource("课程学习");


            activityName.append(course.getName());

            //积分创建时间
            pointParamVO.setCreatePointTime(new Date());

            // 积分发放规则  1按课程  2 按章 3 按节
            int portRule = course.getPortRule();

            // 课程id
            pointParamVO.setCourseId(course.getId());

            //积分来源ID，（业务ID主键）
            pointParamVO.setSourceId(course.getId());

            //发放条件
            pointParamVO.setActivityType("课程学习完成积分发放-按课程发放");
            //发放规则
            pointParamVO.setReleaseRules("按课程发放");
            if (portRule == 3) {
                pointParamVO.setReleaseRules("按节发放");
                //发放条件
                pointParamVO.setActivityType("课程学习完成积分发放-按节发放");

                activityName.append(":");
                activityName.append(chapter.getName());
                //积分来源ID，（业务ID主键）
                pointParamVO.setSourceId(chapter.getId());

            } else if (portRule == 2) {
                pointParamVO.setActivityType("课程学习完成积分发放-按章发放");
                pointParamVO.setReleaseRules("按章发放");

                activityName.append(":");
                activityName.append(chapter.getName());
                //积分来源ID，（业务ID主键）
                pointParamVO.setSourceId(chapter.getId());
            }

            pointParamVO.setActivityName(activityName.toString());

            //积分数量，  正数 加积分，  负数减积分
            pointParamVO.setOperatingPoint(score);

            //学员ID
            pointParamVO.setAccountId(recorde.getAccountId());
            //学员名称
            //pointParamVO.setAccountName(accountName);
            pointParamVO.setSiteId(recorde.getSiteId());
            pointParamVO.setCompanyId(recorde.getCompanyId());
            pointParamVO.setOrgId(recorde.getOrgId());

            pointParamVO.setPortRule(portRule);

            //调用积分接口添加积分
            String sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);
            LOGGER.info("准备发送积分，设置{}对象到redis，sitePointId={}", pointParamVO, sitePointId);
            if (null != sitePointId) {
                //发送积分获取消息
                //amqpTemplate.convertAndSend(mqKey, sitePointId); //web端 定义 MQqueuesEnum.COURSE.
                // 配合小伟的经验发放
                /*String sitePointId1 = pointRedisFeignClients.addPointRedis(pointParamVO);
                LOGGER.info("积分发放完毕，准备发送经验值，设置{}对象到redis，sitePointId={}",pointParamVO,sitePointId);
                amqpTemplate.convertAndSend(EXPERIENCE_COURSE, sitePointId1);
                LOGGER.info("积分和经验消息队列发放成功");*/
                return "消息发送成功";
            }
            return "消息没发送，sitePointId为空";
        } catch (Exception e) {
            LOGGER.error("积分活动名为：" + activityName.toString() + "的积分消息发送失败！");
            return "消息发送异常:"+e.getMessage();
        }

    }

    /**
     * 获取学时统计列表
     *
     * @param userName
     * @param orgNameLike
     * @param pageNo
     * @param pageSize
     * @return
     * @author ding
     */
    @GetMapping("/report/list")
    Page<StudyHourReportVO> getStudyHourList(@RequestParam(value = "userName", required = false) String userName,
                                             @RequestParam(value = "orgNameLike", required = false) String orgNameLike,
                                             @RequestParam("pageNo") Integer pageNo,
                                             @RequestParam("pageSize") Integer pageSize) {
        RequestContext res = ContextHolder.get();
        Long companyId = res.getCompanyId();
        Long siteId = res.getSiteId();
        boolean isAdmin = res.isAdmin();
        Long accountId = res.getAccountId();
        return studyHourService.getStudyHourList(companyId, siteId, isAdmin, accountId, userName, orgNameLike, pageNo, pageSize);
    }

    /**
     * 获取学时统计列表下载
     *
     * @param userName
     * @param orgNameLike
     * @return
     * @author ding
     */
    @GetMapping("/report/listDownload")
    String getStudyHourListDownload(@RequestParam(value = "userName", required = false) String userName,
                                    @RequestParam(value = "orgNameLike", required = false) String orgNameLike) {
        RequestContext res = ContextHolder.get();
        Map<String, Object> map = new HashMap<>();
        Long taskId = System.currentTimeMillis();
        String serialNo = "STUDY-HOUR-REPORT-EXPORT-" + taskId.toString();
        String taskName = "学时统计下载-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        map.put("userName", userName);
        map.put("orgNameLike", orgNameLike);
        studyHourReportExport.execute(map, true);
        return serialNo;
    }

    /**
     * 学时明细下载
     *
     * @param userName
     * @param orgNameLike
     * @return
     */
    @GetMapping("/details/download")
    public String studyHourDetailsDownload(
            @RequestParam(value = "userName", required = false) String userName,
            @RequestParam(value = "orgNameLike", required = false) String orgNameLike){
        RequestContext res = ContextHolder.get();
        Map<String, Object> map = new HashMap<>();
        Long taskId = System.currentTimeMillis();
        String serialNo = "STUDY-HOUR-DETAILS-EXPORT-" + taskId.toString();
        String taskName = "学时明细下载-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        map.put("userName", userName);
        map.put("orgNameLike", orgNameLike);
        studyHourDetailsExport.execute(map, true);
        return serialNo;
    }

    /**
     * 保存发送MQ记录
     * @param envet 时间名称
     * @param courseId 课程id
     * @param accoutId 账户id
     * @param chapterId 章节id
     */
    private void saveSendRQRecode(String envet, Long courseId, Long accoutId, Long chapterId){
        Object isOpen = redisCache.get(IS_OPEN_COURSE_MQ_RECODE);
        if(isOpen != null && "on".equals(isOpen.toString())){
            CourseMqRecord courseMqRecord = new CourseMqRecord();
            courseMqRecord.setId(idGenerator.generate());
            if(envet.length()>250){
                envet = envet.substring(0, 250);
            }
            courseMqRecord.setEnvetType(envet);
            courseMqRecord.setCourseId(courseId);
            courseMqRecord.setAccoutId(accoutId);
            courseMqRecord.setChapterId(chapterId);

            courseMqRecordService.insert(courseMqRecord);
        }
    }
    /**
     * 复旦mini mba项目获取多个培训项目总学习时长
     *
     * @param paramVO
     * @return
     */
    @PostMapping("/projects/totalDuration")
    public Integer getProjectsTotalDuration(@RequestBody ProjectsDurationParamVO paramVO) {
        return scormService.getProjectsTotalDuration(paramVO);
    }

    /**
     * 复旦mini mba项目获取多个个课程的学习总时长
     *
     * @param paramVO
     * @return
     */
    @PostMapping("/course/totalDuration")
    public Map<Long, Map<String, Object>> getCourseTotalDuration(@RequestBody ProjectsDurationParamVO paramVO) {
        return scormService.getCourseTotalDuration(paramVO.getBizIds());
    }

    /**
     * 记录mq发送记录的开关
     * @param key
     * @return
     */
    @GetMapping("course/mq/record/open")
    public Boolean openRedis(@RequestParam("key") String key){
        redisCache.set(IS_OPEN_COURSE_MQ_RECODE, key);
        return true;
    }
}
