package com.yizhi.course.application.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.course.application.domain.Recorde;
import com.yizhi.course.application.service.IChapterService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.vo.ChapterDetailedInfoVO;
import com.yizhi.course.application.vo.HeartBeatVo;
import com.yizhi.course.application.vo.StudyLog;
import com.yizhi.studylog.application.feign.StudyLogClient;
import com.yizhi.studylog.application.vo.StudyLogVo;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 心跳处理控制器
 */
@RestController
@RequestMapping("/heartbeat/handle")
public class HeartbeatHandleController {

    private Logger LOGGER = LoggerFactory.getLogger(HeartbeatHandleController.class);

    @Autowired
    private StudyLogClient studyLogClient;

    @Autowired
    private IRecordeService recordeService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IChapterService chapterService;

    /**
     * 学习日志计算处理
     * @return
     */
    @GetMapping("/calculate")
    public Boolean calculate(@RequestParam("sessionId") String sessionId){
        LOGGER.info("HeartbeatHandleController处理开始");

        HeartBeatVo heartBeatVo = new HeartBeatVo();// todo 心跳参数设置，目前为vo固定，获取根据接口获取


        // 获取当前关于该sessionId的心跳记录(数据以id倒序，最后的时间排在前面)
        List<StudyLogVo> data = studyLogClient.getRecodeBySessionId(null,sessionId);
        if(data == null || data.size()==0){
            return Boolean.TRUE;
        }
        int studycount = data.size();
        // 当前学习💹时长
        int currStudyHour = (studycount*heartBeatVo.getCount());

        // 查询以存在的学习记录
        EntityWrapper<Recorde> ewExist = QueryUtil.condition(new Recorde());
        ewExist.eq("session_id",sessionId).orderBy("duration",false);
        List<Recorde> existRecordes = recordeService.selectList(ewExist);
        Recorde generateRecorde = new Recorde();
        generateRecorde.setDuration(0);
        // 最后学习时间
        StudyLogVo lastStudyLog = data.get(0);
        generateRecorde.setEndTime(new Date(lastStudyLog.getTimestamp()));
        // 完成状态
        generateRecorde.setOvered(lastStudyLog.getMaterialCompleted());
        generateRecorde.setChapterId(lastStudyLog.getChapterId());

        if(CollectionUtils.isNotEmpty(existRecordes)){
            int size = existRecordes.size();
            // 默认第一条作为旧的数据
            Recorde oldRecorde =  existRecordes.get(0);
            generateRecorde.setId(oldRecorde.getId());
            generateRecorde.setOvered(oldRecorde.getOvered());

            generateRecorde.setDuration(oldRecorde.getDuration().intValue() + currStudyHour );
            generateRecorde.setDurationTmp(currStudyHour);


            if(size > 1){ // 否则可能是前端调用完成接口产生了一条，需要计算得出
                Recorde removeRecorde =  existRecordes.get(size-1);
                Integer reOvered = removeRecorde.getOvered();
                if(Objects.nonNull(reOvered) && reOvered.intValue()==1){ // 如果接口调用完成，就直接完成
                    generateRecorde.setOvered(reOvered);
                    Integer removeDuration = removeRecorde.getDuration();
                    generateRecorde.setDuration(generateRecorde.getDuration().intValue()+
                            (removeDuration==null?0:removeDuration.intValue()));
                    generateRecorde.setStartTime(removeRecorde.getStartTime());
                    recordeService.deleteById(removeRecorde.getId());
                }
            }
        }else{
            StudyLogVo first = null;
            if(studycount>1){
                first = data.get(studycount-1);
            }else{
                first = data.get(0);
            }

            generateRecorde.setId(idGenerator.generate());
            generateRecorde.setDuration(currStudyHour );
            generateRecorde.setDurationTmp(currStudyHour);
            generateRecorde.setStartTime(new Date(first.getTimestamp()));//开始时间

            generateRecorde.setCourseId(first.getCourseId());
            generateRecorde.setTerminalDevice(first.getTerminalDevice());
            generateRecorde.setTerminalType(first.getTerminalType());
            generateRecorde.setSessionId(first.getSessionId());
            generateRecorde.setAccountId(Long.valueOf(first.getAccountId()));
            generateRecorde.setOrgId(Long.valueOf(first.getOrgId()));
            generateRecorde.setSiteId(Long.valueOf(first.getSiteId()));
            generateRecorde.setCompanyId(Long.valueOf(first.getCompanyId()));
            generateRecorde.setValid(1);
            generateRecorde.setExitstates(1);

        }

        LOGGER.info("组装完的学习记录={}", JSON.toJSONString(generateRecorde));
        int currOvered = generateRecorde.getOvered();
        // 计算章节的完成
        if(currOvered == 0){
            // 获取章节详细信息ChapterDetailedInfoVO
            ChapterDetailedInfoVO vo =chapterService.getChapterDetailedInfo(generateRecorde.getChapterId());
            LOGGER.info("章节详细信息，包含课程和素材的信息={}",JSON.toJSONString(vo));
            if(Objects.isNull(vo)){
                generateRecorde.setOvered(1);
            }else{
                int overed = 0;
                // 素材类型1音频 2 视频 3 文本 4 富媒体 5 Scrom
                int materialType = vo.getMaterialType();
                switch (materialType) {
                    case 1:
                        int audioComplete = vo.getAudioComplete();//音频完成条件 0 播放即完成 1听完整个课程
                        if (audioComplete == 0) {
                            overed = 1;
                        }
                        break;
                    case 2:
                        int videoComplete = vo.getVideoComplete();//视频完成条件 0 播放即完成 1听完整个课程
                        if (videoComplete == 0) {
                            overed = 1;
                        }
                        break;
                    case 3:
                        int docComplete = vo.getDocComplete();//文档完成条件 0 观看即完成 1停留时长达到学习时长即完成
                        if (docComplete == 0) {
                            overed = 1;
                        }
                        overed = getOvered(vo, generateRecorde.getAccountId());
                        break;
                    case 4:
                        int richComplete = vo.getRichComplete();//多媒体完成条件 0 观看即完成 1停留时长达到学习时长即完成
                        overed = getOvered(vo, generateRecorde.getAccountId());
                        break;
                    default:
                }
            }
        }

        return Boolean.TRUE;
    }

    private int getOvered(ChapterDetailedInfoVO vo, long accountId) {
        int complete = 1;
/*        if(complete==0){
            return  1;
        }else{
            // 查询之前学习的时长
            //LOG.info("查询历史时长参数为accountId={},siteId={},chapterId="+chapter.getId(),accountId,siteId);
            Long countDuration = recordeClient.countStudyDuration(accountId,siteId,chapter.getId());
            if(Objects.nonNull(countDuration)){
                duration += countDuration.longValue();
            }
            Material material = materialMap.get(chapter.getMaterialId());
            if(material == null){
                return -1;
            }
            int timeLong = 0;
            timeLong = getTimeLong(material, timeLong);
            if(duration>=timeLong){
                overed =1;
            }
        }*/
        return complete;
    }
}
