package com.yizhi.course.application.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.service.IClassifyService;
import com.yizhi.course.application.vo.ClassifyVo;
import com.yizhi.course.application.vo.MarketingClassifyVo;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 营销课程分类 前端控制器
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@RestController
@RequestMapping("/marketing/classify")
public class MarketingClassifyController {
    private Logger LOGGER= LoggerFactory.getLogger(MarketingClassifyController.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IClassifyService iClassifyService;

    //@ApiOperation("添加分类")
    @RequestMapping(value = "/save",method = RequestMethod.POST)
    public MarketingClassifyVo saveClassify(@RequestBody MarketingClassifyVo marketingClassifyVo){
        LOGGER.info("新增分类请求的上下文="+ JSON.toJSONString(ContextHolder.get()));
        LOGGER.info("marketingClassifyVo="+JSON.toJSONString(marketingClassifyVo));
        Classify classify = new Classify();
        marketingClassifyVo.setId(idGenerator.generate());
        BeanUtils.copyProperties(marketingClassifyVo,classify);
        classify.setSiteId(ContextHolder.get().getSiteId());
        classify.setCompanyId(ContextHolder.get().getCompanyId());
        classify.setOrgId(ContextHolder.get().getOrgId());
        classify.setCreateById(ContextHolder.get().getAccountId());
        classify.setCreateByName(ContextHolder.get().getAccountName());
        classify.setCreateTime(new Date());
        classify.setSource(4);
        iClassifyService.insert(classify);
        return marketingClassifyVo;
    }

    //@ApiOperation("分类删除")
    @RequestMapping(value = "/delete",method = RequestMethod.GET)
    public boolean deleteClassify(@RequestParam Long id){
        LOGGER.info("删除分类请求的上下文="+JSON.toJSONString(ContextHolder.get()));
        LOGGER.info("id="+id);
        return iClassifyService.deleteById(id);
    }

    //@ApiOperation("分类名称修改")
    @RequestMapping(value = "/update",method = RequestMethod.POST)
    public boolean updateClassify(@RequestBody MarketingClassifyVo marketingClassifyVo){
        LOGGER.info("更新分类请求的上下文="+JSON.toJSONString(ContextHolder.get()));
        LOGGER.info("marketingClassifyVo="+JSON.toJSONString(marketingClassifyVo));
        Classify classify = new Classify();
        BeanUtils.copyProperties(marketingClassifyVo,classify);
        classify.setOrgId(ContextHolder.get().getOrgId());
        classify.setUpdateById(ContextHolder.get().getAccountId());
        classify.setUpdateByName(ContextHolder.get().getAccountName());
        classify.setUpdateTime(new Date());
        return iClassifyService.updateById(classify);
    }

    //@ApiOperation("分类查询")
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    public List<ClassifyVo> listClassify(){
        RequestContext requestContext = ContextHolder.get();
        LOGGER.info("查询全部营销分类请求的上下文="+JSON.toJSONString(requestContext));

        Classify classify = new Classify();
        classify.setSource(4);
        classify.setCompanyId(requestContext.getCompanyId());

        Long siteId = requestContext.getSiteId();
        if(siteId != null && siteId.longValue() != 0){
            classify.setSiteId(siteId);
        }
        EntityWrapper<Classify> ew = QueryUtil.condition(classify);
        // 数据过滤
        List<Long> relationIds = requestContext.getRelationIds();
        if(relationIds != null && relationIds.size()>0){
            // 学员端待过滤
        } else if (!requestContext.isAdmin()) {
            /*List<Long> orgIds = requestContext.getOrgIds();
            if(org.apache.commons.collections.CollectionUtils.isNotEmpty(orgIds)){
                ew.and().in("org_id", orgIds);
            }else {
                ew.and().eq("org_id", -1L);
            }*/
            // 分级授权修改
            HQueryUtil.startHQ(Classify.class);
        }
        List<Classify> queryList = iClassifyService.selectList(ew);
        List<ClassifyVo> retList = new ArrayList<ClassifyVo>();
        if(! CollectionUtils.isEmpty(queryList)){
            ClassifyVo classifyVo = null;
            for(Classify obj : queryList){
                classifyVo = new ClassifyVo();
                BeanUtils.copyProperties(obj,classifyVo);
                classifyVo.setDescription(obj.getDescription());
                retList.add(classifyVo);
            }
        }
        return  retList;
    }

    @RequestMapping(value = "/view",method = RequestMethod.GET)
    public ClassifyVo viewClassify(@RequestParam Long id){
        LOGGER.info("更新分类请求的上下文="+JSON.toJSONString(ContextHolder.get()));
        LOGGER.info("id="+id);
        Classify classify = new Classify();
        classify.setSource(4);
        classify.setId(id);
        ClassifyVo classifyVo = new ClassifyVo();
        try {
            classify = iClassifyService.selectOne(QueryUtil.condition(classify));
            BeanUtils.copyProperties(classify,classifyVo);
        } catch (Exception e) {
            return null;
        }
        return classifyVo;
    }

    @RequestMapping(value = "/name/get",method = RequestMethod.GET)
    public ClassifyVo getByName(@RequestParam(value = "name") String name,@RequestParam("siteId") Long siteId,
                                @RequestParam(value = "classifyId",required = false) Long classifyId){
        LOGGER.info("根据名称查询分类请求的上下文="+JSON.toJSONString(ContextHolder.get()));
        LOGGER.info("name="+name);
        Classify classify = new Classify();
        classify.setSource(4);
        classify.setName(name);
        classify.setSiteId(siteId);
        ClassifyVo classifyVo = new ClassifyVo();
        EntityWrapper<Classify> ew = QueryUtil.condition(classify);
        if(classifyId != null){
            ew.ne("id",classifyId);
        }
        List<Classify> retList = iClassifyService.selectList(ew);
        if(CollectionUtils.isEmpty(retList)){
            return  null;
        }else{
            BeanUtils.copyProperties(retList.get(0),classifyVo);
        }

        return classifyVo;
    }

}

