package com.yizhi.course.application.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.enums.SqlLike;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.page.PageUtil;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.*;
import com.yizhi.course.application.service.*;
import com.yizhi.course.application.vo.*;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.util.application.page.PageInfo;
import com.yizhi.util.application.page.PageParam;
import com.yizhi.wechat.application.feign.WeiXinClient;
import com.yizhi.wechat.application.vo.wechat.domain.TrAccountUserinfoVo;
import com.yizhi.wechat.application.vo.wechat.domain.UserInfoVo;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import com.yizhi.system.application.vo.AccountVO;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: XieHaijun
 * @Description: 营销课程
 * @Date: Created in 10:26 2018/4/8
 */
@RestController
@RequestMapping("/marketing/course")
public class MarketingCourseController {

    private Logger LOGGER = LoggerFactory.getLogger(MarketingCourseController.class);

    @Autowired
    private ICourseService courseService;

    @Autowired
    private IChapterService iChapterService;

    @Autowired
    private ICourseAccountService iCourseAccountService;

    @Autowired
    private ICourseShareService iCourseShareService;

    @Autowired
    private IBrowseRecordsService iBrowseRecordsService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IClassifyService iClassifyService;

    @Autowired
    private IMaterialService materialService;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private IMarketingCourseViewLogService marketingCourseViewLogService;

    @Autowired
    private IWechatInfoService wechatInfoService;

    @Autowired
    private ICourseDescriptionService courseDescriptionService;

    @Autowired
    private WeiXinClient weiXinClient;

    /**
     * 新增营销课程
     *
     * @param marketingCourseVo
     * @return
     */
    @PostMapping("/save")
    @Transactional(rollbackFor = RuntimeException.class)
    public MarketingCourseVo save(@RequestBody MarketingCourseVo marketingCourseVo) {
        Course course = new Course();
        course.setId(idGenerator.generate());
        course.setMarketingCourseType(marketingCourseVo.getMarketingCourseType());
        course.setSource(4);
        course.setDelFlg(1);
        course.setShelves(0);
        RequestContext rc = ContextHolder.get();
        course.setSiteId(rc.getSiteId());
        course.setCompanyId(rc.getCompanyId());
        course.setOrgId(rc.getOrgId());
        course.setCreateById(rc.getAccountId());
        course.setCreateByName(rc.getAccountName());
        //course.setDelFlg(0);
        course.setCreateTime(new Date());
        boolean saveStatus = courseService.insert(course);
        if (saveStatus) {
            marketingCourseVo.setId(course.getId());
        }
        return marketingCourseVo;
    }

    @GetMapping("/extis/chapte")
    public boolean extisChapte(@RequestParam("courseId") Long courseId) {
        // 查询关联的素材
        Chapter chapter = new Chapter();
        chapter.setCourseId(courseId);
        return iChapterService.selectCount(QueryUtil.condition(chapter)) > 0;
    }

    /**
     * 更新营销课程信息
     *
     * @param marketingCourseVo
     * @return
     */
    @PostMapping("/update")
    @Transactional(rollbackFor = RuntimeException.class)
    public boolean update(@RequestBody MarketingCourseVo marketingCourseVo) {

        LOGGER.info("更新营销课程的信息={}", JSON.toJSONString(marketingCourseVo));
        // 更新时课程ID不能为空
        if (marketingCourseVo.getId() != null) {
            Course course = new Course();
            RequestContext rc = ContextHolder.get();
            BeanUtils.copyProperties(marketingCourseVo, course);
            //course.setOrgId(ContextHolder.get().getOrgId());
            course.setUpdateById(rc.getAccountId());
            course.setUpdateByName(rc.getAccountName());
            course.setUpdateTime(new Date());
            course.setDelFlg(0);
            boolean courseUpFlg = courseService.updateById(course);
            LOGGER.info("主体信息更新状态={}", courseUpFlg);
            if (courseUpFlg) {
                // 更新指定人
                Integer scope = marketingCourseVo.getScope();
                if (Objects.nonNull(scope)) { // 如果scope不为空，说明要进行用户可见范围更新
                    LOGGER.info("更新可见范围");
                    // 先查询数据库配置的用户可见范围信息
                    CourseAccount courseAccountDel = new CourseAccount();
                    courseAccountDel.setCourseId(marketingCourseVo.getId());
                    iCourseAccountService.delete(new EntityWrapper<>(courseAccountDel));
//                    if(!CollectionUtils.isEmpty(delAids)){
////                    List<Long> acIds = delAids.stream().map(obj1->obj1.getId()).collect(Collectors.toList());
////                    iCourseAccountService.deleteBatchIds(acIds);
//                        // mybatisplus 的逻辑删除封装有bug，批量删除报错
//                        /*delAids.forEach(item -> iCourseAccountService.deleteById(item.getId()));*/
//                        EntityWrapper<CourseAccount> ew1 = QueryUtil.condition(new CourseAccount());
//                        ew1.in("id", delAids);
//                        iCourseAccountService.delete(ew1);
//                    }
                    List<CourseAccountVO> accountIds = marketingCourseVo.getAccountIds();
                    if (scope.byteValue() == 2 && !CollectionUtils.isEmpty(accountIds)) {
                        addAccount(marketingCourseVo, accountIds);
                    }

                }
                Integer marketingCourseType = marketingCourseVo.getMarketingCourseType();
                LOGGER.info("营销的类型：{}", marketingCourseType);
                if (Integer.valueOf(0).equals(marketingCourseType) || Integer.valueOf(2).equals(marketingCourseType)) {
                    // 更新课程关联的素材信息
                    Chapter chapter = new Chapter();
                    chapter.setCourseId(marketingCourseVo.getId());
                    List<Chapter> delChapters = iChapterService.selectList(QueryUtil.condition(chapter));
                    List<Long> chapterIds = delChapters.stream().map(obj -> obj.getId()).collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(chapterIds)) {
                        iChapterService.deleteBatchIds(chapterIds);
                        //iChapterService.delete(QueryUtil.condition(chapter));
                    }
                    List<ChapterVo> chapters = marketingCourseVo.getChapters();
                    if (!CollectionUtils.isEmpty(chapters)) {
                        iChapterService.saveList(chapters, marketingCourseVo.getId());
                    }

                    LOGGER.info("更新完关联章节，准备更新直播类型的简介");
                    // 营销课程的直播简介字段更新到课程简介
                    String liveDescription = marketingCourseVo.getLiveDescription();
                    // 如果存在课程简介信息，执行简介大字段表保存
                    if (Integer.valueOf(2).equals(marketingCourseType)) {
                        EntityWrapper<CourseDescription> whereCd = QueryUtil.condition(new CourseDescription());
                        whereCd.eq("course_id", course.getId());
                        CourseDescription courseDescription = courseDescriptionService.selectOne(whereCd);
                        if (courseDescription == null) {
                            courseDescription = new CourseDescription();
                            courseDescription.setId(idGenerator.generate());
                            courseDescription.setCourseId(course.getId());
                            courseDescription.setDescription(liveDescription);
                            courseDescriptionService.insert(courseDescription);
                        } else if (StringUtils.isNotEmpty(liveDescription)) {
                            courseDescription.setDescription(liveDescription);
                            courseDescriptionService.updateById(courseDescription);
                        }
                    }
                }

            }
            return true;
        }
        return false;
    }


    /**
     * 批量删除用户
     *
     * @param courseIds
     * @return
     */
    @GetMapping("/delete")
    public boolean delete(@RequestParam String courseIds) {
        String[] courseIdArr = courseIds.split(",");
        for (String courseId : courseIdArr) {
            Course course = new Course();
            course.setId(Long.valueOf(courseId));
            course.setDelFlg(1);
            courseService.updateById(course);
        }
        return true;
    }

    /**
     * 上架
     *
     * @param id
     * @return
     */
    @GetMapping("/up")
    public Boolean up(@RequestParam(value = "id") Long id) {
        Course course = new Course();
        course.setShelves(1);
        course.setId(id);
        courseService.updateById(course);
        return Boolean.TRUE;
    }

    /**
     * 下架
     *
     * @param id
     * @return
     */
    @GetMapping("/down")
    public Boolean down(@RequestParam(value = "id") Long id) {
        Course course = new Course();
        course.setShelves(0);
        course.setId(id);
        courseService.updateById(course);
        return Boolean.TRUE;
    }

    @PostMapping("/list")
    public Page<Course> list(@RequestBody QueryCourseRequest queryCourseRequest) {
        CourseVo courseVo = new CourseVo();
        if (StringUtils.isNotEmpty(queryCourseRequest.getNameOrSortNum())) {
            courseVo.setCode(queryCourseRequest.getNameOrSortNum());
            courseVo.setName(queryCourseRequest.getNameOrSortNum());
        }
        if (queryCourseRequest.getClassifyId() != null) {
            courseVo.setClassifyId(queryCourseRequest.getClassifyId());
        }

        Page<Course> page = courseService.list(courseVo, PageUtil.vice(queryCourseRequest.getPageParam()));
        List<Course> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return page;
        }
        List<Long> classifyIds = records.stream().map(Course::getClassifyId).distinct().collect(Collectors.toList());
        List<Classify> classifies = iClassifyService.selectBatchIds(classifyIds);
        Map<Long, String> result1 = classifies.stream().collect(Collectors.toMap(Classify::getId, Classify::getName));

        // 拿到所有课程ID，然后查询课程的浏览人数
        List<Long> courseIds = records.stream().map(Course::getId).collect(Collectors.toList());
        List<Course> readCountList = marketingCourseViewLogService.findCourseViewCountByIds(courseIds);
        Map<Long,Long> readCountMap = readCountList.stream().collect(Collectors.toMap(Course::getId,Course::getReadCount,(k1,k2) -> k2));

        records.forEach(item -> {
            item.setClassifyName(result1.get(item.getClassifyId()));
            Long readCount = readCountMap.get(item.getId());
            if (readCount == null) {
                readCount = 0L;
            }
            item.setReadCount(readCount);
        });
        page.setRecords(records);

        return page;
    }

    @GetMapping("/name/get")
    public MarketingCourseVo getByName(@RequestParam("courseName") String courseName, @RequestParam("siteId") Long siteId,
                                       @RequestParam(value = "courseId", required = false) Long courseId) {
        MarketingCourseVo marketingCourseVo = null;
        Course course = new Course();
        //course.setDelFlg(0);
        course.setSource(4);
        course.setSiteId(siteId);
        course.setName(courseName);
        EntityWrapper<Course> ew = QueryUtil.condition(course);
        if (courseId != null && courseId.longValue() > 0) {
            ew.ne("id", courseId);
        }
        course = courseService.selectOne(ew);
        LOGGER.info("***********************************根据名称查询课程info=" + JSON.toJSONString(course));
        if (course != null) {
            marketingCourseVo = new MarketingCourseVo();
            BeanUtils.copyProperties(course, marketingCourseVo);
        }
        return marketingCourseVo;
    }

    @GetMapping("/code/get")
    public MarketingCourseVo existByCode(@RequestParam("courseCode") String courseCode, @RequestParam("siteId") Long siteId,
                                         @RequestParam(value = "courseId", required = false) Long courseId) {
        MarketingCourseVo marketingCourseVo = null;
        Course course = new Course();
        //course.setDelFlg(0);
        course.setSource(4);
        course.setSiteId(siteId);
        course.setCode(courseCode);
        course.setDelFlg(0);
        EntityWrapper<Course> ew = QueryUtil.condition(course);
        if (courseId != null && courseId.longValue() > 0) {
            ew.ne("id", courseId);
        }
        course = courseService.selectOne(ew);
        LOGGER.info("***********************************根据名称查询课程info=" + JSON.toJSONString(course));
        if (course != null) {
            marketingCourseVo = new MarketingCourseVo();
            BeanUtils.copyProperties(course, marketingCourseVo);
        }
        return marketingCourseVo;
    }

    @GetMapping("/classify/id/get")
    public Integer getByClassifyId(@RequestParam Long classifyId) {
        Course course = new Course();
        course.setDelFlg(0);
        course.setSource(4);
        course.setClassifyId(classifyId);
        List<Course> ret = courseService.selectList(QueryUtil.condition(course));
        return ret == null ? 0 : ret.size();
    }

    /**
     * 课程查看
     *
     * @param courseId  课程id
     * @param shareId   分享id
     * @param accountId 分享人id
     * @return
     */
    @GetMapping("/view")
    public MarketingCourseVo view(@RequestParam Long courseId
            , @RequestParam(required = false, value = "shareId") Long shareId
            , @RequestParam(value = "accountId") Long accountId
            , @RequestParam(required = false, value = "openId") String openId
            , @RequestParam(required = false, value = "manage") Boolean manage
            , @RequestParam(required = false, value = "childShareOpenId") String childShareOpenId) {
        MarketingCourseVo marketingCourseVo = new MarketingCourseVo();

        // 获取上下文用户信息
        RequestContext requestContext = ContextHolder.get();


        // 拼接查询条件 ,查询课程信息
        Course courseWhere = new Course();
        courseWhere.setId(courseId);
        courseWhere.setSiteId(requestContext.getSiteId());
        Course course = courseService.selectOne(QueryUtil.condition(courseWhere));
        if (course == null) {
            return null;
        }
        BeanUtils.copyProperties(course, marketingCourseVo);
        if (manage == null || !manage) {
            if (Objects.nonNull(requestContext.getAccountId()) && requestContext.getAccountId().longValue() > 0) {
                marketingCourseVo.setTelAuthorize(false);
            }
        }

        if (Integer.valueOf(2).equals(marketingCourseVo.getMarketingCourseType())) { // 如果是带直播的营销课程
            EntityWrapper<CourseDescription> whereCd = QueryUtil.condition(new CourseDescription());
            whereCd.eq("course_id", marketingCourseVo.getId());
            CourseDescription courseDescription = courseDescriptionService.selectOne(whereCd);
            if (courseDescription != null) {
                marketingCourseVo.setLiveDescription(courseDescription.getDescription());
            }
        }


        // 查看用户信息是否存在，不存在返回null (获取第一级分享人的信息，该用户必须是平台用户【ps：如果是管理端进来的，就是管理员的id】）
        AccountVO accountVO = accountClient.findCardById(accountId);
        if (course == null || course.getId() == null) {
            return null;
        }
        // 设在分析人的用户信息作为分享名片

        com.yizhi.course.application.vo.AccountVO vo = new com.yizhi.course.application.vo.AccountVO();

        BeanUtils.copyProperties(accountVO, vo);
        marketingCourseVo.setAccountInfo(vo);// 设置名片信息
        marketingCourseVo.setUserCard(accountVO.getId());

        // 上下文的用户id（如果能获取到大于0的值，说明是平台登陆的用户查看）
        Long tkAccountId = requestContext.getAccountId();
        tkAccountId = (tkAccountId == null || tkAccountId.equals(0L) ? -1L : tkAccountId);

        LOGGER.info("******************课程详情查看的各个参数：courseId={},shareId={},accountId={},openId={}，manage={},childShareOpenId={},tkAccountId={}",
                courseId, shareId, accountId, openId, manage, childShareOpenId, tkAccountId);
        // 对于
        // 如果上下文获取的用户id不等于url请求的账号id，说明不是营销课程管理课程查看（增加营销观看记录）
        if (shareId != null && !tkAccountId.equals(accountId)) {

            iBrowseRecordsService.addReadRecodes(
                    courseId, shareId,
                    accountId, openId, marketingCourseVo,
                    course, vo, tkAccountId, childShareOpenId);
            if (marketingCourseVo.getTelAuthorize() && StringUtils.isNotBlank(openId)) {
                LOGGER.info("需要手机号授权");
                TrAccountUserinfoVo trAccountUserinfo = new TrAccountUserinfoVo();
                Long cid = requestContext.getCompanyId();
                if (cid != null && cid.longValue() > 0) {
                    trAccountUserinfo.setCompanyId(cid);
                }
                trAccountUserinfo.setWechatuserid(openId);
                trAccountUserinfo = weiXinClient.getAccountUserInfo(trAccountUserinfo);
                LOGGER.info("需要手机号授权，查看是否是平台绑定用户{}", JSON.toJSONString(trAccountUserinfo));
                if (trAccountUserinfo != null) {
                    marketingCourseVo.setTelAuthorize(false);
                } else {
                    // 获取该用户是否通过手机号验证
                    WechatInfo wechatInfo = new WechatInfo();
                    wechatInfo.setOpenId(openId);
                    wechatInfo.setCaptchaValidateStatus(true);
                    int validateStatus = wechatInfoService.selectCount(QueryUtil.condition(wechatInfo));
                    LOGGER.info("需要手机号授权，查看是否是平台已经授权的用户{}", validateStatus);
                    if (validateStatus > 0) {
                        marketingCourseVo.setTelAuthorize(false);
                    }
                }

            }
            // 除管理端，在学员端在平台内浏览营销课程的情况下会新增一条【ps：营销课程查看记录和营销分享观看记录是分开的（
        } else if ((shareId != null && shareId.equals(-1L) && tkAccountId.equals(accountId))
                || (shareId == null && tkAccountId > 0)) {// 如果不是分享进来的，记录营销课程查看的相关信息
            /*List<Long> relationIds = requestContext.getRelationIds();
            LOGGER.info("requestContext={}",JSON.toJSONString(requestContext));*/
            // 学员端查看课程记录
            if (!manage) {
                MarketingCourseViewLog viewLog = new MarketingCourseViewLog();
                viewLog.setId(idGenerator.generate());
                viewLog.setViewTime(new Date());
                viewLog.setCourseId(courseId);
                viewLog.setAccountId(tkAccountId);
                viewLog.setSiteId(requestContext.getSiteId());
                viewLog.setCompanyId(requestContext.getCompanyId());
                viewLog.setOrgId(requestContext.getOrgId());
                viewLog.setAccountName(requestContext.getAccountName());
                viewLog.setAccountFullName(requestContext.getAccountFullName());
                marketingCourseViewLogService.insert(viewLog);
            }
        }

        // 如果是管理端需要查询可见范
        if (manage != null && manage.booleanValue()) {
            // 如果指定了查看范围，查询出范围人的账户信息
            String scope = marketingCourseVo.getScope() + "";
            if (StringUtils.equals("2", scope)) {
                getAccounts(courseId, marketingCourseVo);
            }
        }

        // 查询关联的素材
        Chapter chapter = new Chapter();
        chapter.setCourseId(courseId);
        List<ChapterVo> chapterVos = new ArrayList<ChapterVo>();
        // 查询素材的详细信息
        Map<String, MaterialVo> retMap = new HashMap<>();
        EntityWrapper<Chapter> ew = QueryUtil.condition(chapter);
        ew.orderBy("id");
        List<Chapter> chapterlist = null;
        if (Integer.valueOf(1).equals(marketingCourseVo.getMarketingCourseType())) {
            List<ChapterDetailsVo> dataDetailsVo = iChapterService.selectChapterListMarketing(courseId);
            chapterlist = new ArrayList<>();
            for (ChapterDetailsVo detailsVo : dataDetailsVo) {
                Chapter chapter1 = new Chapter();
                BeanUtils.copyProperties(detailsVo, chapter1);
                chapterlist.add(chapter1);
            }
        } else {
            chapterlist = iChapterService.selectList(ew);
        }
        if (chapterlist != null) {
            ChapterVo chapterVo;
            for (Chapter obj : chapterlist) {
                Long materialId = obj.getMaterialId();
                chapterVo = new ChapterVo();
                BeanUtils.copyProperties(obj, chapterVo);
                chapterVos.add(chapterVo);
                if (materialId == null || Integer.valueOf(9).equals(obj.getMaterialType())) {
                    continue;
                }

                MaterialVo material = materialService.getById(materialId);
                retMap.put(materialId.toString(), material);

            }
        }
        marketingCourseVo.setChapters(chapterVos);
        marketingCourseVo.setMaterialListInfo(retMap);
        return marketingCourseVo;
    }


    private void getAccounts(@RequestParam Long courseId, MarketingCourseVo marketingCourseVo) {
        CourseAccount courseAccount = new CourseAccount();
        courseAccount.setCourseId(courseId);
        List<CourseAccount> caList = iCourseAccountService.selectList(QueryUtil.condition(courseAccount));
        if (caList != null) {
            List<CourseAccountVO> subData = new ArrayList<>();
            caList.forEach(
                    item -> {
                        CourseAccountVO vo = new CourseAccountVO();
                        BeanUtils.copyProperties(item, vo);
                        subData.add(vo);
                    }
            );
            marketingCourseVo.setAccountIds(subData);
        }
    }


    /**
     * 获取我的分享列表 （ 增加group分组课程）
     *
     * @return
     */
    @GetMapping("/share/list")
    public PageInfo<CourseShareVo> shareList(@RequestParam(value = "pageNo") int pageNo
            , @RequestParam(value = "pageSize", required = false) Integer pageSize
            , @RequestParam(value = "searchFiled", required = false) String searchFiled) {
        // 查询我的分享列表
        PageParam<CourseShareVo> pageParam = new PageInfo<>();
        pageParam.setPageNo(pageNo);
        pageParam.setPageSize(pageSize);
        PageInfo<CourseShareVo> retList = iCourseShareService.searchMyShareInfo(ContextHolder.get().getAccountId(), pageParam, searchFiled);
        // 返回数据
        return retList;
    }


    /**
     * 查询被分享课程的浏览记录
     *
     * @return
     */
    @GetMapping("/share/browse/records")
    public Page<BrowseRecords> shareBrowseRecords(@RequestParam(value = "courseId") Long courseId,
                                                  @RequestParam(value = "pageNo") int pageNo
            , @RequestParam(value = "pageSize", required = false) Integer pageSize
            , @RequestParam(value = "searchFiled", required = false) String searchFiled) {
        // 获取上下文用户信息
        Page<BrowseRecords> retList = new Page<>();
        retList.setCurrent(pageNo);
        retList.setSize(pageSize);
        RequestContext requestContext = ContextHolder.get();
        EntityWrapper<CourseShare> ew1 = QueryUtil.condition(new CourseShare());
        ew1.eq("account_id", requestContext.getAccountId()).eq("course_id", courseId);
        List<CourseShare> data = iCourseShareService.selectList(ew1);
        if (!CollectionUtils.isEmpty(data)) {
            // 查询我的分享列表
            PageParam<BrowseRecords> pageParam = new PageInfo<>();
            pageParam.setPageNo(pageNo);
            pageParam.setPageSize(pageSize);
            BrowseRecords browseRecords = new BrowseRecords();
            //browseRecords.setShareId(shareId);
            EntityWrapper<BrowseRecords> ew = QueryUtil.condition(browseRecords);
            ew.in("share_id", data.stream().map(CourseShare::getId).collect(Collectors.toList()));
            //ew.and().isNotNull("residence_time");
            if (StringUtils.isNotEmpty(searchFiled)) {
                ew.and().like("accountName", "%" + searchFiled + "%");
            }
            ew.orderBy("id", false);
            retList = iBrowseRecordsService.selectPage(PageUtil.vice(pageParam), ew);
            List<BrowseRecords> listData = retList.getRecords();
            if (!CollectionUtils.isEmpty(listData)) {
                Set<String> openIds = listData.stream().map(opeId -> opeId.getAccountId()).collect(Collectors.toSet());
                List<WechatInfo> wechatList = wechatInfoService.selectBatchIds(openIds);
                if (!CollectionUtils.isEmpty(wechatList)) {
                    Map<String, WechatInfo> wechatMap = wechatList.stream().collect(Collectors.toMap(key -> key.getOpenId(), val -> val));
                    for (BrowseRecords item : listData) {
                        if (wechatMap.containsKey(item.getAccountId())) {
                            item.setTel(item.getTel());
                        }
                    }
                }
            }
        }
        // 查询我的分享列表
        return retList;
    }

    /**
     * 分享
     *
     * @param vo
     * @return
     */
    @PostMapping("/share")
    public CourseShareVo share(@RequestBody CourseShareVo vo) {
        LOGGER.info("****************************************************分享请求参数={}", JSON.toJSONString(vo));
        if (vo.getId() == null || vo.getId() == 0L) {

            Long accountId = ContextHolder.get().getAccountId();
            CourseShare courseShareWhere = new CourseShare();
            courseShareWhere.setCourseId(vo.getCourseId());
            courseShareWhere.setAccountId(accountId);
/*            List<CourseShare> retList = iCourseShareService.selectList(QueryUtil.condition(courseShareWhere));
            if(!CollectionUtils.isEmpty(retList)){
                BeanUtils.copyProperties(retList.get(0),vo);
                return vo;
            }*/
            CourseShare obj = iCourseShareService.selectOne(QueryUtil.condition(courseShareWhere));
            LOGGER.info("根据课程id{}，用户id{}，查询分享记录={}", vo.getCourseId(), accountId, JSON.toJSONString(obj));
            if (obj != null) {
                BeanUtils.copyProperties(obj, vo);
                return vo;
            }
            //vo.setShareTime(new Date());
            vo.setId(idGenerator.generate());
            AccountVO accountVO = accountClient.findById(accountId);
            vo.setAccountId(accountVO.getId());
            String tel = accountVO.getMobile();
            vo.setTel(tel == null ? accountVO.getTelephone() : tel);
            vo.setName(accountVO.getName());
            vo.setFullName(accountVO.getFullName());
            CourseShare courseShare = new CourseShare();
            BeanUtils.copyProperties(vo, courseShare);
            courseShare.setCompanyId(ContextHolder.get().getCompanyId());
            courseShare.setSiteId(ContextHolder.get().getSiteId());
            courseShare.setOrgId(accountVO.getOrgId());
            iCourseShareService.insert(courseShare);
            LOGGER.info("新增一条分享记录={}", JSON.toJSONString(courseShare));
        } else {
            CourseShare courseShare = new CourseShare();
            courseShare.setShareTime(new Date());
            courseShare.setShareUrl(vo.getShareUrl());
            courseShare.setId(vo.getId());
            iCourseShareService.updateById(courseShare);
            LOGGER.info("更新分享={}", JSON.toJSONString(courseShare));
        }

        return vo;
    }

    /**
     * 活动检查，记录分享页面被最后浏览的时间
     *
     * @param browseRecordId
     * @return
     */
    @GetMapping("/activity")
    public boolean activity(@RequestParam Long browseRecordId) {
        BrowseRecords browseRecords = iBrowseRecordsService.selectById(browseRecordId);
        if (browseRecords != null) {
            Date currDate = new Date();
            browseRecords.setReadEndTime(currDate);
            // 相差的毫秒数
            long diffMi = currDate.getTime() - browseRecords.getReadStartTime().getTime();
            browseRecords.setResidenceTime(diffMi);
            iBrowseRecordsService.updateById(browseRecords);
        }
        return true;
    }

    /**
     * 为课程指定用户
     *
     * @param marketingCourseVo
     * @param accountIds
     */
    private void addAccount(MarketingCourseVo marketingCourseVo, List<CourseAccountVO> accountIds) {
        List<CourseAccount> addList = new ArrayList<>();

        accountIds.forEach(
                item -> {
                    CourseAccount courseAccount = new CourseAccount();
                    BeanUtils.copyProperties(item, courseAccount);
                    courseAccount.setCourseId(marketingCourseVo.getId());
                    courseAccount.setId(idGenerator.generate());
                    courseAccount.setSiteId(ContextHolder.get().getSiteId());
                    addList.add(courseAccount);
                }
        );
        iCourseAccountService.insertBatch(addList);
    }

    /**
     * 分享次数 (分享的课程数，去掉一二级下级分享课程）
     *
     * @return
     */
    @GetMapping("/share/count")
    public int shareCount() {
        RequestContext requestContext = ContextHolder.get();
        return iCourseShareService.countMyShareCourseNum(requestContext.getSiteId(), requestContext.getAccountId());
    }

    @GetMapping("/share/user/count")
    public List<CourseForwardUserVo> courseIdShareCount(@RequestBody CourseForwardUserReq req) {
        List<CourseForwardUserVo> ret = iCourseShareService.courseShareCount(req.getCourseId(), req.getLoginName());
        return ret;
    }

    // https://blog.csdn.net/tz_gx/article/details/30070649或者http://blog.sina.com.cn/s/blog_65b9816e0102xcbw.html
    @PostMapping("/share/date/count")
    public List<CourseForwardUserVo> courseIdShareDateCount(@RequestBody CourseForwardUserReq req) {
        List<CourseForwardUserVo> ret = iCourseShareService.courseShareCount(req.getCourseId(), req.getLoginName());
        return ret;
    }

    @GetMapping("/name/find")
    public List<CourseVo> courseFindByName(@RequestParam String name,
                                           @RequestParam Long companyId,
                                           @RequestParam Long siteId,
                                           @RequestParam Long orgId) {
        EntityWrapper<Course> condition = QueryUtil.condition(new Course());
        condition.where("company_id={0}", companyId);
        condition.and("site_id={0}", siteId);
        condition.and().eq("source", 4);
        condition.and().eq("shelves", 1);
        condition.and().eq("del_flg", 0);
        if (!StringUtils.trimToEmpty(name).equals(StringUtils.EMPTY)) {
            condition.and().like("name", name, SqlLike.DEFAULT);
        }

        /*List<Long> orgIds = ContextHolder.get().getOrgIds();
        if(!CollectionUtils.isEmpty(orgIds)){
            condition.and().in("org_id",orgIds);
        }*/
        // 分级授权修改
        HQueryUtil.startHQ(Course.class);
        List<Course> ret = courseService.selectList(condition);
        if (!CollectionUtils.isEmpty(ret)) {
            LOGGER.info("&&&&&&&&&&&&&&&根据名称模糊匹配课程列表=" + JSON.toJSONString(ret));
            List<CourseVo> courseVoList = new ArrayList<>();
            for (Course item : ret) {
                CourseVo courseVo = new CourseVo();
                BeanUtils.copyProperties(item, courseVo);
                courseVoList.add(courseVo);
            }
            return courseVoList;
        }
        return null;
    }

    @GetMapping("/share/spread/list")
    public PageInfo<CourseShareVo> shareSpreadList(
            @RequestParam(value = "courseId") Long courseId,
            @RequestParam(value = "pageNo") int pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "firstShareId", required = false) String firstShareId) {

        // 获取上下文用户信息
        RequestContext rc = ContextHolder.get();
        // 查询我的分享列表
        return iCourseShareService.findShareSpreadList(rc.getAccountId(), courseId, rc.getSiteId(), pageNo, pageSize, firstShareId);
    }

    @GetMapping("/wechat/openid/get")
    public WechatInfo wachatOpenIdGet(
            @RequestParam("openId") String openId
    ) {
        // 查询我的分享列表
        WechatInfo wechatInfo = wechatInfoService.selectById(openId);
        return wechatInfo;
    }

    @GetMapping("/sms/pre/check")
    public Integer smsSendPreCheck(@RequestParam("openId") String openId, @RequestParam("tel") String tel) {
        LOGGER.info("openId={}, tel={}", openId, tel);
        WechatInfo wechatInfo = wechatInfoService.selectById(openId);
        if (wechatInfo == null) {
            try {
                Map<String, UserInfoVo> data = weiXinClient.getWeChatUserInfoList(Arrays.asList(openId));
                if (CollectionUtils.isEmpty(data)) {
                    return 1; // 未经过微信授权
                }
                wechatInfo = new WechatInfo();
                wechatInfo.setOpenId(openId);
                UserInfoVo userInfo = data.get(openId);
                wechatInfo.setAvatar(userInfo.getHeadimgurl());
                wechatInfo.setNikeName(userInfo.getNickname());
                wechatInfoService.insert(wechatInfo);
            } catch (Exception e) {
                LOGGER.info("调用微信报错{}", e);
                return 1; // 未经过微信授权
            }
        }
        // 根据手机号查询关联验证码消息
        EntityWrapper<WechatInfo> wechatInfoWhere = QueryUtil.condition(new WechatInfo());
        wechatInfoWhere.eq("tel", tel);
        WechatInfo wechatInfo1 = wechatInfoService.selectOne(wechatInfoWhere);
        if (wechatInfo1 != null) {
            if (wechatInfo1.getCaptchaValidateStatus() || !wechatInfo1.getOpenId().equals(openId)) {
                return 2; // 手机号码已经被使用过
            } else if ((System.currentTimeMillis() - wechatInfo1.getCaptchaSendTime().getTime()) < 90000) {
                return 3; // 验证码获取太频繁
            }
        }
        return 0;
    }

    @GetMapping("/sms/captcha/save")
    public Boolean saveCaptcha(@RequestParam("openId") String openId,
                               @RequestParam("tel") String tel,
                               @RequestParam("captcha") String captcha) {
        WechatInfo wechatInfo = new WechatInfo();
        wechatInfo.setOpenId(openId);
        wechatInfo.setTel(tel);
        wechatInfo.setCaptcha(captcha);
        wechatInfo.setCaptchaSendTime(new Date());
        return wechatInfoService.updateById(wechatInfo);
    }

    @GetMapping("/sms/captcha/validate")
    public Integer smsCaptchaValidate(@ApiParam(name = "openId", value = "微信openId", required = true) @RequestParam("openId") String openId,
                                      @ApiParam(name = "tel", value = "手机号", required = true) @RequestParam("tel") String tel,
                                      @ApiParam(name = "captcha", value = "验证", required = true) @RequestParam("captcha") String captcha) {
        WechatInfo wechatInfo = new WechatInfo();
        wechatInfo.setOpenId(openId);
        wechatInfo.setTel(tel);
        wechatInfo.setCaptchaValidateStatus(false);
        wechatInfo.setCaptcha(captcha);
        try {
            wechatInfo = wechatInfoService.selectOne(QueryUtil.condition(wechatInfo));
        } catch (Exception e) {
            return 1; // 该用户和手机存在多次验证
        }

        if (wechatInfo == null) {
            return 2; // 验证码不存在
        }
        if ((System.currentTimeMillis() - wechatInfo.getCaptchaSendTime().getTime()) > 90000) {
            return 3; // 验证码已过期
        }
        wechatInfo.setCaptchaValidateStatus(true);
        wechatInfoService.updateById(wechatInfo);
        return 0;
    }
}
