package com.yizhi.course.application.controller;

import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.task.MarketingCourseExportAsync;
import com.yizhi.course.application.task.MarketingCourseExportXlsxAsync;
import com.yizhi.course.application.task.MarketingCourseViewExportAsync;
import com.yizhi.util.application.constant.MarketingCourseReportType;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author: XieHaijun
 * @Description: 营销课程导出处理
 * @Date: Created in 20:19 2019/1/10
 * @Modified By
 */
@RestController
@RequestMapping("/marketing/course/export")
public class MarketingCourseExportController {

    private Logger LOGGER = LoggerFactory.getLogger(MarketingCourseExportController.class);

    @Autowired
    private MarketingCourseExportAsync marketingCourseExportAsync;

    @Autowired
    private MarketingCourseExportXlsxAsync marketingCourseExportXlsxAsync;

    @Autowired
    private MarketingCourseViewExportAsync marketingCourseViewExportAsync;

    @Autowired
    private ICourseService courseService;

    /**
     * 按转发阅读的组织导出
     * @param parament
     * @return
     */
    @PostMapping({"/forword/org"})
    public Boolean marketingCourseGroupExport(@RequestBody Map<String, Object> parament){
        RequestContext requestContext = ContextHolder.get();
        parament.put("requestContext",requestContext);
        marketingCourseExportAsync.execute(parament, true);
        return true;
    }

    /**
     * 按转发的用户导出（明细）
     * @param parament
     * @return
     */
    @PostMapping({"/forword/account"})
    public Boolean marketingCourseAccountGroupExport(@RequestBody Map<String, Object> parament){
        RequestContext requestContext = ContextHolder.get();
        parament.put("requestContext",requestContext);
        String business = (String) parament.get("business");// 获取业务标识
        if(StringUtils.equals(MarketingCourseReportType.BY_USER_EXPORT,business)){
            marketingCourseExportAsync.execute(parament, true);
        }else {
            marketingCourseExportXlsxAsync.execute(parament, true);
        }

        return true;
    }

    /**CourseController
     * 营销课程访问数据下载结果
     * @param courseId
     * @return
     */
    @GetMapping("/course/group/view/list/export")
    public String marketingCourseViewListExport(
            @RequestParam(name = "courseId") Long courseId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "accountId") Long accountId,
            @RequestParam(name = "companyId") Long companyId){
        Map<String, Object> parament = new HashMap<>();
        parament.put("courseId",courseId );
        parament.put("siteId",siteId );
        parament.put("accountId",accountId );
        parament.put("companyId",companyId );
        LocalDateTime now = LocalDateTime.now();
        Date submitTime = Date.from(now.atZone(ZoneId.systemDefault()).toInstant());//任务提交时间
        Long taskId = submitTime.getTime();//任务标识
        parament.put("taskId",taskId );
        Course course  = courseService.selectById(courseId);
        if(course == null){
            return taskId.toString();
        }
        parament.put("courseName",course.getName() );
        LOGGER.info("学习营销课程打开转发统计 处理开始");
        marketingCourseViewExportAsync.execute(parament, true);
        String ret = "任务编号："+taskId.toString()+"。任务名称：学习营销课程打开转发统计-"+taskId.toString();
        return ret;
    }

}
