package com.yizhi.course.application.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.AudioVideo;
import com.yizhi.course.application.domain.Material;
import com.yizhi.course.application.service.IAudioVideoService;
import com.yizhi.course.application.service.IMaterialService;
import com.yizhi.course.application.task.MaterialListExportAsync;
import com.yizhi.course.application.vo.MaterialVo;
import com.yizhi.wechat.application.utils.HttpClientUtils;
import io.swagger.annotations.ApiParam;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 素材库 前端控制器
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Log4j2
@RestController
@RequestMapping("/material")
public class MaterialController {

    @Autowired
    private IMaterialService materialService;

    @Autowired
    private IAudioVideoService audioVideoService;

    @Autowired
    private MaterialListExportAsync materialListExportAsync;

    @Value("${polyv.remote.url}")
    private String polyvUrl;

    @Value("${polyv.writetoken}")
    private String writeToken;

    @Value("${polyv.cataid}")
    private String cataId;

    private Logger LOGGER = LoggerFactory.getLogger(MaterialController.class);
    /**
     * 素材保存 包含5大类型 太麻烦就不详细写了
     * @param map
     * @return
     */
    @PostMapping("/save")
    public Boolean insert(@RequestBody Map<String,Object> map) {
        Material material =JSON.parseObject(JSON.toJSONString(map.get("material")),new TypeReference<Material>() {});
        Material materialResult = materialService.saveMaterial(material,map);
        if (materialResult!=null){
            materialService.uploadPolyVideoRemote(materialResult);
            return Boolean.TRUE;
        } else {
            return Boolean.FALSE;
        }
    }

    /**
     * 新增并返回新增对象
     * @param map
     * @return
     */
    @PostMapping("/save/obj")
    public Material insertObj(@RequestBody Map<String,Object> map) {
        Material material =JSON.parseObject(JSON.toJSONString(map.get("material")),new TypeReference<Material>() {});
        Material materialResult = materialService.saveMaterial(material,map);
        if (materialResult!=null){
            materialService.uploadPolyVideoRemote(materialResult);
            return materialResult;
        } else {
            return materialResult;
        }
    }

    /**
     * 素材删除 已关联的删不掉，，
     * @param ids
     * @return
     */
    @PostMapping("/delete")
    public Boolean delete(@RequestBody List<Long> ids) {
        Boolean f = materialService.deleteMaterial(ids);
        return f;
    }


    /**
     * 素材详情获得  很大的一个类 不详写
     * @param id
     * @return
     */
    @GetMapping("/get")
    public MaterialVo get(@RequestParam(value = "id") Long id) {
        MaterialVo material = materialService.getById(id);
        return material;
    }

    /**
     *  素材详情  简化版
     * @param id
     * @return
     */
    @GetMapping("/view")
    public Material view(@RequestParam(value = "id") Long id) {
        Material material = materialService.selectById(id);
        return material;
    }

    /**
     * 素材更新
     * @param map
     * @return
     */
    @PostMapping("/update")
    public Boolean update(@RequestBody Map<String,Object> map) {
        Material material =JSON.parseObject(JSON.toJSONString(map.get("material")),new TypeReference<Material>() {});
        String originalUrlOld = null;
        String originalUrlNew = null;
        //Material materialResult = null;
        Integer type =  material.getType();
        try {
            originalUrlOld = materialService.selectById(material.getId()).getOriginalFileUrl();
            // 没有源文件url，不做
            originalUrlNew = material.getOriginalFileUrl();
            log.info("originalUrlOld="+originalUrlOld);

            if(Integer.valueOf(2).equals(type)||Integer.valueOf(1).equals(type)){
                if (material!=null) { // 只有视频素材才能调用保利威的远程上传接口
                    if (StringUtils.isNotBlank(originalUrlOld) && originalUrlOld.equals(originalUrlNew)){
                        log.info("远程的地址没有变化，不要更新保利威视!");
                        material.setTextHour(null);
                        material.setTextMinute(null);
                        material.setTextSecond(null);
                    } else {
                        if(StringUtils.isNotBlank(originalUrlNew)){
//                            materialService.uploadPolyVideoRemote(material);
                            MaterialVo materialVo = polyUploadVid(material);
                            if (materialVo!=null) {
                                material.setTextHour(materialVo.getTextHour());
                                material.setTextMinute(materialVo.getTextMinute());
                                material.setTextSecond(materialVo.getTextSecond());
                                material.setPolyVid(materialVo.getPolyVid());
                            }
                        }

                    }
                } else {
                    LOGGER.info("素材id不存在");
                    return Boolean.FALSE;
                }
            }

            materialService.updateMaterial(material,map);
            return Boolean.TRUE;
        } catch (Exception e) {
            LOGGER.info("素材更新失败，异常原因={}",JSON.toJSONString(e));
            return Boolean.FALSE;
        }

    }

    /**
     * 管理端素材列表
     * @param map
     * @return
     */
    @GetMapping("/list")
    public Page<Material> list(@RequestBody Map<String,Object> map) {
        Long companyId = null;
        Long siteId = null;
        Integer pageNo = Integer.valueOf (map.get("pageNo").toString());
        Integer pageSize = Integer.valueOf  (map.get("pageSize").toString());
        Integer source = null;
        List<Long> orgId = null;
        if (map.get("source")!=null){
             source = Integer.valueOf(map.get("source").toString());
        }
        Integer type = null;
        if (map.get("type")!=null) {
            type = Integer.valueOf  (map.get("type").toString());
        }
        String  name = null;
        if (map.get("name")!=null) {
            name = map.get("name").toString();
        }
        if (map.containsKey("companyId")) {
            companyId = Long.valueOf(map.get("companyId").toString());
        }
        if (map.containsKey("siteId")){
            siteId = Long.valueOf(map.get("siteId").toString());
        }
        if (map.containsKey("orgId")) {
            orgId = (List<Long>) map.get("orgId");
        }
        Page<Material> materialPage = materialService.searchPage(name,type,source,companyId,siteId,orgId,pageNo,pageSize);
        return materialPage;
    }



    @GetMapping("/video/view")
    public AudioVideo videoView(@RequestParam(value = "videoId") String videoId){
        AudioVideo audioVideo = new AudioVideo();
        audioVideo.setVideoId(videoId);
        EntityWrapper<AudioVideo> ew = new EntityWrapper<>(audioVideo);
        AudioVideo av = audioVideoService.selectOne(ew);
        return av;
    }


    @PostMapping("/video/update")
    public Boolean videoUpdate(@RequestBody Map<String,Object> map){
        return audioVideoService.updateAudioVideo(map);
    }


    /**
     * 查询素材列表  ( Scorm课程 )
     *
     * @return
     */
    @GetMapping("/scormList")
    public  Page<Material> queryScormCourseList(@RequestParam Map<String,Object> map){
        Long companyId = null;
        Long siteId = null;
        Integer pageNo = Integer.valueOf (map.get("pageNo").toString());
        Integer pageSize = Integer.valueOf  (map.get("pageSize").toString());
        Integer source = null;
        List<Long> orgId = null;
        if (!org.springframework.util.StringUtils.isEmpty(map.get("source"))){
            source = Integer.valueOf(map.get("source").toString());
        }
        Integer type = null;
        if (!org.springframework.util.StringUtils.isEmpty(map.get("type"))) {
            type = Integer.valueOf  (map.get("type").toString());
        }
        String  name = null;
        if (!org.springframework.util.StringUtils.isEmpty(map.get("name"))) {
            name = map.get("name").toString();
        }
        if (!org.springframework.util.StringUtils.isEmpty(map.get("companyId"))) {
            companyId = Long.valueOf(map.get("companyId").toString());
        }
        if (!org.springframework.util.StringUtils.isEmpty(map.get("siteId"))){
            siteId = Long.valueOf(map.get("siteId").toString());
        }
        if (!org.springframework.util.StringUtils.isEmpty(map.get("orgId"))) {
            orgId = (List<Long>) map.get("orgId");
        }
        return materialService.queryScormCourseList(name,type,source,companyId,siteId,orgId,pageNo,pageSize);
    }

    /**
     * 查询素材列表  ( 一般课程 )
     * @param map
     * @return
     */
    @GetMapping("/commonList")
    public  Page<Material> queryCommonCourseList(@RequestParam Map<String,Object> map){
        Long companyId = null;
        Long siteId = null;
        Integer pageNo = Integer.valueOf (map.get("pageNo").toString());
        Integer pageSize = Integer.valueOf  (map.get("pageSize").toString());
        Integer source = null;
        List<Long> orgId = null;
        if (!org.springframework.util.StringUtils.isEmpty(map.get("source"))){
            source = Integer.valueOf(map.get("source").toString());
        }
        Integer type = null;
        if (!org.springframework.util.StringUtils.isEmpty(map.get("type"))) {
            type = Integer.valueOf  (map.get("type").toString());
        }
        String  name = null;
        if (!org.springframework.util.StringUtils.isEmpty(map.get("name"))) {
            name = map.get("name").toString();
        }
        if (!org.springframework.util.StringUtils.isEmpty(map.get("companyId"))) {
            companyId = Long.valueOf(map.get("companyId").toString());
        }
        if (!org.springframework.util.StringUtils.isEmpty(map.get("siteId"))){
            siteId = Long.valueOf(map.get("siteId").toString());
        }
        /*if (map.containsKey("orgId")) {
            orgId = (List<Long>) map.get("orgId");
        }*/
        Integer excludType = null;
        if (!org.springframework.util.StringUtils.isEmpty(map.get("excludType"))) {
            excludType = Integer.valueOf(map.get("excludType").toString());
        }

        return materialService.queryCommonCourseList(name,type,source,companyId,siteId,orgId,pageNo,pageSize,excludType);
    }

    /**
     * 上传保利威视
     * @date 2019年3月4日
     * @param material
     */

    public MaterialVo polyUploadVid(Material material) {
        MaterialVo materialVo = new MaterialVo();
        BeanUtils.copyProperties(material,materialVo);

        HttpClientUtils httpClientUtils = new HttpClientUtils();


        // 远程上传保利威视 视频
        Map<String, String> polyvMap = new HashMap<String, String>(16);
        polyvMap.put("fileUrl", material.getOriginalFileUrl());
        polyvMap.put("writetoken", writeToken);
        polyvMap.put("title", material.getName());
        polyvMap.put("cataid",cataId);
        log.info("参数的polyvMap：" + polyvMap);
        String reStr = httpClientUtils.httpPost(polyvUrl, polyvMap);
        log.info("上传保利威视的远程视的结果：" + reStr);

        JSONObject polyResult = JSON.parseObject(reStr);
        log.info("保利威视返回的结果：" + reStr);
        if (null != reStr && polyResult.containsKey("data")) {
            String data = polyResult.getString("data");
            JSONArray jsonArray = JSON.parseArray(data);
            JSONObject dat = JSON.parseObject(jsonArray.get(0).toString());
            String vid = dat.getString("vid");
            log.info("保利威视的vid：" + vid);
            String duration = dat.getString("duration");

            materialVo.setPolyVid(vid);

            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
            try {
                Date date = sdf.parse(duration);
                System.out.println(sdf.parse(duration));
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(date);
                int hour = calendar.get(Calendar.HOUR_OF_DAY);
                int minute = calendar.get(Calendar.MINUTE);
                int second = calendar.get(Calendar.SECOND);
                log.info("hour:" + hour + ",minute:" + minute + ",second:" + second);
                materialVo.setTextHour(hour);
                materialVo.setTextMinute(minute);
                materialVo.setTextSecond(second);
            } catch (ParseException e) {
                log.info("异常{}", e);
            }
            return materialVo;
        } else {
            LOGGER.info("视频上传异常!");
            return null;
        }
    }



    /***
     *  导出素材清单
     * @param classifyId
     * @return
     */
    @GetMapping("/list/export")
    public Map<String, Object> materialListExport(@RequestParam(required = false, value = "type", defaultValue = "0") Integer type,
                                          @ApiParam(name = "name", value = "课程名称或作者", required = false) @RequestParam(name = "name", required = false) String name,
                                          @ApiParam(name = "classifyId", value = "分类id ", required = false) @RequestParam(name = "classifyId", required = false) Long classifyId,
                                          @ApiParam(name = "shelves", value = "0 未上架 1 已上架 2 草稿", required = false) @RequestParam(name = "shelves", required = false) Integer shelves,
                                          @ApiParam(name = "courseType", value = "课程类型", required = false) @RequestParam(required = false, value = "courseType") Integer courseType) {
        RequestContext requestContext = ContextHolder.get();
        Date submitTime = new Date();
        Long taskId = submitTime.getTime();
        String serialNo = "Material-EXPORT-" + taskId;
        String taskName = "导出素材清单-" + taskId;
        String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("accountId", requestContext.getAccountId());
        map.put("siteId", requestContext.getSiteId());
        map.put("companyId", requestContext.getCompanyId());
        map.put("serialNo", serialNo);
        map.put("taskId", taskId);
        map.put("submitTime", submitTime);
        map.put("taskName", taskName);
        map.put("type", type);
        map.put("name", name);
        map.put("classifyId", classifyId);
        map.put("shelves", shelves);
        map.put("courseType", courseType);
        map.put("result", result);
        materialListExportAsync.execute(map, true);
        return map;
    }

}

