package com.yizhi.course.application.controller;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.OfflineClassify;
import com.yizhi.course.application.service.IOfflineClassifyService;
import com.yizhi.course.application.vo.OfflineClassifyVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * <p>
 * 线下课程分类 前端控制器
 * </p>
 *
 * @author huangliang123
 * @since 2018-10-16
 */
@RestController
@RequestMapping("/manage/offlineClassify")
public class OfflineClassifyController {

    @Autowired
    private IOfflineClassifyService offlineClassifyService;

    /**
     * 根据id删除线下课程分类
     * notice: 只能删除没有子分类的实体
     *
     * @param
     * @return
     */
    @PostMapping("/delete")
    public Integer delete(@RequestBody Map<String, String> map) {
        String idStr = map.get("id");
        if (!StringUtils.isEmpty(idStr)) {
            return offlineClassifyService.deleteById(Long.valueOf(idStr));
        } else {
            return 0;
        }
    }

    /**
     * 添加一个新的线下课程分类
     *
     * @param offlineClassify
     * @return
     */
    @PostMapping("/save")
    public Integer saveOfflineClassify(@RequestBody OfflineClassify offlineClassify) {
        return offlineClassifyService.saveOfflineClassify(offlineClassify);
    }

    /**
     * 修改线下课程分类
     *
     * @param offlineClassify
     * @return
     */
    @PostMapping("/update")
    public Integer updateOfflineClassify(@RequestBody OfflineClassify offlineClassify) {
        return offlineClassifyService.updateOfflineClassify(offlineClassify);
    }

    /**
     * 线下课程分类排序
     *
     * @param list
     * @return
     */
    @PostMapping("/sort/update")
    public Integer offlineClassifySort(@RequestBody List<OfflineClassify> list) {
        return offlineClassifyService.offlineClassifySort(list);
    }

    /**
     * 线下课程分类分页查询
     *
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/page/list")
    public Page<OfflineClassifyVo> pageList(
            @RequestParam(name = "name", required = false) String name,
            @RequestParam(name = "pageNo") Integer pageNo,
            @RequestParam(name = "pageSize") Integer pageSize
    ) {
        RequestContext context = ContextHolder.get();
        Page<OfflineClassifyVo> page = offlineClassifyService.selectListByCompanyId(context.getCompanyId(),
                name, context.getOrgIds(), context.getSiteId(), pageNo, pageSize);

        List<OfflineClassify> offlineClassifies2 = offlineClassifyService.selectOfflineClassifyByLayer(context.getCompanyId(), context.getSiteId(), 2);
        List<OfflineClassify> offlineClassifies3 = offlineClassifyService.selectOfflineClassifyByLayer(context.getCompanyId(), context.getSiteId(), 3);

        Map<Long, List<OfflineClassifyVo>> map3 = new HashMap<>();
        Map<Long, List<Long>> mapList3 = new HashMap<>();
        for (OfflineClassify classify3 : offlineClassifies3) {
            OfflineClassifyVo classifyVo = new OfflineClassifyVo();
            classifyVo = copyBean(classifyVo, classify3);

            if (map3.containsKey(classify3.getParentId())) {
                map3.get(classify3.getParentId()).add(classifyVo);
                mapList3.get(classify3.getParentId()).add(classify3.getId());
            } else {
                List<OfflineClassifyVo> classifyVos = new ArrayList<>();
                List<Long> list3 = new ArrayList<>();

                classifyVos.add(classifyVo);
                list3.add(classify3.getId());

                map3.put(classify3.getParentId(), classifyVos);
                mapList3.put(classify3.getParentId(), list3);
            }
        }
        buildUpAndDownId(map3, mapList3);

        Map<Long, List<OfflineClassifyVo>> map2 = new TreeMap<>();
        Map<Long, List<Long>> mapList2 = new HashMap<>();
        for (OfflineClassify classify2 : offlineClassifies2) {
            OfflineClassifyVo classifyVo = new OfflineClassifyVo();
            classifyVo = copyBean(classifyVo, classify2);
            classifyVo.setChildren(map3.get(classify2.getId()));

            if (map2.containsKey(classify2.getParentId())) {
                map2.get(classify2.getParentId()).add(classifyVo);
                mapList2.get(classify2.getParentId()).add(classify2.getId());
            } else {
                List<OfflineClassifyVo> classifyVos = new ArrayList<>();
                List<Long> list2 = new ArrayList<>();
                classifyVos.add(classifyVo);
                list2.add(classify2.getId());
                map2.put(classify2.getParentId(), classifyVos);
                mapList2.put(classify2.getParentId(), list2);
            }
        }
        buildUpAndDownId(map2, mapList2);

        List<OfflineClassifyVo> offlineClassifyVos = page.getRecords();
        if (offlineClassifyVos != null) {
            Map<Long, List<OfflineClassifyVo>> map = new HashMap<>();
            Map<Long, List<Long>> mapList = new HashMap<>();
            map.put(0L, new ArrayList<>());
            mapList.put(0L, new ArrayList<>());

            for (OfflineClassifyVo offlineClassifyVo : offlineClassifyVos) {
                offlineClassifyVo.setChildren(map2.get(offlineClassifyVo.getId()));
                map.get(0L).add(offlineClassifyVo);
                mapList.get(0L).add(offlineClassifyVo.getId());
            }
            buildUpAndDownId(map, mapList);
        }
        return page;
    }

    private OfflineClassifyVo copyBean(OfflineClassifyVo offlineClassifyVo, OfflineClassify offlineClassify) {
        offlineClassifyVo.setId(offlineClassify.getId());
        offlineClassifyVo.setParentId(offlineClassify.getParentId());
        offlineClassifyVo.setLayer(offlineClassify.getLayer());
        offlineClassifyVo.setName(offlineClassify.getName());
        offlineClassifyVo.setDescription(offlineClassify.getDescription());
        offlineClassifyVo.setDeleted(offlineClassify.getDeleted());
        offlineClassifyVo.setCreateById(offlineClassify.getCreateById());
        offlineClassifyVo.setCreateByName(offlineClassify.getCreateByName());
        offlineClassifyVo.setCreateTime(offlineClassify.getCreateTime());
        offlineClassifyVo.setOrgId(offlineClassify.getOrgId());
        offlineClassifyVo.setCompanyId(offlineClassify.getCompanyId());
        offlineClassifyVo.setRootId(offlineClassify.getRootId());
        offlineClassifyVo.setSiteId(offlineClassify.getSiteId());
        offlineClassifyVo.setSort(offlineClassify.getSort());
        offlineClassifyVo.setSource(offlineClassify.getSource());
        offlineClassifyVo.setUpdateById(offlineClassify.getUpdateById());
        offlineClassifyVo.setUpdateByName(offlineClassify.getUpdateByName());
        offlineClassifyVo.setUpdateTime(offlineClassify.getUpdateTime());
        return offlineClassifyVo;
    }

    private void buildUpAndDownId(Map<Long, List<OfflineClassifyVo>> map, Map<Long, List<Long>> mapList) {
        List<OfflineClassifyVo> vos = null;
        List<Long> ids = null;
        for (Map.Entry<Long, List<OfflineClassifyVo>> entry : map.entrySet()) {
            vos = entry.getValue();
            if (!CollectionUtils.isEmpty(vos) && vos.size() > 1) {
                ids = mapList.get(entry.getKey());
                for (int i = 0; i < vos.size(); i++) {
                    // 設置下移 id
                    if (i == 0) {
                        vos.get(i).setMoveDownId(ids.get(i + 1));
                    }
                    // 只設置上移 id
                    else if (i == vos.size() - 1) {
                        vos.get(i).setMoveUpId(ids.get(i - 1));
                    } else {
                        vos.get(i).setMoveDownId(ids.get(i + 1));
                        vos.get(i).setMoveUpId(ids.get(i - 1));
                    }
                }
            }
        }
    }

    /**
     *  分类查询
     *
     * @return
     */
    @PostMapping("/list")
    public List<OfflineClassify> listOfflineClassify() {
        return offlineClassifyService.listOfflineClassify();
    }
}
