package com.yizhi.course.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.OfflineClassify;
import com.yizhi.course.application.domain.OfflineCourse;
import com.yizhi.course.application.service.IOfflineClassifyService;
import com.yizhi.course.application.service.IOfflineCourseService;
import com.yizhi.course.application.vo.OfflineCourseDeleteVo;
import com.yizhi.course.application.vo.OfflineCourseVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 线下课程表 前端控制器
 * </p>
 *
 * @author huangliang123
 * @since 2018-10-16
 */
@RestController
@RequestMapping("/offlineCourse")
public class OfflineCourseController {

    private static final Logger LOGGER = LoggerFactory.getLogger(OfflineCourseController.class);

    @Autowired
    private IOfflineCourseService offlineCourseService;
    @Autowired
    private IOfflineClassifyService offlineClassifyService;


    /**
     * 线下课程删除, 支持批量
     *
     * @param offlineCourseDeleteVo
     * @return
     */
    @PostMapping("/delete")
    public boolean delete(@RequestBody OfflineCourseDeleteVo offlineCourseDeleteVo) {
        String[] offlineCourseArr = offlineCourseDeleteVo.getIds().split(",");
        if (offlineCourseArr.length == 1) {
            OfflineCourse offlineCourse = offlineCourseService.selectById(Long.valueOf(offlineCourseArr[0]));
            if (offlineCourse.getStatus() == 0 ) {
                return Boolean.FALSE;
            }
        }
        Boolean flag = Boolean.FALSE;
        for (String offlineCourseId : offlineCourseArr) {
            OfflineCourse offlineCourse = offlineCourseService.selectById(Long.valueOf(offlineCourseId));
            if (offlineCourse.getStatus() == 0 ) {
                return Boolean.FALSE;
            } else {
                offlineCourse.setDeleted(1);
                RequestContext context = ContextHolder.get();
                offlineCourse.setUpdateById(context.getAccountId());
                offlineCourse.setUpdateByName(context.getAccountName());
                offlineCourse.setUpdateTime(new Date());
                offlineCourseService.updateById(offlineCourse);
                flag = Boolean.TRUE;
            }
        }
        return flag;
    }

    /**
     * 返回线下课程保存的内存
     *
     * @param offlineCourseVo
     * @return
     */
    @PostMapping("/save")
    public OfflineCourseVo save(@RequestBody OfflineCourseVo offlineCourseVo) {

        OfflineCourseVo f = offlineCourseService.save(offlineCourseVo);
        return f;
    }

    /**
     * 线下课程code 唯一性验证
     *
     * @param id
     * @param code
     * @param siteId
     * @return
     */
    @GetMapping("/code/verifyCourse")
    public Boolean verifyCourseCode(@RequestParam(value = "id", required = false) Long id,
                                    @RequestParam(value = "code") String code,
                                    @RequestParam(value = "siteId") Long siteId
    ) {
        return offlineCourseService.verifyCourseCode(id, code, siteId);
    }

    /**
     * 修改线下课程
     *
     * @param offlineCourseVo
     * @return
     */
    @PostMapping("/update")
    public Boolean update(@RequestBody OfflineCourseVo offlineCourseVo) {
        Boolean f = offlineCourseService.update(offlineCourseVo);
        return f;
    }

    /**
     * 线下课程分页查询
     *
     * @param map
     * @return
     */
    @GetMapping("/list")
    public Page<OfflineCourse> list(@RequestBody Map<String, Object> map) {

        Integer pageNo = Integer.valueOf(map.get("pageNo").toString());
        Integer pageSize = Integer.valueOf(map.get("pageSize").toString());
        Long companyId = null;
        Long siteId = null;
        Long offlineClassifyId = null;
        Integer status = null;
        String name = null;
        List<Long> orgIds = null;
        if (map.containsKey("companyId")) {
            companyId = Long.valueOf(map.get("companyId").toString());
        }
        if (map.containsKey("siteId")) {
            siteId = Long.valueOf(map.get("siteId").toString());
        }
        if (map.containsKey("orgIds")) {
            orgIds = (List<Long>) map.get(orgIds);
        }
        if (map.get("offlineClassifyId") != null) {
            offlineClassifyId = Long.valueOf(map.get("offlineClassifyId").toString());
        }
        if (map.get("status") != null) {
            status = Integer.valueOf(map.get("status").toString());
        }
        if (map.get("name") != null) {
            name = map.get("name").toString();
        }

        Page<OfflineCourse> page = offlineCourseService.searchPage(name, offlineClassifyId, status, companyId, siteId, orgIds, pageNo, pageSize);
        return page;
    }

    /**
     * 全文检索  (待定)
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param orgId
     * @return
     */
    @GetMapping("/search/list")
    public List<OfflineCourse> getCourseList(
            @RequestParam(name = "name") String name,
            @RequestParam(name = "companyId") Long companyId,
            @RequestParam(name = "siteId") Long siteId,
            @RequestParam(name = "orgId") Long orgId
    ) {
        List<OfflineCourse> offlineCourses = offlineCourseService.selectCourseList(name, companyId, siteId, orgId);
        return offlineCourses;
    }

    /**
     * 线下课程_课程上架
     *
     * @param offlineCourse 参数中有课程主键id
     * @return
     */
    @PostMapping("/up")
    public Boolean up(@RequestBody OfflineCourse offlineCourse) {
        OfflineCourse course = new OfflineCourse();
        course.setStatus(0);
        course.setId(offlineCourse.getId());

        RequestContext context = ContextHolder.get();
        course.setUpdateTime(new Date());
        course.setUpdateByName(context.getAccountName());
        course.setUpdateById(context.getAccountId());

        return course.updateById();
    }

    /**
     * 线下课程_课程下架
     *
     * @param offlineCourse
     * @return
     */
    @PostMapping("/down")
    public Boolean down(@RequestBody OfflineCourse offlineCourse) {
        OfflineCourse currentCourse = new OfflineCourse();
        currentCourse.setStatus(1);
        currentCourse.setId(offlineCourse.getId());

        RequestContext context = ContextHolder.get();
        currentCourse.setUpdateById(context.getAccountId());
        currentCourse.setUpdateByName(offlineCourse.getUpdateByName());
        currentCourse.setUpdateTime(new Date());

        return currentCourse.updateById();
    }

    /**
     * 课程查看
     *
     * @param id
     * @return
     */
    @GetMapping("/get")
    public OfflineCourseVo get(@RequestParam(value = "id") Long id) {
        OfflineCourseVo offlineCourseVo = new OfflineCourseVo();
        OfflineCourse offlineCourse = offlineCourseService.selectById(id);
        if (offlineCourse != null) {
            offlineCourseVo.setId(id);
            offlineCourseVo.setCode(offlineCourse.getCode());
            offlineCourseVo.setName(offlineCourse.getName());
            offlineCourseVo.setOfflineClassifyId(offlineCourse.getOfflineClassifyId());
            OfflineClassify offlineClassify = offlineClassifyService.selectById(offlineCourse.getOfflineClassifyId());
            if (offlineClassify != null) {
                offlineCourseVo.setPath(offlineClassify.getPath());
            }

            offlineCourseVo.setDescription(offlineCourse.getDescription());
            offlineCourseVo.setImage(offlineCourse.getImage());
            offlineCourseVo.setTags(offlineCourse.getTags());
            offlineCourseVo.setAuthorUnit(offlineCourse.getAuthorUnit());
        }
        return offlineCourseVo;
    }

    /**
     * 通过id 获取用的信息
     * @param id
     * @return
     */
    @GetMapping("/get/offlineCourse")
    public OfflineCourse getCourseById(@RequestParam(value = "id") Long id) {

        return  offlineCourseService.selectById(id);
    }

    /**
     * 获取下线课程
     * @param courseNo
     * @return
     */
    @GetMapping("/getOfflineCourseByCode")
    public OfflineCourse getOfflineCourseByCode(@RequestParam(value = "courseNo") String courseNo,@RequestParam(value = "siteId") Long siteId) {
        OfflineCourse offlineCourse = new OfflineCourse();
        offlineCourse.setCode(courseNo);
        offlineCourse.setDeleted(0);
        offlineCourse.setStatus(0);
        offlineCourse.setSiteId(siteId);
        EntityWrapper<OfflineCourse> condition = QueryUtil.condition(offlineCourse);
        return  offlineCourseService.selectOne(condition);
    }

}

