package com.yizhi.course.application.controller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.file.domain.FileInfo;
import com.yizhi.core.application.file.domain.Template;
import com.yizhi.core.application.file.domain.TemplateColumn;
import com.yizhi.core.application.file.exp.ExcelExportor;
import com.yizhi.core.application.log.LogEventPublisher;
import com.yizhi.core.application.log.LogQueue;
import com.yizhi.core.application.log.TaskLogDetailEvent;
import com.yizhi.core.application.log.TaskLogEvent;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.course.application.domain.OfflineCourse;
import com.yizhi.course.application.domain.OfflineRecorde;
import com.yizhi.course.application.service.IOfflineCourseService;
import com.yizhi.course.application.service.IOfflineRecordeService;
import com.yizhi.course.application.vo.OfflineCourseVo;
import com.yizhi.course.application.vo.ReportOfflineCourseStatisticsDetailVo;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.point.application.vo.PointParamVO;
import com.yizhi.system.application.model.AccountNameForIdsModel;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.constant.TpActivityType;
import com.yizhi.util.application.date.DateUtil;
import com.yizhi.util.application.event.TrainingProjectEvent;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
//import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 线下学习记录 前端控制器
 * </p>
 *
 * @author huangliang123
 * @since 2018-10-16
 */
@RestController
@RequestMapping("/offlineRecorde")
public class OfflineRecordeController {

	private static final Logger LOGGER = LoggerFactory.getLogger(OfflineRecordeController.class);

	@Autowired
	private IOfflineCourseService offlineCourseService;
	@Autowired
	private IOfflineRecordeService offlineRecordeService;
	@Autowired
	private AccountClient accountClient;
	@Autowired
	private IdGenerator idGenerator;
	@Autowired
	private LogEventPublisher logEventPublisher;
	/*@Autowired
	AmqpTemplate amqpTemplate;*/
/*
	@Autowired
	RabbitTemplate rabbitTemplate;*/

	@Autowired
	private TaskExecutor taskExecutor;
	@Autowired
	private CloudEventPublisher cloudEventPublisher;

	@Autowired
	private PointRedisFeignClients pointRedisFeignClients;

	@Autowired
	private ReportClient reportClient;

	/**
	 * 上传文件 (导入数据)
	 *
	 * @param offlineCourseVo
	 * @return
	 */
	@PostMapping(value = "/upload")
	public String upload(@RequestBody OfflineCourseVo offlineCourseVo) {

		Date date = new Date();
		String serialNo = DateUtil.getYMDHMSS();
		Long taskId = idGenerator.generate();
		Integer result = offlineCourseService.upload(offlineCourseVo);
		RequestContext context = ContextHolder.get();
		CustomImporter importer = new CustomImporter(new Date(), context, offlineCourseVo, taskId);
		String taskName = "线下课程-按课程统计-导入数据统计- " + serialNo;
		TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, context.getAccountId(), date,
				context.getSiteId(), context.getCompanyId());
		importer.bindContext(taskContext);
		importer.bindPublisher(logEventPublisher);
		importer.handle();
		return taskName;
	}

	@PostMapping("download")
	@ApiOperation(value = "下载报表数据")
	public String download(
			@ApiParam("参数形式如：{\"startTime\": \"2018-01-10\", \"endTime\": \"2018-01-10\", \"name\" : \"course\"}") @RequestBody Map<String, Object> param) {
		Date startTime = null;
		Date endTime = null;
		String serialNo = param.get("serialNo").toString();
		try {
			startTime = DateUtils.truncate(
					DateUtils.parseDate((String) param.get("startTime"), new String[] { "yyyy-MM-dd" }), Calendar.DATE);
			endTime = DateUtils.truncate(
					DateUtils.parseDate((String) param.get("endTime"), new String[] { "yyyy-MM-dd" }), Calendar.DATE);
			String name = (String) param.get("name");
			RequestContext context = ContextHolder.get();
			CustomExporter exporter = new CustomExporter(null, null, startTime, endTime, name, name, context);
			String taskName = "线下课程-按课程统计-下载结果数据-" + serialNo;
			TaskContext taskContext = new TaskContext(idGenerator.generate(), serialNo, taskName,
					context.getAccountId(), new Date(), context.getSiteId(), context.getCompanyId());
			exporter.bindContext(taskContext);
			exporter.bindPublisher(logEventPublisher);
			exporter.handle();
			return taskName;
		} catch (Exception e) {
			LOGGER.error("", e);
			return null;
		}
	}

	// *********************************************** 导入 开始
	// ******************************************************* //
	private Map<String, List> analysisExcel(Date now, RequestContext context, OfflineCourseVo vo, Long taskId)
			throws Exception {
		// 创建workbook
		URL url = null;
		HttpURLConnection conn = null;
		InputStream inputStream = null;
		Workbook book = null;
		try {
			String ossUrl = vo.getOssUrl();
			String fileName = ossUrl.substring(ossUrl.lastIndexOf("/") + 1);
			String ossUrlPrefix = ossUrl.substring(0, ossUrl.lastIndexOf("/") + 1);
			ossUrl = ossUrlPrefix + URLEncoder.encode(fileName, "utf-8");
			url = new URL(ossUrl);
			conn = (HttpURLConnection) url.openConnection();
			// 防止屏蔽程序抓取而返回403错误
			conn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");
			inputStream = conn.getInputStream();
			book = WorkbookFactory.create(inputStream);
		} catch (MalformedURLException e) {
			LOGGER.error("创建ossurl 连接错误", e.getMessage());
			throw e;
		} catch (IOException e) {
			LOGGER.error("创建ossurl 连接错误", e.getMessage());
			throw e;
		} catch (InvalidFormatException e) {
			LOGGER.error("创建Workbook 对象错误", e.getMessage());
			throw e;
		} finally {
			if (inputStream != null) {
				inputStream.close();
			}

		}

		if (null == book) {
			String errMsg = "创建Workbook 对象失败， OfflineCourseVo:" + vo.toString();
			LOGGER.error(errMsg);
			throw new Exception(errMsg);
		}

		Sheet sheet = book.getSheetAt(0);
		if (null != sheet) {
			Row row;
			Cell cell;
			int lastRowNum = sheet.getLastRowNum();

			// 组装姓名：id map
			String accountName = null;
			List<String> names = new ArrayList<>(lastRowNum);
			for (int i = 0; i < lastRowNum; i++) {
				row = sheet.getRow(i + 1);
				if (null != row) {
					cell = row.getCell(0);
					if (null != cell) {
						// 设置单元格类型
						cell.setCellType(CellType.STRING);
						accountName = cell.getStringCellValue();
						names.add(accountName);
					}
				}
			}

			// Map<String, Long> accountNameIdMap = getAccountIdMap(context, names,
			// ModuleType.OFFLINE_COURSE);
			Map<String, Long> accountNameIdMap = null;
			return buildRecord(sheet, vo.getId(), context, now, accountNameIdMap, taskId);
		}
		return null;
	}

	private Map<String, List> buildRecord(Sheet sheet, Long offlineCourseId, RequestContext context, Date now,
			Map<String, Long> accountNameIdMap, Long taskId) throws Exception {
		Row row;
		Cell cell;
		int lastRowNum = sheet.getLastRowNum();

		Integer status = 0;
		Double score = null;
		Double integral = null;
		Date startTime = null;
		Date endTime = null;
		String accountName = null;
		String startDateFormat = null;
		String endDateFormat = null;
		String startTimeFormat = "00:00:00";
		String endTimeFormat = "00:00:00";
		int passCount = 0;
		int failCount = 0;
		
		List<TaskLogDetailEvent> failEvents = new ArrayList<TaskLogDetailEvent>();

		Map<String, List> result = new HashMap<>();
		List<OfflineRecorde> recordes = new ArrayList<>();
		List<String> errorLines = new ArrayList<>();
		Integer lastBatch = offlineRecordeService.getLastBatch(offlineCourseId, context.getSiteId(),
				context.getCompanyId());
		int currentBatch = lastBatch == null ? 0 : lastBatch + 1;

		// 查询站点下用户
		// List<String> siteUserNames = null;
		boolean isexec = false;
		try {
			accountNameIdMap = reportClient.findSiteAccountIds(context.getSiteId());
			if (!CollectionUtils.isEmpty(accountNameIdMap)) {
				isexec = true;
			}
		} catch (Exception e) {
			// LOGGER.info("查询站点下用户数量发送异常{}",e);
			errorLines.add("该次导入失败，没有查询到用户信息");
			failEvents.add(new TaskLogDetailEvent(taskId, "该次导入失败，用户不存在"));
		}

		List<String> listIsTrueOrFalse = new ArrayList<String>();
		listIsTrueOrFalse.add("是");
		listIsTrueOrFalse.add("否");

		if (isexec) {
			for (int i = 0; i < lastRowNum; i++) {
				try {
					status = 0;// -----
					score = null;// -----
					integral = null;// -----
					startTime = null;// --
					endTime = null;// -----
					accountName = null; // -----
					startDateFormat = null;
					endDateFormat = null;
					startTimeFormat = "00:00:00";
					endTimeFormat = "00:00:00";
					row = sheet.getRow(i + 2);
					if (null != row) {
						// 用户名
						cell = row.getCell(0);

						if (null != cell) {
							try {
								String accountNameStr = cell.getStringCellValue();
								LOGGER.info("用户名：" + accountNameStr);
								if (StringUtils.isEmpty(accountNameStr) || "0".equals(accountNameStr.trim())) {
									errorLines.add("第" + (i + 3) + "行用户名为空导入失败");
									LOGGER.error("第" + (i + 3) + "行用户名为空导入失败");
									throw new Exception("第" + (i + 3) + "行用户名为空导入失败");
								} else {
									if (!accountNameIdMap.containsKey(accountNameStr)) {
										errorLines.add("第" + (i + 3) + "行用户名不属于该站点,导入失败");
										LOGGER.error("第" + (i + 3) + "行用户名不属于该站点,导入失败");
										throw new Exception("第" + (i + 3) + "行用户名不属于该站点,导入失败");
									}
									// 获取用户是否存在
									accountName = accountNameStr;
								}
							} catch (Exception e) {
								String msg = e.getMessage();
								msg = StringUtils.indexOf(msg, "第") >= 0 ? msg : ("第" + (i + 3) + "行:" + "用户名导入异常");
								failEvents.add(new TaskLogDetailEvent(taskId, msg));
								LOGGER.error("用户名导入异常");
								throw new Exception("用户名导入异常");
							}

						} else {
							failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "用户名导入异常"));
							LOGGER.error("用户名导入异常");
							throw new Exception("用户名导入异常");
						}

						// 状态
						cell = row.getCell(1);

						if (null != cell) {
							try {
								String statusString = cell.getStringCellValue();
								LOGGER.info("状态：" + statusString);
								if (StringUtils.isEmpty(statusString)) {
									errorLines.add("第" + (i + 3) + "行状态为空导入失败");
									LOGGER.error("第" + (i + 3) + "行状态为空导入失败");
									throw new Exception("第" + (i + 3) + "行状态为空导入失败");
								} else if (!listIsTrueOrFalse.contains(statusString)) {
									errorLines.add("第" + (i + 3) + "行状态输入不符合规定导入失败");
									LOGGER.error("第" + (i + 3) + "行状态输入不符合规定导入失败");
									throw new Exception("第" + (i + 3) + "行状态输入不符合规定导入失败");
								} else if (statusString.equals("是")) {
									status = 1;
								}
							} catch (Exception e) {
								failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "状态导入异常"));
								LOGGER.error("第" + (i + 3) + "行:" + "状态导入异常");
								throw new Exception("状态异常");
							}
						} else {
							failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "状态导入异常"));
							LOGGER.error("第" + (i + 3) + "行状态为空导入失败");
							throw new Exception("获取cell内容信息异常");
						}

						// 分数
						cell = row.getCell(2);
						if (null != cell) {
							try {
								double scoreString = cell.getNumericCellValue();
								LOGGER.info("分数：" + scoreString);
								score = Double.valueOf(scoreString);
								LOGGER.info("格式转换后分数：" + score);
							} catch (Exception e) {
								failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "分数导入异常"));
								errorLines.add("第" + (i + 3) + "行分数导入异常");
								LOGGER.error("第" + (i + 3) + "行分数导入异常");
								throw new Exception("第" + (i + 3) + "行分数导入异常");
							}
						}

						// 积分
						cell = row.getCell(3);
						if (null != cell) {
							try {
								double pointString = cell.getNumericCellValue();
								LOGGER.info("积分：" + pointString);
								integral = Double.valueOf(pointString);
								if (null == integral) {
									integral = (double) 0;
								}
								LOGGER.info("格式转换后积分：" + integral);
							} catch (Exception e) {
								failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "积分导入异常"));
								errorLines.add("第" + (i + 3) + "行积分导入异常");
								LOGGER.error("第" + (i + 3) + "行积分导入异常");
								throw new Exception("第" + (i + 3) + "行积分导入异常");
							}
						}

						// 开始日期
						cell = row.getCell(4);
						if (null != cell) {
							try {
								Date startDateString = cell.getDateCellValue();
								if (null != startDateString) {
									LOGGER.info("开始日期：" + startDateString);
									SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
									startDateFormat = format.format(startDateString);
									LOGGER.info("格式转换后开始日期：" + startDateFormat);
								}
							} catch (Exception e) {
								failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "开始日期导入异常"));
								errorLines.add("第" + (i + 3) + "行开始日期导入异常");
								LOGGER.error("第" + (i + 3) + "行开始日期导入异常");
								throw new Exception("第" + (i + 3) + "行开始日期导入异常");
							}
						}

						// 开始时间
						cell = row.getCell(5);
						if (null != cell) {
							try {
								Date startTimeString = cell.getDateCellValue();
								if (null != startTimeString) {
									LOGGER.info("开始时间：" + startTimeString);
									SimpleDateFormat format = new SimpleDateFormat("HH:mm:ss");
									startTimeFormat = format.format(startTimeString);
									LOGGER.info("格式转换后开始时间：" + startTimeFormat);
								}else {
									startTimeFormat = "00:00:00";
								}
							} catch (Exception e) {
								failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "开始时间导入异常"));
								errorLines.add("第" + (i + 3) + "行开始时间导入异常");
								LOGGER.error("第" + (i + 3) + "行开始时间导入异常");
								throw new Exception("第" + (i + 3) + "行开始时间导入异常");
							}
						}

						// 结束日期
						cell = row.getCell(6);
						if (null != cell) {
							try {
								Date endDateString = cell.getDateCellValue();
	                        	if (null!=endDateString) {
								LOGGER.info("结束日期：" + endDateString);
								SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
								endDateFormat = format.format(endDateString);
								LOGGER.info("格式转换后结束日期：" + endDateFormat);
	                        	}
							} catch (Exception e) {
								failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "结束日期导入异常"));
								errorLines.add("第" + (i + 3) + "行结束日期导入异常");
								LOGGER.error("第" + (i + 3) + "行结束日期导入异常");
								throw new Exception("第" + (i + 3) + "行结束日期导入异常");
							}

						}

						// 结束时间
						cell = row.getCell(7);
						if (null != cell) {
							try {
								Date endTimeString = cell.getDateCellValue();
	                        	if (null!=endTimeString) {
								LOGGER.info("结束时间：" + endTimeString);
								SimpleDateFormat format = new SimpleDateFormat("HH:mm:ss");
								endTimeFormat = format.format(endTimeString);
								LOGGER.info("格式转换后结束时间：" + endTimeFormat);
	                        	}else {
	                        		endTimeFormat = "00:00:00";
								}
							} catch (Exception e) {
								failEvents.add(new TaskLogDetailEvent(taskId, "第" + (i + 3) + "行:" + "结束时间导入异常"));
								errorLines.add("第" + (i + 3) + "行结束时间导入异常");
								LOGGER.error("第" + (i + 3) + "行结束时间导入异常");
								throw new Exception("第" + (i + 3) + "行结束时间导入异常");
							}
						}
						
						Long accountId = accountNameIdMap.get(accountName);
						OfflineRecorde recorde = new OfflineRecorde();
						SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
						recorde.setId(idGenerator.generate());
						
						if (null!=startDateFormat) {
							String startString = startDateFormat + " " + startTimeFormat;
							startTime = format.parse(startString);
							recorde.setStartTime(startTime);
						}else {
							recorde.setStartTime(null);
						}
						if (null!=endDateFormat) {
							String endString = endDateFormat + " " + endTimeFormat;
							endTime = format.parse(endString);
							recorde.setEndTime(endTime);
						}else {
							recorde.setEndTime(null);
						}
						recorde.setAccountId(accountId);
						recorde.setAccountName(accountName);
						recorde.setOfflineCourseId(offlineCourseId);
						recorde.setStatus(status);
						recorde.setScore(score == null ? null : new Float(score));
						recorde.setIntegral(
								integral == null ? null : Integer.valueOf(Double.valueOf(integral).intValue()));
						recorde.setCreateTime(now);
						recorde.setOrgId(context.getOrgId());
						recorde.setSiteId(context.getSiteId());
						recorde.setCompanyId(context.getCompanyId());
						recorde.setBatch(currentBatch);
						recorde.setDeleted(0);
						recordes.add(recorde);
						passCount++;
					}

				} catch (Exception e) {
					// TODO Auto-generated catch block
					failCount++;
					errorLines.add("第" + (i + 3) + "行程序检查数据异常");
					LOGGER.error("第" + (i + 3) + "行程序检查数据异常");

				}
			}
		}
		// 错误的信息发送给任务详情表
		if (org.apache.commons.collections.CollectionUtils.isNotEmpty(failEvents)) {
			//rabbitTemplate.convertAndSend(LogQueue.BATCH_TASK_LOG_DETAIL_QUEUE, failEvents);
		}
		result.put("success", recordes);
		result.put("error", Collections.singletonList("成功导入" + passCount + "条，失败" + failCount + "条"));
		return result;
	}

	/**
	 * 查询以前导入的
	 *
	 * @param offlineCourseId
	 * @param siteId
	 * @param companyId
	 * @return
	 */
	private List<OfflineRecorde> queryOldRecords(Long offlineCourseId, Long siteId, Long companyId) {
		Integer lastBatch = offlineRecordeService.getLastBatch(offlineCourseId, siteId, companyId);
		// 之前没有导入过
		if (lastBatch == null) {
			return null;
		}

		OfflineRecorde offlineRecorde = new OfflineRecorde();
		offlineRecorde.setOfflineCourseId(offlineCourseId);
		offlineRecorde.setSiteId(siteId);
		offlineRecorde.setCompanyId(companyId);
		offlineRecorde.setDeleted(0);
		offlineRecorde.setBatch(lastBatch);

		List<OfflineRecorde> oldRecords = offlineRecorde.selectList(new EntityWrapper(offlineRecorde));
		return oldRecords;
	}

	/**
	 * 批量插入当前导入记录
	 *
	 * @param recordes
	 */
	private void insertNewAndHandleOld(List<OfflineRecorde> recordes, Long siteId, Long companyId) {
		if (!CollectionUtils.isEmpty(recordes)) {
			// 先查出之前最后一批次导入的
			List<OfflineRecorde> oldRecords = queryOldRecords(recordes.get(0).getOfflineCourseId(), siteId, companyId);

			if (null != oldRecords && oldRecords.size() > 0) {
				// 第一次记录中含有 "已完成" 状态的accountId,那么最新的记录中含有该accountId,就显示状态为 "已完成"
				Set<Long> oldIdsList = new HashSet<>(oldRecords.size());

				for (OfflineRecorde oldRecord : oldRecords) {
					if (oldRecord.getStatus().equals(1)) {
						oldIdsList.add(oldRecord.getAccountId());
					}
				}
				for (OfflineRecorde recorde : recordes) {
					if (oldIdsList.contains(recorde.getAccountId())) {
						recorde.setStatus(1);
					}
				}
			}
			// 插入当前导入的
			boolean flag = offlineRecordeService.insertBatch(recordes);
			// 循环当前记录，1. 向培训项目发送消息，2. 想积分发送消息
			OfflineCourse course = new OfflineCourse();
			course.setId(recordes.get(0).getOfflineCourseId());
			course = course.selectById();
			if (null == course) {
				LOGGER.error("批量插入线下课程记录错误,没有查询到线下课程, id: {}", recordes.get(0).getOfflineCourseId());
				return;
			}

			if (flag) {
				try {
					Date time = new Date();
					RequestContext requestContext = ContextHolder.get();
					PointParamVO pointParamVO = new PointParamVO();
					for (OfflineRecorde recorde : recordes) {
						Long accountId = recorde.getAccountId();
						if (recorde.getIntegral() != null && recorde.getIntegral() > 0) {

							pointParamVO.setEventName("pointOfflineCourse");
							pointParamVO.setActivityType("线下课程通过,积分发放");
							pointParamVO.setActivitySource("线下课程学习记录");
							pointParamVO.setActivityName(course.getName());
							pointParamVO.setReleaseCondition("通过才发放");
							pointParamVO.setReleaseRules("按照通过取积分");
							pointParamVO.setCreatePointTime(time);
							pointParamVO.setSourceId(recorde.getOfflineCourseId());
							pointParamVO.setOperatingPoint(recorde.getIntegral());
							pointParamVO.setAccountId(accountId);
							pointParamVO.setAccountName(recorde.getAccountName());
							pointParamVO.setOrgId(requestContext.getOrgId());
							pointParamVO.setSiteId(requestContext.getSiteId());
							pointParamVO.setCompanyId(requestContext.getCompanyId());
							String sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);
							// 发送积分
							if (StringUtils.isNotBlank(sitePointId)) {
								//amqpTemplate.convertAndSend("offlineCourse", sitePointId);
							}
						}

						EventWrapper<TrainingProjectEvent> eventWrapper = new EventWrapper<TrainingProjectEvent>(
								recorde.getOfflineCourseId(),
								TrainingProjectEvent.getInstance(recorde.getOfflineCourseId(),
										TpActivityType.TYPE_OFFLINE_COURSE, accountId, time, requestContext.getSiteId(),
										recorde.getStatus().equals(1)));
						// 发送培训异步消息
						cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE, eventWrapper);
					}
				} catch (Exception e) {
					LOGGER.error("线下课程导入发送异步消息出错：", e);
				}
			}
			// 如果之前的不为空
			if (flag && !CollectionUtils.isEmpty(oldRecords)) {

				Set<Long> oldIds = new HashSet<>(oldRecords.size());
				for (OfflineRecorde oldRecord : oldRecords) {
					oldIds.add(oldRecord.getId());
				}

				// 逻辑删除以前的记录
				OfflineRecorde recorde = new OfflineRecorde();
				recorde.setDeleted(1);
				EntityWrapper<OfflineRecorde> ew = new EntityWrapper<>();
				ew.in("id", oldIds);
				offlineRecordeService.update(recorde, ew);
			}
		}
	}

	/**
	 * 获取 用户名：用户 id map
	 *
	 * @param context
	 * @param names
	 * @param moduleType
	 * @return
	 */
	private Map<String, Long> getAccountIdMap(RequestContext context, List<String> names, String moduleType) {
		AccountNameForIdsModel model = new AccountNameForIdsModel(names, context);
		return accountClient.getAccountIdByNames(model, context.getCompanyId(), context.getSiteId(), moduleType);
	}

	@Data
	class CustomImporter extends AbstractTaskHandler {

		private Date date;

		private RequestContext context;

		private OfflineCourseVo vo;

		private Long taskId;

		public CustomImporter(Date date, RequestContext context, OfflineCourseVo vo, Long taskId) {
			this.date = date;
			this.context = context;
			this.vo = vo;
			this.taskId = taskId;
		}

		@Override
		public void handle() {
			this.publishTaskEvent(TaskLogEvent.working(this.taskContext));
			try {
				Map<String, List> map = analysisExcel(this.date, this.context, this.vo, this.taskId);
				insertNewAndHandleOld(map.get("success"), this.context.getSiteId(), this.context.getCompanyId());
				this.publishTaskEvent(
						TaskLogEvent.success(this.taskContext, map.get("error").toString(), vo.getOssUrl()));
			} catch (Exception e) {
				e.printStackTrace();
				this.publishTaskEvent(TaskLogEvent.fail(this.taskContext, e.getLocalizedMessage()));
			}
		}
	}

	// *********************************************** 导入 结束
	// ******************************************************* //

	// *********************************************** 导出 开始
	// ******************************************************* //
	private List<ReportOfflineCourseStatisticsDetailVo> getList(Date startTime, Date endTime, String code, String name,
																RequestContext context) {
		return offlineRecordeService.statisticsDetailExport(startTime, endTime, code, name, context);
	}

	private Template buildTemplate(RequestContext context, String name) {
		Template template = new Template();
		List<TemplateColumn> columns = new ArrayList<>();
		columns.add(new TemplateColumn("courseCode", "课程编码", 1));
		columns.add(new TemplateColumn("courseName", "课程名称", 2));
		columns.add(new TemplateColumn("accountNum", "学习人数", 3));
		columns.add(new TemplateColumn("createTime", "创建时间", 4));
		template.setColumns(columns);
		if (name == null || "".equals(name)) {
			template.setFileName("线下课程记录导出_" + DateUtil.getYMDHMSS() + ".xls");
		} else {
			template.setFileName(name + "_线下课程记录导出_" + DateFormatUtils.format(new Date(), "yyyyMMdd") + ".xls");
		}

		template.setBrowserExport(false);
		return template;
	}

	class CustomExporter extends ExcelExportor {

		private Date startTime;
		private Date endTime;
		private String code;
		private String name;
		private RequestContext context;

		public CustomExporter(Template template, List data, Date startTime, Date endTime, String code, String name,
							  RequestContext context) {
			super(template, data);
			this.startTime = startTime;
			this.endTime = endTime;
			this.code = code;
			this.name = name;
			this.context = context;
		}

		private void beforderRender() {
			this.data = getList(this.startTime, this.endTime, this.code, this.name, this.context);
			this.template = buildTemplate(this.context, this.name);
		}

		@Override
		public FileInfo render() {
			beforderRender();
			return super.render();
		}
	}
	// *********************************************** 导出 结束
	// ******************************************************* //

}
