package com.yizhi.course.application.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.domain.CourseStudentStatus;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.service.ICourseStudentStatusService;
import com.yizhi.course.application.vo.OutsideEbscnCourseRecordVO;
import com.yizhi.course.application.vo.OutsideEbscnCourseVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/outside/ebscn")
public class OutsideEbscnCourseController {

    @Autowired
    ICourseService courseService;

    @Autowired
    ICourseStudentStatusService courseStudentStatusService;

    public static final Logger LOGGER = LoggerFactory.getLogger(OutsideEbscnCourseController.class);

    @PostMapping("/study/record/get")
    public List<OutsideEbscnCourseRecordVO> getStudyRecord(@RequestBody OutsideEbscnCourseVO vo){
        Long siteId = vo.getSiteId();
        List<Long> courseIds = vo.getCourseIds();
        EntityWrapper<Course> ewCourse = QueryUtil.condition(new Course());
        ewCourse.in("id",courseIds).eq("site_id", siteId).isNotNull("total_duration");
        List<Course> coursesData = courseService.selectList(ewCourse);
        LOGGER.info("查询包含学时的课程数据={}", JSON.toJSONString(coursesData));
        if(!CollectionUtils.isEmpty(coursesData)){
            courseIds = coursesData.stream().map(courseIdItem -> courseIdItem.getId()).collect(Collectors.toList());
            EntityWrapper<CourseStudentStatus> ewCourseSs = QueryUtil.condition(new CourseStudentStatus());
            ewCourseSs.eq("site_id",siteId).eq("student_status",1).in("course_id",courseIds)
                    .in("account_id", vo.getAccountIds());
            List<CourseStudentStatus> courseStudentStatuses = courseStudentStatusService.selectList(ewCourseSs);
            LOGGER.info("查询学习完成的用户课程信息={}", JSON.toJSONString(courseStudentStatuses));
            if(courseStudentStatuses != null){
                // 获取每个人完成的课程信息
                Map<Long,List<CourseStudentStatus>> allMap =
                        courseStudentStatuses.parallelStream().collect(Collectors.groupingBy(groupKey -> groupKey.getAccountId()));
                // 对每个课程的学时进行字典转换
                Map<Long,Float> courserMap = coursesData.parallelStream()
                        .collect(Collectors.toMap(courseKey -> courseKey.getId(), courseVal -> courseVal.getTotalDuration()));

                List<OutsideEbscnCourseRecordVO> retData = new ArrayList<>();
                for(Map.Entry<Long,List<CourseStudentStatus>> item : allMap.entrySet()){
                    OutsideEbscnCourseRecordVO itemObj = new OutsideEbscnCourseRecordVO();
                    // 获取每个人的课程完成记录
                    Long accountId = item.getKey();
                    itemObj.setName(accountId.toString());
                    List<CourseStudentStatus> courseStudentStatusList = item.getValue();
                    float totalDuration = 0;
                    for (CourseStudentStatus studentStatus : courseStudentStatusList){
                        Long courseId = studentStatus.getCourseId();
                        if(courserMap.containsKey(courseId)){
                            totalDuration += courserMap.get(courseId).floatValue();
                        }
                    }
                    itemObj.setTrainhours(totalDuration);
                    retData.add(itemObj);
                }
                return retData;
            }
        }
        return null;
    }
}
