package com.yizhi.course.application.controller;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.yizhi.core.application.token.TokenHelper;
import com.yizhi.course.application.service.ICourseStudyHourService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.vo.AccountDurationVO;
import com.yizhi.course.application.vo.OrganizationDurationVO;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.AccountVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/rank")
public class RankListController {

	private static final Logger logger = LoggerFactory.getLogger(RankListController.class);

	@Autowired
	private AccountClient accountClient;

	@Autowired
	private OrganizationClient organizationClient;

	@Autowired
	private SiteClient siteClient;

	@Autowired
	private IRecordeService recordeService;

	@Autowired
	private TokenHelper tokenHelper;
	
    @Autowired
    private ICourseStudyHourService studyHourService;

	@GetMapping("/user/top")
	public AccountDurationVO userRank(@RequestParam(value = "accountId") Long accountId,
									  @RequestParam(value = "companyId") Long companyId, @RequestParam(value = "siteId") Long siteId,
									  @RequestParam(value = "top") Integer top) {
		logger.info("进入用户学时风云榜接口");
		// Long accountId = 1314l;
		// Long companyId = 1314l;
		// Long siteId = 1314l;
		Long chapterId = null;
		String rankingNum = null;
		AccountVO accountVO = accountClient.findById(accountId);
		AccountDurationVO accountDurationVO = new AccountDurationVO();
		Float duration = studyHourService.studyHourSum(siteId, accountId, null);
	    rankingNum = String.valueOf(recordeService.findRankingNumById(siteId, accountId));
		if (duration==null || duration==0) {
			duration=0f;
			rankingNum="未上榜";
		}
		logger.info("当前的排名："+rankingNum);
		accountDurationVO.setRankingNum(rankingNum);
		accountDurationVO.setTotalDuration(duration);
		accountDurationVO.setHeadPortrait(accountVO.getHeadPortrait());
		accountDurationVO.setName(accountVO.getFullName());
		accountDurationVO.setOrgName(accountVO.getOrgName());
		List<AccountVO> account = siteClient.getSitAccount(siteId, companyId);
		List<Long> accountIds = new ArrayList<Long>();
		Map<Long, Float> avgMap = new LinkedHashMap<Long, Float>();
		for (AccountVO vo : account) {
			accountIds.add(vo.getId());
		}
		List<OrganizationDurationVO> list = recordeService.getOrgDuration(siteId, top, accountIds);
		List<Long> orgIds = new ArrayList<Long>();
		Map<Long, Integer> accountCount = accountClient.getAccountCount(companyId);
		for (OrganizationDurationVO orgVO : list) {
			orgIds.add(orgVO.getOrgId());
			Float avgDuration = 0f;
			Integer count = accountCount.get(orgVO.getOrgId());
			if (count!=null&&count!=0) {
				avgDuration = orgVO.getSumDuration()/count;
				orgVO.setAvgDuration((float) (Math.round(avgDuration*10))/10);
			}else {
				orgVO.setAvgDuration(0f);
			}
		}
		Map<Long, String> map = organizationClient.getParentNameWithCurrentName(orgIds);
		for (OrganizationDurationVO orgVO : list) {
			orgVO.setOrgName(map.get(orgVO.getOrgId()));
		}
	       Collections.sort(list, new Comparator<OrganizationDurationVO>() {  
	    	   
	            public int compare(OrganizationDurationVO o1, OrganizationDurationVO o2) {  
	  
	                // 按照部门平均学时进行降序排列  
	                if (o1.getAvgDuration() > o2.getAvgDuration()) {  
	                    return -1;  
	                }  
	                if (o1.getAvgDuration() == o2.getAvgDuration()) {  
	                    return 0;  
	                }  
	                return 1;  
	            }  
	        });  
		accountDurationVO.setList(list.size()>top?list.subList(0, top):list);
		logger.info("返回用户学时风云榜"+accountDurationVO);
		return accountDurationVO;
	}

}
