package com.yizhi.course.application.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.service.IReportMkCourseService;
import com.yizhi.course.application.vo.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author: XieHaijun
 * @Description: 营销课程报表处理
 * @Date: Created in 16:37 2018/8/14
 * @Modified By
 */
@RestController
@RequestMapping("/marketing/course")
public class ReportMkCourseController {

    private Logger LOGGER = LoggerFactory.getLogger(MarketingCourseController.class);

    @Autowired
    private IReportMkCourseService reportMkCourseService;

    /**
     * 图标数据获取
     * @param startDate
     * @param endDate
     * @param type
     * @param pageSize
     * @param pageNo
     * @return
     */
    @GetMapping("/report/view")
    public Page<ReportMarketingCourseDataChartsVo> chartGroup(
            @RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,
            @RequestParam(name = "type", required = true, defaultValue = "1") Integer type,//value = "类型：1按日 2按周 3按月（默认按日）
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo){

        Long cid = ContextHolder.get().getCompanyId();
        Long siteId = ContextHolder.get().getSiteId();
        List<Long> orgs = ContextHolder.get().getOrgIds();

        Page<ReportMarketingCourseDataChartsVo> ret = reportMkCourseService.
                chartGroup(startDate, endDate, type, pageSize, pageNo, cid, siteId, orgs);
        //LiveActivityServiceImpl.searchPage
        //Page<Object> page = PageHelper.startPage(pageNo, pageSize);

        return ret;
    }

    @GetMapping("/report/course/group")
    public Page<ReportMarketingCourseVo> marketingCourseGroup(
            @RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,
            @RequestParam(name = "kwd", required = false) String kwd,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo){

        Long cid = ContextHolder.get().getCompanyId();
        Long siteId = ContextHolder.get().getSiteId();
        List<Long> orgs = ContextHolder.get().getOrgIds();

        boolean isPage = true;
        if(pageSize == null || pageSize==0){
            isPage = false;
        }

        if(StringUtils.isEmpty(StringUtils.trimToEmpty(kwd))){
            kwd = null;
        }
        Page<ReportMarketingCourseVo> result =
                reportMkCourseService.marketingCourseGroup(startDate,endDate,kwd,pageSize,pageNo,cid,siteId,orgs,isPage);

        return result;
    }

    @GetMapping("/report/course/group/view/list")
    public Page<MarketingCourseViewVo> marketingCourseViewList(
            @RequestParam(name = "courseId") Long courseId,
            @RequestParam(name = "nikeName",required = false) String nikeName,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo){

        return reportMkCourseService.marketingCourseViewList(courseId,nikeName,pageSize,pageNo);
    }



    @GetMapping("/report/account/group")
    public Page<ReportMarketingCourseAccountVo> marketingCourseAccountGroup(
            @RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,
            @RequestParam(name = "orgKwd", required = false) String orgKwd,//orgIds多个,号隔开
            @RequestParam(name = "accountKwd", required = false) String accountKwd,//accountIds多个,号隔开
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo){
        Long cid = ContextHolder.get().getCompanyId();
        Long siteId = ContextHolder.get().getSiteId();
        List<Long> orgs = ContextHolder.get().getOrgIds();

        if(StringUtils.isEmpty(StringUtils.trimToEmpty(orgKwd))){
            orgKwd = null;
        }

        if(StringUtils.isEmpty(StringUtils.trimToEmpty(accountKwd))){
            accountKwd = null;
        }

        List<Long> orgKwds= null;
        if(StringUtils.isNotBlank(orgKwd)){
            orgKwds = Stream.of(orgKwd.split(",")).map(item->Long.valueOf(item)).collect(Collectors.toList());
        }
        List<Long> accountKwds = null;
        if(StringUtils.isNotBlank(accountKwd)){
            accountKwds = Stream.of(accountKwd.split(",")).map(item->Long.valueOf(item)).collect(Collectors.toList());
        }
        boolean isPage = true;
        if(pageSize == null || pageSize==0){
            isPage = false;
        }
        Page<ReportMarketingCourseAccountVo> result =
                reportMkCourseService.marketingCourseAccountGroup(startDate,endDate,
                        orgKwds, accountKwds,
                        pageSize,pageNo,
                        cid, siteId, orgs,isPage);
        return result;
    }

    @PostMapping("/report/account/group/export")
    public Page<ReportMarketingCourseAccountVo> marketingCourseAccountGroupExp(
            @RequestBody MkCourseExportRepVO vo){
        LOGGER.info("***************按用户导出传入参数="+ JSON.toJSONString(vo));
        RequestContext requestContext = vo.getRequestContext();
        Long cid = requestContext.getCompanyId();
        Long siteId = requestContext.getSiteId();
        List<Long> orgs = requestContext.getOrgIds();

        List<Long> orgKwds= null;
        String orgKwd = vo.getKwd();
        if(StringUtils.isNotBlank(orgKwd)){
            orgKwds = Stream.of(orgKwd.split(",")).map(item->Long.valueOf(item)).collect(Collectors.toList());
        }
        List<Long> accountKwds = null;
        String accountKwd = vo.getAccountKwd();
        if(StringUtils.isNotBlank(accountKwd)){
            accountKwds = Stream.of(accountKwd.split(",")).map(item->Long.valueOf(item)).collect(Collectors.toList());
        }
        boolean isPage = false;

        Page<ReportMarketingCourseAccountVo> result =
                reportMkCourseService.marketingCourseAccountGroup(vo.getStartDate(),vo.getEndDate(),
                        orgKwds, accountKwds,
                        null,null,
                        cid, siteId, orgs,isPage);
        return result;
    }

    @PostMapping("/report/course/group/export")
    public Page<ReportMarketingCourseVo> marketingCourseGroupExp(
            @RequestBody MkCourseExportRepVO vo){
        RequestContext requestContext = vo.getRequestContext();
        LOGGER.info("***************按课程导出传入参数="+ JSON.toJSONString(vo));
        Long cid = requestContext.getCompanyId();
        Long siteId = requestContext.getSiteId();
        List<Long> orgs = requestContext.getOrgIds();

        boolean isPage = false;
        String kwd = vo.getKwd();
        if(StringUtils.isEmpty(StringUtils.trimToEmpty(kwd))){
            kwd = null;
        }
        Page<ReportMarketingCourseVo> result =
                reportMkCourseService.marketingCourseGroup(vo.getStartDate(),
                        vo.getEndDate(),
                        kwd,null,null,cid,siteId,orgs,isPage);

        return result;
    }

}
