package com.yizhi.course.application.domain;

import com.baomidou.mybatisplus.activerecord.Model;
import com.baomidou.mybatisplus.annotations.TableField;
import com.baomidou.mybatisplus.annotations.TableName;
import com.baomidou.mybatisplus.enums.FieldFill;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 课程表
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Data
@Api(tags = "Course", description = "课程表")
@TableName("course")
public class Course extends Model<Course> {

    private static final long serialVersionUID = 1L;


    private Long id;

    @ApiModelProperty(value = "课程名称")
    private String name;

    @ApiModelProperty(value = "课程code")
    private String code;

    @ApiModelProperty(value = "课程封面图")
    private String image;

    @ApiModelProperty(value = "作者单位")
    @TableField("author_unit")
    private String authorUnit;

    @ApiModelProperty(value = "课程介绍")
    private String description;

    @ApiModelProperty(value = "课程类型 : 0 一般课程  1 Scorm课程")
    @TableField("course_type")
    private Integer courseType;

    private String tags;

    @TableField(value = "create_by_id", fill = FieldFill.INSERT)
    private Long createById;

    @TableField(value = "create_by_name", fill = FieldFill.INSERT)
    private String createByName;

    @TableField(value = "create_time", fill = FieldFill.INSERT)
    private Date createTime;

    @TableField(value = "update_by_id", fill = FieldFill.UPDATE)
    private Long updateById;

    @TableField(value = "update_by_name", fill = FieldFill.UPDATE)
    private String updateByName;

    @TableField(value = "update_time", fill = FieldFill.UPDATE)
    private Date updateTime;

    @TableField(value = "release_time")
    private Date releaseTime;

    @ApiModelProperty(value = "0 未上架 1 已上架")
    private Integer shelves;

    @TableField("company_id")
    private Long companyId;

    @TableField("org_id")
    private Long orgId;

    @TableField("site_id")
    private Long siteId;

    @ApiModelProperty(value = "分类id")
    @TableField("classify_id")
    private Long classifyId;

    @ApiModelProperty(value = "可见范围 0 游客 1 平台用户 2 指定范围")
    private Integer scope;

    @ApiModelProperty(value = "评论设置0上架评论及评分1关闭评论")
    @TableField("comment_enable")
    private Integer commentEnable;

    @ApiModelProperty(value = "下载设置0允许用户在APP端下载课件1禁止下载")
    @TableField("down_enable")
    private Integer downEnable;

    @ApiModelProperty(value = "播放设置 0允许用户未完成观看前拖动进度条1禁止拖动")
    @TableField("fast_forward_enable")
    private Integer fastForwardEnable;

    @ApiModelProperty(value = "章节播放设置0自动播放下一段1手动播放下一段")
    @TableField("auto_enable")
    private Integer autoEnable;

    @ApiModelProperty(value = "跳播设置：0允许1不允许")
    @TableField("jump_enable")
    private Integer jumpEnable;

    @ApiModelProperty(value = "评分设置 0 允许 1 不允许")
    @TableField("grade_enable")
    private Integer gradeEnable;

    @ApiModelProperty(value = "音频完成条件 0 播放即完成 1听完整个课程")
    @TableField("audio_complete")
    private Integer audioComplete;

    @ApiModelProperty(value = "视频完成条件 0 播放即完成 1听完整个课程")
    @TableField("video_complete")
    private Integer videoComplete;

    @ApiModelProperty(value = "文档完成条件 0 观看即完成 1停留时长达到学习时长即完成")
    @TableField("doc_complete")
    private Integer docComplete;

    @ApiModelProperty(value = "多媒体完成条件 0 观看即完成 1停留时长达到学习时长即完成")
    @TableField("rich_complete")
    private Integer richComplete;

    @ApiModelProperty(value = "来源 1内部定制2 外部定制3外部采购 4课程营销")
    @TableField("source")
    private Integer source;

    @ApiModelProperty(value = "0:默认未删除；1表示删除")
    @TableField("del_flg")
    private Integer delFlg;

    @ApiModelProperty(value = "被浏览次数")
    @TableField("read_count")
    private Long readCount;

    @ApiModelProperty(value = "积分规则  1按课程  2 按章节 3 按素材")
    @TableField("port_rule")
    private Integer portRule;

    @ApiModelProperty(value = "课程分类名称")
    @TableField(exist=false)
    private String classifyName;

    @ApiModelProperty(value = "0.过期,1.有效")
    @TableField("valid")
    private Integer valid;

    @ApiModelProperty(value = "名片是否显示 0(false):不显示 ；1(true):显示")
    @TableField("card_show")
    private Boolean cardShow;

    @ApiModelProperty(value = "分享标题")
    @TableField("share_title")
    private String shareTitle;

    @ApiModelProperty(value = "分享描述")
    @TableField("share_desc")
    private String shareDesc;

    @ApiModelProperty(value = "课程学时")
    @TableField("total_duration")
    private Float totalDuration;

    @ApiModelProperty(value = "课程学时是否发放: true表示发放，false表示不发放")
    @TableField("total_duration_grant")
    private Boolean totalDurationGrant;

    @ApiModelProperty(value = "课程素材总时间")
    @TableField(exist=false)
    private Integer duration;

    @ApiModelProperty(value = "选择发布终端:0 所有终端;1 仅发布到pc端;2 仅发布到移动端")
    @TableField("publishing_terminal")
    private Integer publishingTerminal;

    @ApiModelProperty(value = "视频播放模式：0 横屏；1竖屏")
    @TableField("video_play_model")
    private Integer videoPlayModel;

    @ApiModelProperty(value = "是否启用在日历任务中显示")
    @TableField("enable_task")
    private Integer enableTask;

    @ApiModelProperty(value = "日历任务设置的开始时间")
    @TableField("task_start_date")
    private Date taskStartDate;

    @ApiModelProperty(value = "日历任务设置的结束时间")
    @TableField("task_end_date")
    private Date taskEndDate;

    @ApiModelProperty(value = "是否发放证书： true 配置开启证书，false 不开启证书")
    @TableField("grant_certificate")
    private Boolean grantCertificate;

    @ApiModelProperty(value = "证书id列表")
    @TableField("certificates")
    private String certificates;

    @ApiModelProperty(value = "营销课程的课程类型 【分类3种类型：多素材课程=0（默认）、章节课程=1、直播=2】")
    @TableField("marketing_course_type")
    private Integer marketingCourseType;

    @ApiModelProperty(value = "手机号授权设置 【启用授权 true默认启用授权、关闭授权:false 】")
    @TableField("tel_authorize")
    private Boolean telAuthorize;

    @ApiModelProperty(value = "营销课程章节类型的课程图片")
    @TableField("marketing_course_image")
    private String marketingCourseImage;

    @Override
    protected Serializable pkVal() {
        return this.id;
    }

}
