package com.yizhi.course.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.yizhi.course.application.domain.Chapter;
import com.yizhi.course.application.vo.ChapterDetailedInfoVO;
import com.yizhi.course.application.vo.ChapterDetailsVo;
import com.yizhi.course.application.vo.ChapterJoinTestVo;
import com.yizhi.course.application.vo.ChapterVo;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;
import java.util.Set;

/**
 * <p>
 * 课程章节 Mapper 接口
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
public interface ChapterMapper extends BaseMapper<Chapter> {

    @Select("<script>" +
            "SELECT c.id,c.material_type AS materialType,c.course_id AS courseId,c.material_id AS materialId," +
            " ce.audio_complete AS audioComplete ,ce.video_complete AS videoComplete ,ce.doc_complete AS docComplete,ce.rich_complete AS richComplete," +
            " m.text_hour AS textHour,m.text_minute AS textMinute,m.text_second AS textSecond" +
            " FROM (SELECT id,material_type,course_id,material_id FROM chapter WHERE id = #{chapterId}) c" +
            " LEFT JOIN course ce ON c.course_id = ce.id LEFT JOIN material m ON c.material_id=m.id "+
            "</script>")
    ChapterDetailedInfoVO getChapterDetailedInfo(@Param("chapterId") Long chapterId);

    @Select("<script>" +
            "SELECT course_id FROM chapter WHERE material_id IS NOT NULL AND material_type IS NOT NULL AND course_id IN " +
            "<foreach collection=\"ids\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">"+
            "#{cid}"+
            "</foreach>"+
            " GROUP BY course_id" +
            "</script>")
    List<Long> dataLack(@Param("ids") List<Long> ids);

    @Select("<script>" +
            "SELECT course_id AS courseId , IFNULL(SUM(score),0) AS score FROM chapter WHERE material_id IS NOT NULL AND course_id IN " +
            "<foreach collection=\"ids\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">"+
            "#{cid}"+
            "</foreach>"+
            " GROUP BY course_id"+
            "</script>")
    List<ChapterVo> getCourseScoreByIds(@Param("ids") List<Long> ids);

    Chapter lockById(@Param("chapterId") Long chapterId);

    Integer getSort(@Param("parentId") Long parentId);

    List<Chapter> listByCourseId(@Param("courseId") Long courseId);

    List<ChapterDetailsVo> selectChapterList(@Param("courseId") Long courseId, @Param("accountId") Long accountId);

    List<ChapterDetailsVo> selectChapterListUp(@Param("courseId") Long courseId, @Param("accountId") Long accountId);

    List<ChapterDetailsVo> selectChapterListMarketing(@Param("courseId") Long courseId);

    List<ChapterDetailsVo> selectChapterListUpOptimize(@Param("courseId") Long courseId, @Param("accountId") Long accountId);

    Integer selectSumScoreByParentId(@Param("id") Long id);

    Integer getMaxIndex(@Param("courseId") Long courseId);

    /**
     * 获取需要完成的章节（有的章节可能仅仅是一层结构，不需要查出）
     *
     * @param courseId
     * @return
     */
    Set<Long> getIdsByCourseId(@Param("courseId") Long courseId);

    /**
     * 查询章节关联素材的次数
     * @param materialIds
     * @return
     */
    @Select("<script>" +
            "SELECT c.material_id AS materialId, count(0) AS countNum FROM chapter c LEFT JOIN course ce ON c.course_id = ce.id WHERE ce.del_flg=0 AND c.material_id in " +
            "<foreach collection=\"materialIds\" open=\"(\" close=\")\" separator=\",\" item=\"mid\" index=\"index\">"+
            "#{mid}"+
            "</foreach>"+
            " GROUP BY c.material_id"+
            "</script>")
    List<ChapterJoinTestVo> selectChapterTest(@Param("materialIds") List<Long> materialIds);

    /**
     * 查询课程完成的章节
     * @param courseId
     * @param accountId
     * @param siteId
     * @return
     */
    List<Chapter> selectCourseFinishChapter(@Param("courseId") Long courseId,
                                          @Param("accountId") Long accountId,
                                          @Param("siteId") Long siteId);

    /**
     * 查询我的课程学习的章节信息
     * @param courseIds
     * @param accountId
     * @param siteId
     * @return
     */
    List<Chapter> selectStudyCourseChapter(
                                    @Param("courseIds") List<Long> courseIds,
                                    @Param("accountId") Long accountId,
                                    @Param("siteId") Long siteId);
    
    Integer getScoreByCourseId(@Param("courseId") Long courseId);

    /**
     * 查询学习完成的章节 ，根据完成查询完成的章节，并根据章节分组
     * @param accountId
     * @param chapterIds
     * @return
     */
    @Select("<script>" +
            "SELECT chapter_id FROM recorde WHERE account_id = #{accountId} AND chapter_id IN " +
            "<foreach collection=\"chapterIds\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">"+
            "#{cid}"+
            "</foreach>"+
            " AND overed = 1 GROUP BY chapter_id"+
            "</script>")
    List<Long> getFinishChapterIds(@Param("accountId") Long accountId, @Param("chapterIds") List<Long> chapterIds);

    /**
     * 获取课程下素材ids
     * @param courseId 课程id
     */
    List<Long> selectMaterialIdsByCourse(@Param("courseId") Long courseId);
}
