package com.yizhi.course.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.domain.ViewCourseCount;
import com.yizhi.course.application.vo.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.session.RowBounds;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 课程表 Mapper 接口
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
public interface CourseMapper extends BaseMapper<Course> {

    Long getTotalDuration(@Param("courseId") Long courseId);

    @Select("SELECT c.grant_certificate grantCertificate, c.certificates AS certificateStr, c.video_play_model videoPlayModel,c.publishing_terminal publishingTerminal,c.total_duration_grant totalDurationGrant,c.total_duration totalDuration," +
            " c.id,c.`name`,c.`code`,c.image,c.author_unit AS authorUnit,c.tags,c.shelves,c.create_by_id AS createById,c.create_by_name AS createByName,c.create_time AS createTime,c.update_by_id AS updateById," +
            " c.update_by_name AS updateByName,c.update_time AS updateTime,c.company_id AS companyId,c.org_id AS orgId,c.site_id AS siteId,c.classify_id AS classifyId,c.scope,c.comment_enable AS commentEnable," +
            " c.down_enable AS downEnable,c.fast_forward_enable AS fastForwardEnable,c.auto_enable AS autoEnable,c.jump_enable AS jumpEnable,c.grade_enable AS gradeEnable,c.audio_complete AS audioComplete," +
            " c.video_complete AS videoComplete,c.doc_complete AS docComplete,c.rich_complete AS richComplete,c.source,c.del_flg AS delFlg,c.read_count AS readCount," +
            " c.port_rule AS portRule,c.valid,IFNULL(c.course_type,0) AS courseType,c.old_id AS oldId,c.release_time AS releaseTime,cd.description" +
            " FROM course AS c LEFT JOIN course_description AS cd ON cd.course_id = c.id WHERE c.id = #{id}")
    CourseVo getCourseDetails(@Param("id") Long id);

    List<Course> searchPage(@Param("name") String name, @Param("classifyId") Long classifyId, @Param("shelves") Integer shelves,
                            @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgIds") List<Long> orgIds, @Param("courseIds") List<Long> courseIds,
                            @Param("type") Integer type, Page<Course> page, @Param("courseType") Integer courseType);

    List<Course> searchPageByClassifyAllMg(@Param("name") String name, @Param("classifyIds") List<Long> classifyIds, @Param("shelves") Integer shelves,
                            @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgIds") List<Long> orgIds, @Param("courseIds") List<Long> courseIds,
                            @Param("type") Integer type, Page<Course> page, @Param("courseType") Integer courseType);

    Integer searchPageCount(@Param("name") String name, @Param("classifyId") Long classifyId, @Param("shelves") Integer shelves,
                            @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgIds") List<Long> orgIds, @Param("courseIds") List<Long> courseIds,
                            @Param("type") Integer type);


    Integer vailCode(@Param(value = "id") Long id, @Param(value = "code") String code, @Param(value = "siteId") Long siteId);
    /**
     * 查询code的课程数
     * @param id 课程id
     * @param code 课程编码
     * @param siteId 站点id
     */
    Integer vailCodeExclMarketing(@Param(value = "id") Long id, @Param(value = "code") String code,
                                @Param(value = "siteId") Long siteId);

    /**
     * 获取课程总时长
     *
     * @return
     */
    List<ApiListCourseVo> getCourseDurationByIds(@Param("ids") List<Long> ids);

    List<ApiListCourseVo> apiList(@Param("name") String name, @Param("classifyId") Long classifyId, @Param("type") Integer type,
                                  @Param("ids") List<Long> ids,
                                  @Param("companyId") Long companyId, @Param("siteId") Long siteId, RowBounds rowBounds);

    List<ApiListCourseVo> searchPageSubClassify(@Param("name") String name, @Param("classifyIds") List<Long> classifyIds, @Param("type") Integer type,
                                                @Param("ids") List<Long> ids,
                                                @Param("companyId") Long companyId, @Param("siteId") Long siteId, RowBounds rowBounds);

    Integer apiListCount(@Param("name") String name, @Param("classifyId") Long classifyId, @Param("type") Integer type,
                         @Param("ids") List<Long> ids,
                         @Param("companyId") Long companyId, @Param("siteId") Long siteId);

    Double getAvgLevel(@Param("courseId") Long courseId);

    List<ProjectCourseVo> searchProjectCoursePage(@Param("trainingProjectId") Long trainingProjectId, @Param("name") String name, @Param("classifyId") Long classifyId,
                                                  @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                  @Param("orgId") String[] orgId, RowBounds rowBounds);

    Integer searchProjectCoursePageCount(@Param("trainingProjectId") Long trainingProjectId, @Param("name") String name, @Param("classifyId") Long classifyId,
                                         @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                         @Param("orgId") String[] orgId);

    List<HomePageCoursePageVo> homePageCoursePageList(@Param("name") String name, @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                      @Param("ids") List<Long> ids, RowBounds rowBounds);

    List<HomePageCoursePageVo> homePageCoursePageListNew(@Param("name") String name, @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                         @Param("ids") List<Long> ids, RowBounds rowBounds);

    /**
     * 根据分类ID查询最新的课程
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param ids
     * @param rowBounds
     * @return
     */
    List<ApiListCourseVo> classifyTopCourseList(@Param("classifyId") Long classifyId, @Param("name") String name, @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                @Param("ids") List<Long> ids, RowBounds rowBounds);

    /**
     * 分页查询课程分类信息(新)
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param ids
     * @param rowBounds
     * @return
     */
    List<HomePageCourseVo> homePageClassifyList(@Param("name") String name, @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                @Param("ids") List<Long> ids, RowBounds rowBounds);


    Integer getRecordeDuration(@Param("chapterId") Long chapterId, @Param("accountId") Long accountId);


    List<Course> selectCourseList(@Param("name") String name, @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgId") Long orgId);


    List<ApiListCourseVo> selectOfflineCourseList(@Param("ids") List<Long> ids, RowBounds rowBounds);

    List<Course> selectCourseByMaterialId(@Param("materialId") Long materialId);

    //获取课程浏览人次
    List<ViewCourseCount> courseViewCount(@Param("id") Long id);

    Integer courseViewCountNum(@Param("id") Long id);

    //根据课程id 获取课程评分和(课程学习人数)课程浏览人数--
    HomePageCoursePageVo findCourseById(@Param("courseId") Long courseId);


    /**
     * 为简化sql新写的接口--homePageCourseNewPageList
     *
     * @param companyId
     * @param siteId
     * @param ids
     * @param rowBounds
     * @return
     */
    List<HomePageCoursePageVo> homePageCourseNewPageList(@Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                         @Param("ids") List<Long> ids, RowBounds rowBounds);

    List<AuthCourseVO> searchAuthorizePage(@Param("name") String name, @Param("classifyIds") List<Long> classifyIds,
                                           @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgIds") List<Long> orgIds, @Param("courseIds") List<Long> courseIds, RowBounds rowBounds);

    Integer searchAuthorizePageCount(@Param("name") String name, @Param("classifyIds") List<Long> classifyIds,
                                     @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgIds") List<Long> orgIds, @Param("courseIds") List<Long> courseIds);

    List<Long> getCourseIdList(@Param(value = "companyId") Long companyId,
                               @Param(value = "siteId") Long siteId,
                               @Param(value = "courseIds") List<Long> courseIds, RowBounds rowBounds);

    List<HomePageCoursePageVo> findHomeCourseList(@Param(value = "courseIds") List<Long> courseIds);

    List<HomePageCoursePageVo> findCourseList(@Param(value = "courseIds") List<Long> courseIds);

    List<HomePageCoursePageVo> findPcCourseList(@Param(value = "courseIds") List<Long> courseIds);

    @Select("<script>" +
            "SELECT cg.course_id AS courseId ,ROUND(AVG(cg.level), 1) AS level " +
            " FROM course_grade cg " +
            " WHERE cg.course_id IN " +
            "<foreach collection=\"courseIds\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">" +
            "#{cid} " +
            "</foreach>" +
            " GROUP BY cg.course_id" +
            "</script>")
    List<HomePageCoursePageVo> findCourseLevelList(@Param(value = "courseIds") List<Long> courseIds);

    @Select("<script>" +
            "SELECT cvr.course_id AS courseId ,count(DISTINCT(cvr.account_id)) AS studyNum " +
            " FROM course_view_record cvr " +
            " WHERE cvr.course_id IN " +
            "<foreach collection=\"courseIds\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">" +
            "#{cid} " +
            "</foreach>" +
            " GROUP BY cvr.course_id" +
            "</script>")
    List<HomePageCoursePageVo> findCourseStudyNumList(@Param(value = "courseIds") List<Long> courseIds);

    @Select("<script>" +
            "SELECT " +
            " c.id courseId,c.`name`,c.image,c.author_unit authorUnit,c.classify_id classifyId,c.valid,c.`code`, " +
            " IFNULL( cpv.click_count, 0 ) AS clickCount," +
            " IFNULL( css.student_status, - 1 ) AS studentStatus " +
            " FROM" +
            " course c" +
            " LEFT JOIN course_page_view cpv ON c.company_id = cpv.company_id" +
            " AND c.site_id = cpv.site_id" +
            " AND c.id = cpv.course_id" +
            " LEFT JOIN ( SELECT course_id, student_status FROM course_student_status" +
            " WHERE company_id = #{companyId} AND site_id = #{siteId} AND account_id = #{accountId} )" +
            " css ON c.id = css.course_id" +
            " WHERE c.company_id = #{companyId} AND c.site_id = #{siteId} " +
            " AND c.id IN" +
            "<foreach collection=\"courseIds\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">" +
            "#{cid} " +
            "</foreach>" +
            " ORDER BY studentStatus,clickCount DESC,id" +
            "</script>")
    List<HomePageCoursePageVo> findRecommendCourseList(@Param(value = "courseIds") List<Long> courseIds,
                                                       @Param(value = "companyId") Long companyId,
                                                       @Param(value = "siteId") Long siteId,
                                                       @Param(value = "accountId") Long accountId);

    List<HomePageCoursePageVo> findWechatCourseList(@Param(value = "courseIds") List<Long> courseIds);

    List<ApiListCourseVo> findWechatCourseInfoList(@Param(value = "courseIds") List<Long> courseIds, @Param(value = "classifyIds") List<Long> classifyIds);

    List<ApiListCourseVo> findWechatCourseLevelList(@Param(value = "courseIds") List<Long> courseIds);

    List<Map<String, Object>> experiencePoolList(@Param("name") String name, @Param("ids") List<Long> ids,
                                                 @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                 @Param("orgIds") List<Long> orgIds, Page<Map<String, Object>> page);


    List<ModuleAllVoCourse> listCoursePCgrade(@Param("ids") List<Long> ids);

    List<Long> queryCourseByRelationId(@Param("relationIds") List<Long> relationIds, @Param("siteId") Long siteId, @Param("num") Integer num, @Param("listIds") List<Long> listIds);

    @Select(" SELECT id FROM course WHERE id IN (" +
            " SELECT DISTINCT course_id " +
            " FROM recorde WHERE site_id = #{siteId} and account_id = #{accountId} " +
            " AND course_id IS NOT NULL AND course_id > 0 AND chapter_id IS NOT NULL AND chapter_id > 0" +
            " ) and del_flg = 0 and shelves=1")
    List<Long> getStudyCourseId(@Param("siteId") Long siteId, @Param("accountId") Long accountId);

    /**
     * 获取可见用户可见范围的课程
     *
     * @param courseIds
     * @return
     */
    List<Course> getCourseByIds(@Param(value = "courseIds") List<Long> courseIds);


    @Select("SELECT DISTINCT(site_id) FROM `course` WHERE site_id IS NOT NULL ")
    List<Long> getAllSiteId();


    List<CourseListExportVO> listExport(@Param("name") String name, @Param("classifyId") Long classifyId, @Param("shelves") Integer shelves,
                                        @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgIds") List<Long> orgIds, @Param("courseIds") List<Long> courseIds,
                                        @Param("type") Integer type, @Param("courseType") Integer courseType);

    /**
     * o
     * 自定义项目获取课程log
     *
     * @param courseIds
     * @return
     */
    List<CourseLogoVO> getCourseLogos(@Param("courseIds") List<Long> courseIds);

    /**
     * 根据条件查询数据
     *
     * @param finishCourseIds
     * @param courseIdsFromVisibRange
     * @param currentDate
     * @param siteId
     * @param page
     * @return
     */

    List<Course> getPageToCalendar(@Param("finishCourseIds") List<Long> finishCourseIds,
                                   @Param("courseIdsFromVisibRange") List<Long> courseIdsFromVisibRange,
                                   @Param("currentDate") Date currentDate,
                                   @Param("siteId") Long siteId,
                                   Page page);

    Integer getPageToCalendarNum(@Param("finishCourseIds") List<Long> finishCourseIds,
                                 @Param("courseIdsFromVisibRange") List<Long> courseIdsFromVisibRange,
                                 @Param("currentDate") Date currentDate,
                                 @Param("siteId") Long siteId);

    List<Long> getIdsByDate(@Param("currentDate") Date currentDate, @Param("siteId") Long siteId);

    /**
     * 嘉兴银行闯关排行
     *
     * @param courseId
     * @param page
     * @return
     */
    List<JaxinRankingVO> getRankings(@Param("courseId") Long courseId, Page<JaxinRankingVO> page);

    /**
     * v2版本
     * @param courseIds 课程id
        @param hotDesc 人气最高 排序 默认null  人气相同按照最后一次修改时间倒叙
     */
    List<HomePageCoursePageVo> findPcCourseListByHotDesc(@Param("courseIds") List<Long> courseIds,@Param("hotDesc") Boolean hotDesc);
    /**
     * v2版本
     * @param courseIds 课程id
      ** @param gradeDesc 好评优先 好评相同；按照最后一次修改时间倒叙
     */
    List<HomePageCoursePageVo> findPcCourseListByGradeDesc(@Param("courseIds") List<Long> courseIds,@Param("gradeDesc") Boolean gradeDesc);

    /**
     * 获取导出素材的课程ids
     * @param name
     * @param classifyId
     * @param shelves
     * @param companyId
     * @param siteId
     * @param orgIds
     * @param courseIds
     * @param type
     * @param courseType
     * @return
     */
    List<Long> courseIdlistExport(@Param("name") String name, @Param("classifyId") Long classifyId, @Param("shelves") Integer shelves,
                                        @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgIds") List<Long> orgIds, @Param("courseIds") List<Long> courseIds,
                                        @Param("type") Integer type, @Param("courseType") Integer courseType);


}
