package com.yizhi.course.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.yizhi.course.application.domain.CourseStudentStatus;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 课程学习状态 Mapper 接口
 * </p>
 *
 * @author 谢海军123
 * @since 2019-04-01
 */
public interface CourseStudentStatusMapper extends BaseMapper<CourseStudentStatus> {

    @Select("SELECT course_id AS courseId,account_id AS accountId,site_id AS siteId FROM send_course_finish")
    List<CourseStudentStatus> getTrainningMqInfo();

    @Select("<script>" +
            "SELECT css.id,css.company_id companyId,css.site_id siteId,css.org_id orgId,css.account_id accountId," +
            " css.course_id courseId,css.student_status studentStatus,css.last_student_time lastStudentTime,css.percent" +
            " FROM course_student_status css LEFT JOIN course c " +
            " on css.course_id = c.id" +
            " WHERE css.account_id = #{accountId} AND css.site_id = #{siteId} and c.shelves = 1 and c.del_flg = 0" +
            "<if test='startDate != null and endDate != null'>" +
            " AND css.first_finish_time between  #{startDate} and  #{endDate} " +
            "</if>" +
            "</script>")
    List<CourseStudentStatus> findValidStudyCourse(@Param("siteId") Long siteId,
                                                   @Param("accountId") Long accountId,
                                                   @Param("startDate") Date startDate,
                                                   @Param("endDate") Date endDate);

    @Select("<script>" +
            "SELECT count(*) " +
            "FROM course_student_status " +
            "WHERE site_id = #{siteId} " +
            "AND <![CDATA[ DATE_FORMAT(first_finish_time, '%Y-%m-%d') >= DATE_FORMAT(#{startDate}, '%Y-%m-%d') ]]> " +
            "AND <![CDATA[ DATE_FORMAT(first_finish_time, '%Y-%m-%d') <= DATE_FORMAT(#{endDate}, '%Y-%m-%d') ]]> " +
            "</script>")
    Integer courseFinishInWeek(@Param("siteId") long siteId, @Param("startDate") Date startDate,
                                                 @Param("endDate") Date endDate);

    /**
     * 根据accountId查询该学员已完成课程id
     * @param siteId
     * @param accountId
     * @return
     */
    @Select("<script>" +
            "SELECT " +
            " course_id " +
            " FROM course_student_status "+
            " WHERE account_id = #{accountId} AND site_id = #{siteId} and student_status = 1" +
            "<if test=' ids != null and ids.size > 0'>" +
            " AND course_id in  " +
            " <foreach collection='ids' item='id'  open='(' close=')' separator=','> " +
            "         #{id}" +
            " </foreach> " +
            "</if>" +
            "</script>")
    List<Long> getCourseIds(@Param("siteId") Long siteId, @Param("accountId") Long accountId, @Param("ids")List<Long> ids);




    @Select("<script>" +
            "SELECT\n" +
            "\tcss.course_id AS courseId,\n" +
            "\tcss.account_id AS accountId \n" +
            "FROM\n" +
            "\tcourse_student_status css,\n" +
            "\tcourse a \n" +
            "WHERE\n" +
            "\tcss.site_id = #{siteId}  \n" +
            "\tand css.student_status = 1 \n" +
            "\tand a.id = css.course_id \n" +
            "\tand a.shelves in(0,1) \n" +
            "\tAND a.del_flg=0 \n" +
            "\tand a.source in (1,2,3,5,6)\n" +
            "\t<if test='startDate != null and endDate != null'> \n" +
            "   AND css.first_finish_time between  #{startDate} and  #{endDate} \n" +
            "  </if>"+
            "</script>")
    List<CourseStudentStatus> getFinishCourseIdsByDate(@Param("siteId") long siteId, @Param("startDate") Date startDate, @Param("endDate") Date endDate);

}
