package com.yizhi.course.application.service;


import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.vo.AuthorizeChapterVO;
import com.yizhi.course.application.controller.CourseController;
import com.yizhi.course.application.domain.Accredit;
import com.yizhi.course.application.domain.Chapter;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author  lilingye
 * @date  2018-10-25 11:15:23
 */
@Component
@Log4j2
public class ChapterAddTask extends AbstractTaskHandler {
    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IAccreditService accreditService;

    @Autowired
    private IChapterService chapterService;

    @Autowired
    CourseController courseController;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void handle() {
        AuthorizeChapterVO chapterVO = this.taskContext.getChapterVO();
        if (chapterVO.getParentId()==0) {
            chapterVO.setLayer(1);
        } else {
            chapterVO.setLayer(2);
        }

        Accredit accredit = new Accredit();
        accredit.setCourseIdBefore(chapterVO.getCourseId());
        accredit.setValid(1);

        List<Accredit> accredits = new ArrayList<>();

        List<Accredit> accreditList = accreditService.getAccreditList(accredits,chapterVO.getCourseId());
        log.info("列表的数量："+accreditList.size());

        log.info("accreditList："+accreditList);
        // 组装查询条件
        Chapter chapterParam = new Chapter();
        chapterParam.setLayer(chapterVO.getLayer());


        // 初始化chapterList
        List<Chapter> tempChapterList = new ArrayList<>();
        Long id = null;

        for (Accredit a :accreditList){


            Long sourceChapterId =null;
            Chapter chapter = new Chapter();
            id = idGenerator.generate();
            log.info("生成的id："+id);
            if (chapterVO.getLayer()==1){
                chapter.setId(id);
                chapter.setName(chapterVO.getName());
                chapter.setLayer(chapterVO.getLayer());
                chapter.setCourseId(a.getCourseIdAfter());
                chapter.setMaterialId(chapterVO.getMaterialId());
                chapter.setMaterialType(chapterVO.getMaterialType());
                chapter.setMaterialName(chapterVO.getMaterialName());
                chapter.setMaterialCode(chapterVO.getMaterialCode());
                chapter.setChapterBeforeId(chapterVO.getId());
                chapter.setParentId(0L);
                chapter.setPath(id.toString());
                chapter.setSort(1);
                chapter.setIndexId(chapterVO.getIndexId());
            } else {
                Long indexId = chapterService.selectById(chapterVO.getParentId()).getIndexId();
                Chapter c= new Chapter();
                c.setIndexId(indexId);
                c.setLayer(1);
                c.setCourseId(a.getCourseIdAfter());


                    Chapter chapterDO= chapterService.selectOne(QueryUtil.condition(c));
                 if (chapterVO!=null){
                     sourceChapterId = chapterDO.getId();
                 }
                chapter.setId(id);
                chapter.setName(chapterVO.getName());
                chapter.setMaterialId(chapterVO.getMaterialId());
                chapter.setMaterialType(chapterVO.getMaterialType());
                chapter.setMaterialName(chapterVO.getMaterialName());
                chapter.setMaterialCode(chapterVO.getMaterialCode());
                chapter.setLayer(chapterVO.getLayer());
                chapter.setCourseId(a.getCourseIdAfter());
                chapter.setParentId(sourceChapterId);
                chapter.setChapterBeforeId(chapterVO.getId());
                chapter.setPath(sourceChapterId+","+id);
                chapter.setSort(2);
                chapter.setIndexId(chapterVO.getIndexId());

            }

            tempChapterList.add(chapter);
        }
        log.info("缓存中列表："+tempChapterList);
        try {
            if (CollectionUtils.isNotEmpty(tempChapterList)) {
                chapterService.insertBatch(tempChapterList);
            } else {
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
