package com.yizhi.course.application.service;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.course.application.domain.Chapter;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.domain.CourseStudentStatus;
import com.yizhi.course.application.vo.*;
import com.yizhi.course.application.vo.domain.CourseEntityVo;
import com.yizhi.util.application.domain.BizResponse;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 课程表 服务类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
public interface ICourseService extends IService<Course> {

    /**
     * 获取课程的总学时
     *
     * @param courseId
     * @return
     */
    Long getTotalDuration(Long courseId);

    CourseVo getCourseDetails(Long id);

    CourseVo save(CourseVo courseVo);
    BizResponse<CourseVo> saveV2(CourseVo courseVo);

    Boolean update(CourseVo courseVo);
    BizResponse<Boolean> updateV2(CourseVo courseVo);

    Page<Course> searchPage(String name, Long classifyId, Integer shelves,
                            Long companyId, Long siteId, List<Long> orgIds,
                            int pageNo, int pageSize, Integer type, Integer courseType, List<Long> courseIds);

    Page<Course> searchPageByClassifyAllMg(String name, Long classifyId, Integer shelves,
                            Long companyId, Long siteId, List<Long> orgIds,
                            int pageNo, int pageSize, Integer type, Integer courseType, List<Long> courseIds);

    Boolean vailCode(Long id, String code, Long siteId);

    /**
     * 课程编码是否已经存在（营销课程编码除外）
     * @param id 课程id
     * @param code 课程编码 （营销课程编码除外）
     * @param siteId 站点id
     * @return true:通过；false:不通过
     */
    Boolean vailCodeExclMarketing(Long id, String code, Long siteId);

    Page<ApiListCourseVo> apiList(String name, Long classifyId, Integer type, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize);

    Page<ApiListCourseVo> searchPageSubClassify(String name, List<Long> classifyIds, Integer type, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize);

    Double getAvgLevel(Long courseId);

    Page<ProjectCourseVo> searchProjectCoursePage(Long id, String name, Long classifyId, Long companyId, Long siteId, String[] orgId, int pageNo, int pageSize);

    /**
     * 分页查询
     *
     * @param courseVo
     * @param page
     */
    Page<Course> list(CourseVo courseVo, Page page);


    List<HomePageCoursePageVo> homePageCoursePageList(String name, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize);

    List<HomePageCoursePageVo> homePageCoursePageListNew(String name, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize);

    /**
     * 分页查询课程分类信息(新)
     *
     * @param name
     * @param relationIds
     * @param companyId
     * @param siteId
     * @param pageNo
     * @param pageSize
     * @return
     */
    List<HomePageCourseVo> homePageClassifyList(String name, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize);

    /**
     * 根据分类ID查询最新的课程
     *
     * @param name
     * @param relationIds
     * @param companyId
     * @param siteId
     * @param pageNo
     * @param pageSize
     * @return
     */
    List<ApiListCourseVo> classifyTopCourseList(Long classifyId, String name, List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize);


    Integer getRecordeDuration(Long chapterId, Long accountId);


    List<Course> selectCourseList(String name, Long companyId, Long siteId, Long orgId);


    List<ApiListCourseVo> selectOfflineCourseList(List<Long> ids, int pageNo, int pageSize);

    /**
     * 查询站点下课程列表
     *
     * @param name
     * @param siteId
     * @param classifyId
     * @param pageNo
     * @param pageSize
     * @return
     */
    PageCourseAuthorizeVO queryeAuthorizeCoursesList(String name, Long siteId, Long classifyId, Integer pageNo, Integer pageSize);

    /**
     * 课程授权/销售
     *
     * @param course
     * @param companyId
     * @param siteId
     * @param type
     * @return
     */
    Course copyCourse(Course course, Long companyId, Long siteId, Integer type, RequestContext context);

    //获取浏览人次
    Integer courseViewCount(Long id, Long accountId);


    /**
     * 范围人员导出             管理端-------2018/09/19
     *
     * @param courseId
     * @return
     */
    public VisibleRangeExport vsibleRangeExport(Long courseId);

    //根据课程id 获取课程评分和(课程学习人数)课程浏览人数-->
    HomePageCoursePageVo selectMyManageCourseList(Long courseId);

    /**
     * 授权课程的到期下架
     *
     * @param id 课程id
     * @return
     * @since 2018-9-21
     */
    boolean invalidCourse(Long id);

    /**
     * 为了优化新写的接口
     *
     * @param relationIds
     * @param companyId
     * @param siteId
     * @param pageNo
     * @param pageSize
     * @return
     */
    List<HomePageCoursePageVo> homePageCourseNewPageList(List<Long> relationIds, Long companyId, Long siteId, int pageNo, int pageSize);

    Page<AuthCourseVO> findCourseList(String name, List<Long> classifyIds, Integer shelves, Long companyId, Long siteId, List<Long> orgIds, Long authorizeSiteId, int pageNo, int pageSize);

    /**
     * 获取课程可见范围的id列表
     *
     * @return
     */
    List<Long> findCourseIdList(Integer pageNo, Integer pageSize);

    /**
     * 根据课程Id获取课程列表
     *
     * @param courseIds
     * @return
     */
    List<HomePageCoursePageVo> findHomeCourseList(List<Long> courseIds);

    /**
     * 查询pc的课程列表数据
     *
     * @param courseIds
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<HomePageCoursePageVo> findCourseList(List<Long> courseIds, Integer pageNo, Integer pageSize);

    /**
     * 查询推介课程
     *
     * @param courseIds
     * @param companyId
     * @param siteId
     * @param accountId
     * @return
     */
    List<HomePageCoursePageVo> findRecommendCourseList(List<Long> courseIds, Long companyId
            , Long siteId, Long accountId);

    /**
     * 查询微信的课程列表数据
     *
     * @param courseIds
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<HomePageCourseVo> findWechatCourseList(List<Long> courseIds, Integer pageNo, Integer pageSize
            , Long siteId);

    /**
     * 判断课程是否完成
     *
     * @param courseId
     * @param accountId
     * @param isTestSensitive 是否包括测验是否完成
     * @return
     */
    Integer isCourseFinished(Long courseId, Long accountId, Boolean isTestSensitive);

    Page<Map<String, Object>> experiencePoolList(String name, List<Long> ids, Integer pageNo, Integer pageSize);


    List<ModuleAllVoCourse> listCoursePCgrade(List<Long> ids);


    Page<Course> listNotIds(Long siteId, String name, List<Long> ids, Integer pageNo, Integer pageSize);

    /**
     * 查询课程
     *
     * @param relationIds
     * @param num
     * @return
     */

    List<ModuleAllVoCourse> listCourseRelationId(List<Long> relationIds, Integer num, List<Long> listIds);


    List<Map<String, Object>> getServerByCompanyIdAndIds(Long companyId, List<Long> ids);

    Map<String, Object> getCourseTimeMessage(Long companyId, List<Long> ids);

    /**
     * 查询我的课程（进行中和已完成的课程Id） 进行中和完成的章节列表（按课程id分类）
     *
     * @param siteId
     * @param accountId
     * @return
     */
    Map<String, Map<Long, List<Chapter>>> getMyStudyCourseIds(Long siteId, Long accountId);

    Map<String, List<CourseStudentStatus>> getMyStudyCourseStatus(Long siteId, Long accountId, Date startDate, Date endDate);

    /**
     * 根据课程id查询课程基本信息
     *
     * @param courseIds
     * @return
     */
    Page<MyCourseRecordeVo> findCourseBaseInfoByIds(Set<Long> courseIds, Long siteId,
                                                    Long accountId,
                                                    Integer pageNo, Integer pageSize);

    /**
     * 查询课程评级
     *
     * @param courseIds
     * @return
     */
    List<ApiListCourseVo> findWechatCourseLevelList(List<Long> courseIds);


    /**
     * 获取可见范围的课程
     *
     * @param courseIds
     * @return
     */
    List<HomePageCoursePageVo> findCourseIdList(List<Long> courseIds);


    List<CourseListExportVO> listExport(String name, Long classifyId, Integer shelves, Long companyId, Long siteId, List<Long> orgIds, Integer type, Integer courseType);


    /**
     * o
     * 自定义项目获取课程log
     *
     * @param courseIds
     * @return
     */
    Map<Long, String> getCourseLogos(List<Long> courseIds);

    /**
     * 获取《课程id,课程编码》
     */
    Map<Long, String> getCourseCodeName(List<Long> courseIds);

    Page<Course> getPageToCalendar(Date date, Page page);

    /**
     * 嘉兴银行闯关排行
     *
     * @param courseId
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<JaxinRankingVO> getRankings(Long courseId, Integer pageNo, Integer pageSize);

    Page<DroolsVo> getPageByDrools(String field, String value, Page<DroolsVo>  page);

    /**
     * v2版本
     */
    Page<HomePageCoursePageVo> findCourseListV2(List<Long> courseIds, Integer pageNo, Integer pageSize, Boolean hotAsc, Boolean gradeAsc);

    /**
     * 获取课程的总学习时长(=课程下素材总时长)
     * @param courseId 课程id
     * @return 课程总时长；单位是秒
     */
    Integer getCourseMaterialDurationsSeconds(Long courseId,Long companyId,Long siteId);
}

