package com.yizhi.course.application.service;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.course.application.domain.Material;
import com.yizhi.course.application.vo.*;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 素材库 服务类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
public interface IMaterialService extends IService<Material> {


    /**
     *  素材保存
     * @param material 素材库对象
     * @param map  包含类型的集合
     * @return
     */
    Material saveMaterial(Material material, Map<String,Object> map);

    /**
     *  删除物理
     * @param ids id集合
     * @return
     */
    Boolean deleteMaterial(List<Long> ids);

    /**
     * 根据id获得素材全部信息
     * @param id
     * @return
     */
    MaterialVo getById(Long id);

    /**
     * 查询章节列表的素材详情
     * @param materialIds
     * @return
     */
    List<MaterialVo> getMaterialDetails(Set<Long> materialIds);

    /**
     * 更新素材库
     * @param material
     * @param map
     * @return
     */
    Material updateMaterial(Material material, Map<String,Object> map);

    /**
     * 分页列表查询
     * @param name
     * @param type
     * @param source
     * @param companyId
     * @param siteId
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<Material> searchPage(String name, Integer type,Integer source,Long companyId,Long siteId,List<Long> orgId,int pageNo, int pageSize);
    
    
    
    /**
     * 得到我的做课列表
     * @param accountId
     * @return
     */
    Page<CourseStudentListVo> listMyLesson(String name, Integer pageNo, Integer pageSize, Long accountId);
    
    
    /**
     * 根据id主键查看具体细节
     * @param id
     * @return
     */
    CourseStudentListVo getCourseStudentListVo(Long id);
    
    /**
     * 保存素材
     * @param materialSaveVO
     * @return
     */
    Long saveMaterialByType(MaterialSaveVO materialSaveVO);
    
    
    /**
     * 未提交状态更改未提交状态
     * @param materialStateUpdateVo
     * @return
     */
    Boolean updateStateById(MaterialStateUpdateVo materialStateUpdateVo);
    
    
    /**
     * 未提交状态更改未提交状态 并且得到下一个id
     * @param materialStateUpdateVo
     * @return
     */
    Long updateStateByIdAndNextId(MaterialStateUpdateVo materialStateUpdateVo);
    
    
    
    /**
     * 管理端得到做课列表
     */
    Page<MaterialManageVo> manageGetLessonList(String name, List<Long> listCommintIdList, List<Long> listAuditorIdList, Integer state, Integer pageNo, Integer pageSize, Long orgId);
    
    /**
     * 查看做课详情
     * @param id
     * @return
     */
    MaterialManageDetailsVo getDetailsById(Long id);
    
    
    /**
     * 查询已通过的做课的列表
     * @return
     */
    Page<CourseStudentListVo> getPassMaterialList(String name,Long accountId,Integer pageNo,Integer pageSize);
    
    
    
    /**
     * 我的所有做课数量（不包括已经删除的）
     * @param accountId
     * @return
     */
    Integer getMyLessonAllNum(Long accountId);



    /**
     *  查询素材列表  ( Scorm课程 )
     *
     * @param
     * @return
     */
    Page<Material> queryScormCourseList(String name, Integer type, Integer source, Long companyId, Long siteId, List<Long> orgId,Integer pageNo, Integer pageSize);

    /**
     *  查询素材列表  ( 一般课程 )
     *
     * @param name
     * @param type
     * @param source
     * @param companyId
     * @param siteId
     * @param orgId
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<Material> queryCommonCourseList(String name, Integer type, Integer source, Long companyId,
                                         Long siteId, List<Long> orgId,Integer pageNo, Integer pageSize,
                                         Integer excludType);

    /**
     * 异步上传阿里远程视频
     * @param material 素材的Id
     * @return
     */
    void uploadPolyVideoRemote(Material material);

    /**
     * 获取polyId
     * @param material
     * @return
     */
    String getPolyVId(Material material);

    List<Material> queryMaterialList(List<String> codeList,Long beforeSiteId);

    List<MaterialExportVO> getMaterialExportVOList(List<Long> courseIds);
}
