package com.yizhi.course.application.service;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.course.application.domain.Material;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author  lilingye
 * @date  2018-11-21 10:45:44
 */
@Component
@Log4j2
public class PolyUploadRemoteVideoTask extends AbstractTaskHandler {
    @Autowired
    private IMaterialService materialService;

    @Value("${polyv.remote.url}")
    private String polyvUrl;

    @Value("${polyv.writetoken}")
    private String writeToken;
    @Value("${polyv.cataid}")
    private String cataId;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void handle() {
        Long id = this.taskContext.getTaskId();
        // 获取素材实体
        Material material = materialService.selectById(id);

        HttpClientUtils httpClientUtils = new HttpClientUtils();


        // 远程上传保利威视 视频
        Map<String,String> polyvMap = new HashMap<String,String>(16);
        polyvMap.put("fileUrl",material.getOriginalFileUrl());
        polyvMap.put("writetoken",writeToken);
        polyvMap.put("title",material.getName());
        polyvMap.put("cataid",cataId);
        log.info("参数的polyvMap："+polyvMap);
        String reStr = httpClientUtils.httpPost(polyvUrl,polyvMap);
        log.info("上传保利威视的远程视的结果："+reStr);

        JSONObject polyResult = JSON.parseObject(reStr);
        log.info("保利威视返回的结果："+reStr);
        if (null!=reStr&&polyResult.containsKey("data")){
            String data = polyResult.getString("data");
            JSONArray jsonArray = JSON.parseArray(data);
            JSONObject dat = JSON.parseObject(jsonArray.get(0).toString());
            String vid = dat.getString("vid");
            log.info("保利威视的vid："+vid);
            String duration = dat.getString("duration");

            Material newMaterial = new Material();
            newMaterial.setId(id);
            newMaterial.setPolyVid(vid);

            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
            try {
                Date date = sdf.parse(duration);
                System.out.println(sdf.parse(duration));
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(date);
                int hour = calendar.get(Calendar.HOUR_OF_DAY);
                int minute = calendar.get(Calendar.MINUTE);
                int second = calendar.get(Calendar.SECOND);
                log.info("hour:"+hour+",minute:"+minute+",second:"+second);
                newMaterial.setTextHour(hour);
                newMaterial.setTextMinute(minute);
                newMaterial.setTextSecond(second);
            } catch (ParseException e) {
                log.info("异常{}",e);
            }

            //更新polyVid到素材中
            log.info("修改参数"+material);
            boolean ret = materialService.updateById(newMaterial);
            log.info("返回结果"+ret);
        }


    }
}
