package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.course.application.domain.BrowseRecords;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.domain.CourseShare;
import com.yizhi.course.application.domain.WechatInfo;
import com.yizhi.course.application.mapper.BrowseRecordsMapper;
import com.yizhi.course.application.service.IBrowseRecordsService;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.service.ICourseShareService;
import com.yizhi.course.application.service.IWechatInfoService;
import com.yizhi.course.application.vo.AccountVO;
import com.yizhi.course.application.vo.MarketingCourseVo;
import com.yizhi.wechat.application.feign.WeiXinClient;
import com.yizhi.wechat.application.vo.wechat.domain.UserInfoVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author 谢海军123
 * @since 2018-04-09
 */
@Service
public class BrowseRecordsServiceImpl extends ServiceImpl<BrowseRecordsMapper, BrowseRecords> implements IBrowseRecordsService {

    @Autowired
    private ICourseShareService iCourseShareService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ICourseService courseService;

    @Autowired
    private IWechatInfoService wechatInfoService;

    @Autowired
    private WeiXinClient weiXinClient;

    private static final String SUB_SHARE_ID_SPLIT_STR = "\\$\\$";


    /**
     * 增加浏览记录
     * @param courseId 查看的营销课程id
     * @param shareId  分享时候的分享id
     * @param accountId 分享营销课程顶级分享人
     * @param viewWechatInfo  如果微信用户打开的分析，获取该用户的基本信息进行记录（观看的用户信息 格式： {id, nikename, avatar} 微信唯一Id、微信昵称，微信头像）
     * @param marketingCourseVo 分享的营销课程信息(设在了用户名片信息）
     * @param course 课程信息
     * @param accountVO 分享人的用户信息
     * @param tkAccountId 当前登陆人的账号id
     * @param childShareOpenId 下级分享人的微信openId 列表，多个id使用##分割
     */
    @Override
    @Transactional
    public void addReadRecodes(Long courseId,
                               Long shareId, Long accountId,
                               String viewWechatInfo, MarketingCourseVo marketingCourseVo,
                               Course course, AccountVO accountVO, Long tkAccountId,
                               String childShareOpenId) {

        CourseShare courseShare = null;
        Long readCount = 1L;
        // shareId = -1 说明微信端操作
        if(shareId.equals(-1L)){

            // 获取下级分享人
            String firstShareId = null;
            String secondShareId = null;
            if(StringUtils.isNotBlank(childShareOpenId)){
                String[] subShareIds = childShareOpenId.split(SUB_SHARE_ID_SPLIT_STR);
                if(subShareIds.length>0 && StringUtils.isNotBlank(subShareIds[0])){
                    firstShareId = subShareIds[0];
                }
                if(subShareIds.length>1 && StringUtils.isNotBlank(subShareIds[1])){
                    secondShareId = subShareIds[1];
                }

            }

            // 首先查询是否有存在分享记录
            CourseShare courseShare1 = new CourseShare();
            EntityWrapper<CourseShare> ew = QueryUtil.condition(courseShare1);
            ew.eq("course_id",courseId).eq("account_id",accountId);
            if(firstShareId != null){
                ew.eq("first_share_id",firstShareId);
            }
            if(secondShareId != null){
                ew.eq("second_share_id",secondShareId);
            }
            // 根据accountId为该用户生成一个课程的分享记录(判断存在就直接取出)，第一次生成分享次数为1；存在就更新+1
            courseShare = iCourseShareService.selectOne(ew);


            // 如果没有存在分享信息（说明分享出去还是第一次打开，所以进行增加分享人信息
            if(courseShare == null){
                courseShare = new CourseShare();
                shareId = idGenerator.generate();
                courseShare.setId(shareId);
                courseShare.setCourseId(courseId);
                courseShare.setAccountId(accountVO.getId());
                courseShare.setShareUrl("#########");
                courseShare.setShareTime(new Date());
                courseShare.setReadCount(readCount);
                courseShare.setName(accountVO.getName());
                courseShare.setOrgId(accountVO.getOrgId());
                courseShare.setCompanyId(ContextHolder.get().getCompanyId());
                courseShare.setSiteId(ContextHolder.get().getSiteId());
                courseShare.setFullName(accountVO.getFullName());
                String tel = accountVO.getMobile();
                courseShare.setTel(StringUtils.isEmpty(tel)?accountVO.getTelephone():tel);// 优先使用手机号码设置
                courseShare.setFirstShareId(firstShareId);
                courseShare.setSecondShareId(secondShareId);
                iCourseShareService.insert(courseShare);
            // 表示存在（该分享不是第一次打开了）
            }else {
                shareId = courseShare.getId();// 获取分享id
                // 获取shareId 被浏览次数
                readCount = getShareIdReadCount(Arrays.asList(shareId));
                String urlShare = courseShare.getShareUrl();
                if(StringUtils.isBlank(urlShare)){
                    courseShare.setShareUrl("#########");
                }
                courseShare.setReadCount(readCount+1);
                // 更新分享id被浏览的次数
                iCourseShareService.updateById(courseShare);
            }
        // 对于pc端的。分享时候可以携带shareId的，会在分享的时候插入分享信息。打开的时候就直接更新观看次数（现在基本不用pc分享）
        } else {
            courseShare = iCourseShareService.selectById(shareId);
            // 获取shareId 被浏览次数
            readCount = getShareIdReadCount(Arrays.asList(shareId));
            courseShare.setReadCount(readCount+1);
            // 更新分享id被浏览的次数
            iCourseShareService.updateById(courseShare);
        }

        // 生成分享记录，记录开始时间；设置分享记录到返回结果
        // 生成一条浏览记录
        BrowseRecords browseRecords = new BrowseRecords();
        browseRecords.setId(idGenerator.generate());
        browseRecords.setReadStartTime(new Date());

        // 上下文获取当前用户id
        browseRecords.setAccountId(tkAccountId==null?"-1":tkAccountId.toString());
        browseRecords.setAccountName("游客");
        if(tkAccountId!=-1){
            browseRecords.setAccountId(accountVO.getId().toString()); // 平台用户
            browseRecords.setAccountName(accountVO.getName());
        //} else if(openId != null){
        } else if(viewWechatInfo != null){
            try{

                /*JSONObject jsonObject = JSONObject.parseObject(viewWechatInfo);
                String openId = jsonObject.getString("id");
                String nikename = jsonObject.getString("nikename");
                String avatar = jsonObject.getString("avatar");*/
                // 修改成，从wechat服务的接口获取
                Map<String, UserInfoVo>  data = weiXinClient.getWeChatUserInfoList(Arrays.asList(viewWechatInfo));
                if(CollectionUtils.isEmpty(data)){
                    browseRecords.setAccountId(viewWechatInfo.equals("null")?"-1":viewWechatInfo);// 微信用户
                    browseRecords.setAccountName(viewWechatInfo.equals("null")?"游客":viewWechatInfo);
                }else {
                    UserInfoVo userInfo = data.get(viewWechatInfo);
                    browseRecords.setAccountId(viewWechatInfo);// 微信用户
                    browseRecords.setAccountName(userInfo.getNickname());
                    browseRecords.setAvatar(userInfo.getHeadimgurl());

                    // CREATE UNIQUE INDEX uidx_share_user ON course_share(course_id,account_id,first_share_id,second_share_id)
                    WechatInfo wechatInfo = new WechatInfo();
                    wechatInfo.setOpenId(viewWechatInfo);
                    wechatInfo.setNikeName(userInfo.getNickname());
                    wechatInfo.setAvatar(userInfo.getHeadimgurl());
                    wechatInfoService.insertOrUpdate(wechatInfo);
                }
                // Map mapObj = JSONObject.parseObject(str,Map.class);
                // Map mapType = JSON.parseObject(str,Map.class);
                //browseRecords.setAccountId(Long.valueOf(openId));

            }catch (Exception e){
                browseRecords.setAccountId(viewWechatInfo);// 微信用户
                browseRecords.setAccountName(viewWechatInfo);
            }
        }

        browseRecords.setShareId(shareId);
        browseRecords.setResidenceTime(1000L);
        browseRecords.setReadEndTime(new Date());
        this.insert(browseRecords);
        // 设置浏览记录id返回
        marketingCourseVo.setBrowseRecordsId(browseRecords.getId());

        // 最后更新课程最后的浏览次数
        CourseShare courseShare1 = new CourseShare();
        courseShare1.setCourseId(courseId);
        List<CourseShare> ret = iCourseShareService.selectList(QueryUtil.condition(courseShare1));
        List<Long> shareIds = new ArrayList<>();
        Long sunNum = 0L;
        if(!CollectionUtils.isEmpty(ret)){
            for(CourseShare obj : ret){
                shareIds.add(obj.getId());
            }
            sunNum = getShareIdReadCount(shareIds);
        }
        course.setReadCount(sunNum);
        courseService.updateById(course);
    }

    public Long getShareIdReadCount(List<Long> shareId) {
        Long readCount = 0L;

        EntityWrapper<BrowseRecords> ew = QueryUtil.condition(new BrowseRecords());
        ew.in("share_id",shareId);

        readCount = Long.valueOf(this.selectCount(ew));
        return readCount;
    }
}
