package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.ClassifyGroup;
import com.yizhi.course.application.domain.ClassifyGroupInfo;
import com.yizhi.course.application.mapper.ClassifyGroupInfoMapper;
import com.yizhi.course.application.service.IClassifyGroupInfoService;
import com.yizhi.course.application.service.IClassifyGroupService;
import com.yizhi.course.application.vo.ClassifyGroupInfoVO;
import com.yizhi.course.application.vo.ClassifyGroupVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 课程分类组基本信息表 服务实现类
 * </p>
 *
 * @author xiehaijun
 * @since 2019-12-13
 */
@Service
public class ClassifyGroupInfoServiceImpl extends ServiceImpl<ClassifyGroupInfoMapper, ClassifyGroupInfo> implements IClassifyGroupInfoService {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IClassifyGroupService classifyGroupService;

    @Transactional
    @Override
    public List<ClassifyGroup> save(ClassifyGroupInfoVO classifyGroupInfoVO) {
        // 上下文下信息
        RequestContext rc = ContextHolder.get();

        // 分类组的id
        Long id = classifyGroupInfoVO.getId();

        List<ClassifyGroupVO> datas = classifyGroupInfoVO.getData();

        List<ClassifyGroup> addObj = new ArrayList<>();
        // 删除数据
        if(!CollectionUtils.isEmpty(datas)){
            List<Long> delClassifyIds = new ArrayList<>();

            for(ClassifyGroupVO item : datas){
                delClassifyIds.add(item.getClassifyId());
                if(! item.getIsDelete()){
                    ClassifyGroup objSub = new ClassifyGroup();
                    objSub.setId(idGenerator.generate());
                    objSub.setClassifyGroupInfoId(id);
                    objSub.setClassifyId(item.getClassifyId());
                    objSub.setLayer(item.getLayer());
                    addObj.add(objSub);
                }
            }

            // 删除的数据
            if(delClassifyIds.size()>0){
                Collections.sort(delClassifyIds);
                //List<Long> delIds = new ArrayList<>();
                EntityWrapper<ClassifyGroup> delEw = QueryUtil.condition(new ClassifyGroup());
                delEw.eq("classify_group_info_id",id).in("classify_id",delClassifyIds);
                List<Long> del = classifyGroupService.getDelClassifyId(id,delClassifyIds);
                if(!CollectionUtils.isEmpty(del)){
                    //delIds.addAll(del);
                    classifyGroupService.deleteBatchIds(del);
                }
            }

            // 添加数据
            if(!CollectionUtils.isEmpty(addObj)){
                classifyGroupService.insertBatch(addObj);
            }

            // 更新分类组的主信息表数据
            ClassifyGroupInfo classifyGroupInfo = new ClassifyGroupInfo();
            classifyGroupInfo.setId(id);
            classifyGroupInfo.setUpdateById(rc.getAccountId());
            classifyGroupInfo.setUpdateTime(new Date());
            boolean upStatus = this.updateById(classifyGroupInfo);
            if(!upStatus){
                return null;
            }
        }

        return addObj;
    }

    @Transactional
    @Override
    public List<ClassifyGroup> saveAll(ClassifyGroupInfoVO classifyGroupInfoVO) {
            // 上下文下信息
            RequestContext rc = ContextHolder.get();

            // 分类组的id
            Long id = classifyGroupInfoVO.getId();

            List<ClassifyGroupVO> datas = classifyGroupInfoVO.getData();

            List<ClassifyGroup> addObj = new ArrayList<>();

            // 先删除数据
            EntityWrapper<ClassifyGroup> delEw = QueryUtil.condition(new ClassifyGroup());
            delEw.eq("classify_group_info_id",id);

            classifyGroupService.delete(delEw);

            // 删除数据
            if(!CollectionUtils.isEmpty(datas)){

                for(ClassifyGroupVO item : datas){
                    ClassifyGroup objSub = new ClassifyGroup();
                    objSub.setId(idGenerator.generate());
                    objSub.setClassifyGroupInfoId(id);
                    objSub.setClassifyId(item.getClassifyId());
                    objSub.setLayer(item.getLayer());
                    addObj.add(objSub);
                }
                // 添加数据
                if(!CollectionUtils.isEmpty(addObj)){
                    classifyGroupService.insertBatch(addObj);
                }

                // 更新分类组的主信息表数据
                ClassifyGroupInfo classifyGroupInfo = new ClassifyGroupInfo();
                classifyGroupInfo.setId(id);
                classifyGroupInfo.setUpdateById(rc.getAccountId());
                classifyGroupInfo.setUpdateTime(new Date());
                boolean upStatus = this.updateById(classifyGroupInfo);
                if(!upStatus){
                    return null;
                }
            }

            return addObj;
        }
}
