package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.domain.Classify;
import com.yizhi.course.application.domain.CourseAccount;
import com.yizhi.course.application.mapper.CourseAccountMapper;

import com.yizhi.course.application.service.IClassifyService;
import com.yizhi.course.application.service.ICourseAccountService;
import com.yizhi.course.application.vo.CourseAccountReport;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程指定用户 服务实现类
 * </p>
 *
 * @author 谢海军123
 * @since 2018-04-08
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class CourseAccountServiceImpl extends ServiceImpl<CourseAccountMapper, CourseAccount> implements ICourseAccountService {

    private Logger LOG = LoggerFactory.getLogger(CourseAccountServiceImpl.class);

    @Autowired
    private IClassifyService classifyService;

    @Autowired
    IdGenerator idGenerator;

    @Override
    public Boolean saveCourseAccountList(List<CourseAccount> courseAccounts) {
        if (courseAccounts.size() > 0) {
            //保存之前先删除 修改时时 不用做数据对比
            Long courseId = courseAccounts.get(0).getCourseId();
            CourseAccount courseAccount = new CourseAccount();
            courseAccount.setCourseId(courseId);
            EntityWrapper<CourseAccount> wrapper = new EntityWrapper<CourseAccount>(courseAccount);
            this.baseMapper.delete(wrapper);

            //保存
            for (CourseAccount courseAccount1 : courseAccounts) {
                courseAccount1.setId(idGenerator.generate());
            }

            Boolean f = this.baseMapper.insertList(courseAccounts);
            return f;
        } else {
            LOG.error("列表为空");
            return Boolean.FALSE;
        }
    }


	@Override
	public CourseAccountReport listCourseAccountByCourseIdReprot(Long courseId) {
		// TODO Auto-generated method stub
		CourseAccountReport car=null;

		CourseAccount courseAccount=new CourseAccount();
		courseAccount.setCourseId(courseId);
		EntityWrapper<CourseAccount> wrapper=new EntityWrapper<CourseAccount>(courseAccount);
		 List<CourseAccount> courseAccountList =this.selectList(wrapper);
		 if(CollectionUtils.isNotEmpty(courseAccountList)) {
			 car=new CourseAccountReport();
			 List<Long> orgIds=new ArrayList<Long>();
			 List<Long> accountIds=new ArrayList<Long>();
			 for (CourseAccount courseAccount2 : courseAccountList) {
				 if(courseAccount2.getType()==1) {
					 orgIds.add(courseAccount2.getRelationId());
				 }
				 if(courseAccount2.getType()==2) {
					 accountIds.add(courseAccount2.getRelationId());
				 }
			}
			 car.setCourseId(courseId);
			 car.setOrgIds(orgIds);
			 car.setAccountIds(accountIds);
		 }
		return car;
	}

    @Override
    public Boolean insertCourseAccountList(List<CourseAccount> courseAccounts) {
        if (!CollectionUtils.isEmpty(courseAccounts)) {
            Long courseId = courseAccounts.get(0).getCourseId();
            CourseAccount courseAccount = new CourseAccount();
            courseAccount.setCourseId(courseId);
            EntityWrapper<CourseAccount> entityWrapper = new EntityWrapper<>(courseAccount);
            this.baseMapper.delete(entityWrapper);
            //保存
            for (CourseAccount courseAccount1 : courseAccounts) {
                courseAccount1.setId(idGenerator.generate());
            }

            Boolean f = this.baseMapper.insertList(courseAccounts);
            return f;
        } else {
            LOG.error("列表为空");
            return Boolean.FALSE;
        }
    }

    /**
     * 查询可见范围内的课程ids
     *
     * @param classifyId 分类id，可选参数
     * @param pageNo 可选参数,为空表示不分页
     * @param pageSize 可选参数,为空表示不分页
     * @return
     */
    @Override
    public Page<Long> selectLoginAccountCourseIds(Long classifyId, Integer pageNo, Integer pageSize,
                                                  boolean subClassify,String findCourseKey) {
        findCourseKey = StringUtils.trimToNull(findCourseKey);
        List<Long> classifyIds = null;
        if(classifyId != null){
            classifyIds = new ArrayList<>();
            classifyIds.add(classifyId);

            List<Long> whereIds = new ArrayList<>();
            whereIds.add(classifyId);
            if(subClassify){
                do{
                    EntityWrapper<Classify> ew = QueryUtil.condition(new Classify());
                    ew.eq("status",1);
                    ew.in("parent_id",whereIds);
                    List<Classify> data = classifyService.selectList(ew);
                    if(CollectionUtils.isNotEmpty(data)){
                        List<Long> tempIds = data.stream().map(item->item.getId()).collect(Collectors.toList());
                        classifyIds.addAll(tempIds);
                        whereIds = tempIds;
                    }else {
                        whereIds = null;
                    }
                }while (CollectionUtils.isNotEmpty(whereIds));
            }
        }
        RequestContext rc = ContextHolder.get();
        Page<Long> page = null;
        List<Long> ids = null;
        List<Long> relationIds = rc.getRelationIds();
        LOG.info("***************relationIds{}",JSON.toJSONString(relationIds));
        if(CollectionUtils.isEmpty(relationIds)){
            relationIds = new ArrayList<>();
            relationIds.add(rc.getAccountId());
            relationIds.add(rc.getOrgId());
        }
        if(Objects.isNull(pageNo) || Objects.isNull(pageSize)){
            page = new Page<>();
            ids = this.baseMapper.selectLoginAccountCourseIds(classifyIds,relationIds,
                    rc.getCompanyId(),rc.getSiteId(),findCourseKey);
            if(CollectionUtils.isNotEmpty(ids)){
                page.setRecords(ids);
            }
        }else {
            if(pageNo.intValue()==0){
                pageNo = 1;
            }
            page = new Page<>(pageNo,pageSize);
            ids = this.baseMapper.selectLoginAccountCourseIds(classifyIds,relationIds,
                    rc.getCompanyId(),rc.getSiteId(),findCourseKey,page);
            if(CollectionUtils.isNotEmpty(ids)){
                page.setRecords(ids);
            }
        }
        return page;
    }


	@Override
	public List<CourseAccount> listCourseAccountByCourseId(Long courseId) {
		// TODO Auto-generated method stub
		CourseAccount ca=new CourseAccount();
		ca.setCourseId(courseId);
		return this.selectList(new EntityWrapper<CourseAccount>(ca));
	}
}
