package com.yizhi.course.application.service.impl;

import com.yizhi.course.application.domain.CourseStudentStatus;
import com.yizhi.course.application.domain.Material;
import com.yizhi.course.application.mapper.CourseStudentStatusMapper;
import com.yizhi.course.application.mapper.StatisticsCourseMapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.course.application.service.ICourseStudentStatusService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.vo.CourseDashboardParam;
import com.yizhi.course.application.vo.CourseDashboardVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 课程学习状态 服务实现类
 * </p>
 *
 * @author 谢海军123
 * @since 2019-04-01
 */
@Service
public class CourseStudentStatusServiceImpl extends ServiceImpl<CourseStudentStatusMapper, CourseStudentStatus> implements ICourseStudentStatusService {

    @Autowired
    private StatisticsCourseMapper statisticsCourseMapper;

    @Autowired
    IRecordeService recordeService;

    /**
     * 查找当前用户在该站点有效的学习课程记录
     *
     * @param siteId
     * @param accountId
     * @return
     */
    @Override
    public List<CourseStudentStatus> findValidStudyCourse(Long siteId, Long accountId, Date startDate,Date endDate) {
        return this.baseMapper.findValidStudyCourse(siteId,accountId, startDate,endDate);
    }

    @Override
    public List<CourseStudentStatus> getTrainningMqInfo() {
        return this.baseMapper.getTrainningMqInfo();
    }


    @Override
    public Map<Long, CourseDashboardVO> getCourseDashboardData(CourseDashboardParam param) {
        Map<Long, CourseDashboardVO> dashboardDataMap = null;
        //按时间获取用户完成课程记录
        List<CourseStudentStatus> finishRecords = this.baseMapper.getFinishCourseIdsByDate(param.getSiteId(), param.getStartDate(), param.getEndDate());
        if (CollectionUtils.isNotEmpty(finishRecords)){
            dashboardDataMap = new HashMap<>(finishRecords.size());
            for (CourseStudentStatus a:finishRecords){
                if (dashboardDataMap.containsKey(a.getAccountId())){
                    CourseDashboardVO vo = dashboardDataMap.get(a.getAccountId());
                    vo.getCourseIds().add(a.getCourseId());
                    dashboardDataMap.put(a.getAccountId(),vo);
                }else {
                    CourseDashboardVO vo = new CourseDashboardVO();
                    Set<Long> courseIdsSet = new HashSet<>();
                    courseIdsSet.add(a.getCourseId());
                    vo.setCourseIds(courseIdsSet);
                    dashboardDataMap.put(a.getAccountId(),vo);
                }
            }
        }
        //根据完成记录的课程id获取课程时长
        if (dashboardDataMap != null) {
            Map<Long, Long> courseDurationMap = getCourseDurationMap();
            for (Long accountId : dashboardDataMap.keySet()) {
                CourseDashboardVO vo = dashboardDataMap.get(accountId);
                if (vo == null) {
                    continue;
                }
                Set<Long> courseIds = vo.getCourseIds();
                if (CollectionUtils.isNotEmpty(courseIds)) {
                    //课程完成数量赋值
                    vo.setCourseFinishNum(courseIds.size());
                    Long courseTime = getFinalDuration(courseIds,courseDurationMap);
                    //课程时长赋值
                    vo.setCourseTime(courseTime);
                }
            }
        }
        return dashboardDataMap;
    }

    private Long getFinalDuration(Set<Long> courseIds, Map<Long, Long> courseDurationMap) {
        Long time = 0L;
        if (CollectionUtils.isNotEmpty(courseIds)) {
            if (courseDurationMap != null) {
                for (Long a : courseIds) {
                    Long tem = courseDurationMap.get(a);
                    time += (tem == null ? 0 : tem);
                }
            }
        }
        return time;
    }

    /**
     * 获取课程的总学习时长
     *
     * @param ids
     * @return
     */
    private Long getCourseDuration(List<Long> ids) {
        long ms = 0;
        List<Material> listMaterial = statisticsCourseMapper.getCourseDurationById(ids);
        if (!CollectionUtils.isEmpty(listMaterial)) {
            for (Material material : listMaterial) {
                Integer hour = material.getTextHour();
                if (hour != null && hour.intValue() > 0) {
                    ms += (hour.intValue() * 3600);
                }
                Integer minute = material.getTextMinute();
                if (minute != null && minute.intValue() > 0) {
                    ms += (minute.intValue() * 60);
                }
                Integer second = material.getTextSecond();
                if (second != null && second.intValue() > 0) {
                    ms += second.intValue();
                }
            }
        }
        return ms;
    }

    /**
     * 获取课程的总学习时长
     *
     * @return
     */
    private Map<Long,Long> getCourseDurationMap() {
        Map<Long,Long> getCourseDurationMap = new HashMap<>();

        List<Material> listMaterial = statisticsCourseMapper.getCourseDurationById(null);
        if (!CollectionUtils.isEmpty(listMaterial)) {
            for (Material material : listMaterial) {
                long ms = 0;
                Integer hour = material.getTextHour();
                if (hour != null && hour.intValue() > 0) {
                    ms += (hour.intValue() * 3600);
                }
                Integer minute = material.getTextMinute();
                if (minute != null && minute.intValue() > 0) {
                    ms += (minute.intValue() * 60);
                }
                Integer second = material.getTextSecond();
                if (second != null && second.intValue() > 0) {
                    ms += second.intValue();
                }
                //sql 把course_id 赋值给了id
                if (!getCourseDurationMap.containsKey(material.getId())){
                    getCourseDurationMap.put(material.getId(),ms);
                }
            }
        }
        return getCourseDurationMap;
    }
}
