package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.course.application.domain.OfflineRecorde;
import com.yizhi.course.application.mapper.OfflineCourseMapper;
import com.yizhi.course.application.mapper.OfflineRecordeMapper;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.service.IOfflineRecordeService;
import com.yizhi.course.application.vo.ReportOfflineCourseStatisticsDetailVo;
import org.apache.ibatis.session.RowBounds;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 线下课程学习记录 服务实现类
 * </p>
 *
 * @author huangliang123
 * @since 2018-10-16
 */
@Service
@Transactional
public class OfflineRecordeServiceImpl extends ServiceImpl<OfflineRecordeMapper, OfflineRecorde> implements IOfflineRecordeService {

    @Autowired
    private OfflineCourseMapper offlineCourseMapper;

    @Override
    public List<OfflineRecorde> dataStatistics(Date startTime, Date endTime, RequestContext context) {
        return this.baseMapper.dataStatistics(startTime, endTime, context);
    }

    @Override
    public Page<ReportOfflineCourseStatisticsDetailVo> statisticsDetail(Date startTime, Date endTime, String code, String name,
                                                                        int pageSize, int pageNo, RequestContext context) {
        Page<ReportOfflineCourseStatisticsDetailVo> page = new Page(pageNo, pageSize);
        List<ReportOfflineCourseStatisticsDetailVo> list = offlineCourseMapper.statisticsDetail(startTime, endTime, code, name, context, new RowBounds(page.getOffset(), page.getLimit()));
        List<ReportOfflineCourseStatisticsDetailVo> accountNumList = this.baseMapper.statisticsDetail(startTime, endTime, code, name, context, new RowBounds(page.getOffset(), page.getLimit()));
        if (!CollectionUtils.isEmpty(list) && !CollectionUtils.isEmpty(accountNumList)) {
            for (ReportOfflineCourseStatisticsDetailVo vo : list) {
                for (ReportOfflineCourseStatisticsDetailVo voo : accountNumList) {
                    if (vo.getCourseId().equals(voo.getCourseId())) {
                        vo.setAccountNum(voo.getAccountNum());
                        break;
                    }
                }
            }
        }
        page.setRecords(list);
        page.setTotal(offlineCourseMapper.statisticsDetailCount(startTime, endTime, code, name, context));
        return page;
    }

    @Override
    public List<ReportOfflineCourseStatisticsDetailVo> statisticsDetailExport(Date startTime, Date endTime, String code, String name, RequestContext context) {
        List<ReportOfflineCourseStatisticsDetailVo> list = this.baseMapper.statisticsDetailDownload(startTime, endTime, code, name, context);
        return list;
    }

    @Override
    public Integer getLastBatch(Long courseId, Long siteId, Long companyId) {
        return this.baseMapper.getLastBatch(courseId, siteId, companyId);
    }

    /**
     *  查询报表记录详情
     *
     * @param offlineCourseId
     * @return
     */
    @Override
    public Page<OfflineRecorde> queryDetailByOfflineCourseId(Long offlineCourseId, Integer pageNo, Integer pageSize) {
        Page<OfflineRecorde> page = new Page<OfflineRecorde>(pageNo,pageSize);
        List<OfflineRecorde> recordeList = this.baseMapper.queryDetailByOfflineCourseId(offlineCourseId, new RowBounds(page.getOffset(), page.getLimit()));
        Integer count = this.baseMapper.queryDetailByOfflineCourseIdCount(offlineCourseId);
        page.setTotal(count);
        page.setRecords(recordeList);
        return page;
    }

}
