package com.yizhi.course.application.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.plugins.pagination.PageHelper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.course.application.domain.Chapter;
import com.yizhi.course.application.domain.Course;
import com.yizhi.course.application.domain.Material;
import com.yizhi.course.application.domain.Recorde;
import com.yizhi.course.application.mapper.RecordeMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.course.application.service.IChapterService;
import com.yizhi.course.application.service.ICourseService;
import com.yizhi.course.application.service.IMaterialService;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.vo.*;
import com.yizhi.system.application.system.remote.ReportClient;
import org.apache.ibatis.session.RowBounds;
import org.checkerframework.checker.units.qual.C;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * <p>
 * 学习记录 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-19
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class RecordeServiceImpl extends ServiceImpl<RecordeMapper, Recorde> implements IRecordeService {

    private static final Logger LOGGER = LoggerFactory.getLogger(RecordeServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ICourseService courseService;

    @Autowired
    private IChapterService chapterService;

    @Autowired
    private IMaterialService materialService;

    @Autowired
    private ReportClient reportClient;


    @Override
    public Integer updateByHeart(Long dataId, Integer progress, Date endDate) {
        return this.baseMapper.updateByHeart(dataId, progress, endDate);
    }

    @Override
    public List<CourseStudyDurationRanking> studyHourRanking(Long siteId, String active) {
        return this.baseMapper.studyHourRanking(siteId, active);
    }

    @Override
    public CourseStudyStatisticVO getCourseStudyStatistic(Long courseId, Long accountId, Long siteId) {
//        CourseStudyStatisticVO cssv = this.baseMapper.getCourseStudyStatistic(courseId,accountId,siteId);
        List<CourseStudyRecordeVo> csrv = this.baseMapper.getCourseStudyRecorde(courseId, accountId, siteId);
        CourseStudyStatisticVO cssv = new CourseStudyStatisticVO();
        cssv.setStudyDuration(0);
        cssv.setStudyCount(0);
        if (CollectionUtils.isEmpty(csrv)) {
            return cssv;
        }
        for (CourseStudyRecordeVo csr : csrv) {
            int duration = (int) (csr.getEndTime().getTime() - csr.getStartTime().getTime()) / 1000;
            if (csr.getDuration().equals(0) && duration > 0) {
                cssv.setStudyCount(cssv.getStudyCount() + 1);
                cssv.setStudyDuration(cssv.getStudyDuration() + duration);
            } else if (!csr.getDuration().equals(0)) {
                cssv.setStudyCount(cssv.getStudyCount() + 1);
                cssv.setStudyDuration(cssv.getStudyDuration() + csr.getDuration());
            }
        }
        return cssv;
    }

    //记录列表添加站点过滤
    @Override
    public List<RecordeListVo> selectRecordeList(Long accountId, Long courseId) {
        RequestContext context = ContextHolder.get();
        Long siteId = context.getSiteId();
        return this.baseMapper.selectRecordeList(accountId, courseId, siteId);
    }

    @Override
    public List<RecordeListVo> selectRecordeOveredList(Long accountId, Long courseId, Long siteId) {
        return this.baseMapper.selectRecordeOveredList(accountId, courseId, siteId);
    }

    //我的课程记录列表
    @Override
    public Page<MyCourseRecordeVo> selectMyCourseRecordeList(String name, Long accountId, Integer type, Integer pageNo, Integer pageSize) {
        Page<MyCourseRecordeVo> page = new Page<MyCourseRecordeVo>(pageNo, pageSize);
        RequestContext context = ContextHolder.get();
        Long siteId = context.getSiteId();
        List<MyCourseRecordeVo> pageList = this.baseMapper.selectMyCourseRecordeList(name, accountId, type, new RowBounds(page.getOffset(), page.getLimit()), siteId);
        Integer count = this.baseMapper.selectMyCourseRecordeNumber(name, accountId, type, siteId);
        page.setTotal(count);
        page.setRecords(pageList);
        return page;
    }

    //课程时长
    @Override
    public Integer selectDurationByCourseId(Long courseId) {
        return this.baseMapper.selectDurationByCourseId(courseId);
    }

    //统计我的课程记录数量
    @Override
    public Integer selectMyCourseRecordeCount(Long accountId, Integer type) {
        RequestContext context = ContextHolder.get();
        Long siteId = context.getSiteId();
        return this.baseMapper.selectMyCourseRecordeCount(accountId, type, siteId);
    }

    @Override
    public List<RecordeDurationVO> selectAllRecordeDuration() {
        return this.baseMapper.selectAllRecordeDuration();
    }

    /**
     * 根据系统学习记录分析处理并保存到课程的学习记录
     *
     * @param data 按章节的学习日志
     * @return
     */
    @Override
    public Recorde saveRecorde(List<StudyLog> data) {
        StudyLog studyLog = data.get(0);

        // 查询是否存在学习记录
        Recorde recordeEw = new Recorde();
        recordeEw.setSessionId(studyLog.getSessionId());
        recordeEw.setChapterId(studyLog.getChapterId());
        recordeEw.setSiteId(studyLog.getSiteId());
        Recorde recorde = this.selectOne(QueryUtil.condition(recordeEw));

        // 设置学习记录
        setRecorde(recorde, studyLog);

        int size = data.size();
        if (size > 1) {
            StudyLog studyLogLast = data.get(size - 1);
            recorde.setEndTime(new Date(studyLogLast.getTimestamp()));
            recorde.setOvered(studyLogLast.getMaterialCompleted());
            Long diffTime = studyLogLast.getTimestamp() - recorde.getStartTime().getTime();
            recorde.setDuration((int) (diffTime / 1000));
        }

        // 如果未完成，查看课程策略是否是可以设置未王朝
        Integer overed = recorde.getOvered();
        if (Objects.isNull(overed) || overed.intValue() == 0) {
            // 获取课程的规则信息
            Course course = courseService.selectById(recorde.getCourseId());

            // 获取章节信息
            Chapter chapter = chapterService.selectById(recorde.getChapterId());
            // 获取完成状态
            overed = getOvered(overed, course, chapter, recorde.getDuration());
            recorde.setOvered(overed);
        }

        this.insertOrUpdate(recorde);

        return recorde;
    }

    //    @Override
    public Integer selectMyCourseRecordNumber(String name, Long accountId, Integer type, Long siteId) {
        return this.baseMapper.selectMyCourseRecordeNumber(name, accountId, type, siteId);
    }

    /**
     * 最后学习的章节id
     *
     * @param courseId
     * @param accountId
     * @return
     */
    @Override
    public long selectLastStudy(Long courseId, Long accountId) {
        PageHelper.remove();
        Long lastStudy = this.baseMapper.selectLastStudy(courseId, accountId);
        if (lastStudy == null) {
            return 0L;
        }
        return lastStudy;
    }

    @Override
    public Long getCourseFinished(Long accountId, Long chapterId) {
        Chapter chapter = chapterService.selectById(chapterId);
        if (chapter == null) {
            return 0L;
        }
        Long courseId = chapter.getCourseId();
        List<Long> chapterIds = chapterService.getChapters(courseId);
        LOGGER.info("######chapterIds:" + chapterIds);
        if (!CollectionUtils.isEmpty(chapterIds)) {
            for (Long chapterId1 : chapterIds) {
                Recorde recorde = new Recorde();
                recorde.setChapterId(chapterId1);
                recorde.setAccountId(accountId);
                recorde.setOvered(1);
                recorde.setValid(1);
                EntityWrapper<Recorde> entityWrapper = new EntityWrapper<>(recorde);
                List<Recorde> recordeList = this.selectList(entityWrapper);
                LOGGER.info("###############r.size():" + recordeList.size());
                if (!CollectionUtils.isEmpty(recordeList)) {
                    continue;
                } else {
                    return 0L;
                }
            }
            return courseId;
        }
        return 0L;
    }

    @Override
    public List<Long> getFinishedIds(Long courseId) {
        return this.baseMapper.getFinishedIds(courseId);
    }

    @Override
    public Integer getStudyTime(List<Long> courseId) {
        RequestContext context = ContextHolder.get();
        return this.baseMapper.getStudyTime(courseId, context.getSiteId(), context.getAccountId());
    }

    /**
     * @param overed
     * @param course
     * @param chapter
     * @return
     */
    private Integer getOvered(Integer overed, Course course, Chapter chapter, Integer duration) {
        // 章节类型 "素材类型1音频 2 视频 3 文本 4 富媒体 5 Scrom
        int materialType = chapter.getMaterialType();
        switch (materialType) {
            case 1:
                if (course.getAudioComplete().intValue() == 0) { //音频完成条件 0 播放即完成 1听完整个课程
                    overed = 1;
                }
                break;
            case 2:
                if (course.getVideoComplete().intValue() == 0) { //视频完成条件 0 播放即完成 1听完整个课程
                    overed = 1;
                }
                break;
            case 3:
                if (course.getDocComplete().intValue() == 0) { //文档完成条件 0 观看即完成 1停留时长达到学习时长即完成
                    overed = 1;
                } else {
                    overed = execStudyAging(overed, chapter, duration);
                }
                break;
            case 4:
                if (course.getRichComplete().intValue() == 0) { //多媒体完成条件 0 观看即完成 1停留时长达到学习时长即完成
                    overed = 1;
                } else {
                    overed = execStudyAging(overed, chapter, duration);
                }
                break;
            case 5:

                break;
        }
        return overed;
    }

    private Integer execStudyAging(Integer overed, Chapter chapter, Integer duration) {
        Material material = materialService.selectById(chapter.getMaterialId());
        if (material != null) {
            Integer hour = material.getTextHour();
            Integer minute = material.getTextMinute();
            Integer second = material.getTextSecond();

            Integer total = 0;
            if (!StringUtils.isEmpty(hour)) {
                total = total + hour * 3600;
            }
            if (!StringUtils.isEmpty(minute)) {
                total = total + minute * 60;
            }
            if (!StringUtils.isEmpty(second)) {
                total = total + second;
            }
            if (duration >= total) {
                overed = 1;
            }
        }
        return overed;
    }

    // 设置学习记录
    private void setRecorde(Recorde recorde, StudyLog studyLog) {
        if (recorde == null) {
            recorde = new Recorde();
            recorde.setId(idGenerator.generate());
            recorde.setCourseId(studyLog.getCourseId());
            recorde.setCompanyId(Long.parseLong(studyLog.getCompanyId()));
            recorde.setChapterId(studyLog.getChapterId());
            recorde.setOvered(studyLog.getMaterialCompleted());
            recorde.setAccountId(Long.parseLong(studyLog.getAccountId()));
            recorde.setSiteId(studyLog.getSiteId());
            recorde.setOrgId(Long.parseLong(studyLog.getOrgId()));
            recorde.setDuration(5);
            recorde.setCreateTime(new Date());
            recorde.setEndTime(new Date(studyLog.getTimestamp()));
            recorde.setStartTime(new Date((studyLog.getTimestamp() - 5000)));
            recorde.setExitstates(1);// 正常退出
            recorde.setValid(1); // 学习记录有效
            recorde.setSessionId(studyLog.getSessionId());
            recorde.setTerminalDevice(studyLog.getTerminalDevice());
            recorde.setTerminalType(studyLog.getTerminalType());
        } else {
            Long diffTime = studyLog.getTimestamp() - recorde.getStartTime().getTime();
            recorde.setDuration((int) (diffTime / 1000));
            recorde.setEndTime(new Date(studyLog.getTimestamp()));
            recorde.setOvered(studyLog.getMaterialCompleted());
        }

    }

    @Override
    public Long countStudyDuration(Long accountId, Long chapterId, Long siteId) {
        if (Objects.nonNull(chapterId)) {
            return this.baseMapper.countStudyDuration(accountId, chapterId, siteId);
        } else {
            return this.baseMapper.countStudyDurationByUser(siteId, accountId);
        }
    }

    @Override
    public Long countStudyDurationCourseIds(Long accountId, Long siteId, List<Long> courseIds) {
        return this.baseMapper.countStudyDurationCourseIds(accountId, siteId, courseIds);
    }

    @Override
    public List<RecordeListVo> findStuByAccountG(Long siteId) {
        return this.baseMapper.findStuByAccountG(siteId);
    }

    /**
     * 获取学习课程的章节的学习信息
     *
     * @param siteId
     * @param courseId
     * @param accountId
     * @return
     */
    @Override
    public List<RecordeVO> findStuChapterByAccountG(Long siteId, Long courseId, Long accountId) {
        return this.baseMapper.findStuChapterByAccountG(siteId, courseId, accountId);
    }

    @Override
    public Integer findRankingNumById(Long siteId, Long accountId) {
        // TODO Auto-generated method stub
        return this.baseMapper.findRankingNumById(siteId, accountId);
    }

    @Override
    public List<OrganizationDurationVO> getOrgDuration(Long siteId, Integer top, List<Long> accountIds) {
        // TODO Auto-generated method stub
        return this.baseMapper.getOrgDuration(siteId, top, accountIds);
    }

    @Override
    public StudyDurationVO studyDuration(Long accountId, List<Long> siteIds, String startDate, String endDate, List<Long> accounts) {
        // TODO Auto-generated method stub
        Integer finished = this.baseMapper.countFinished(accountId, siteIds, startDate, endDate);
        Integer proportion = this.baseMapper.proportion(accountId, siteIds, startDate, endDate);
        Long totalDuration = this.baseMapper.totalDuration(accountId, siteIds, startDate, endDate);
        StudyDurationVO vo = this.baseMapper.studyDuration(accountId, siteIds, startDate, endDate);
        Date lateDate = this.baseMapper.lateDate(accountId, siteIds, startDate, endDate);
        if (!StringUtils.isEmpty(vo)) {
            vo.setFinish(finished);
            vo.setTotalDuration(totalDuration);
            vo.setLateDate(lateDate);
            vo.setProportion((accounts.size() - proportion) * 100 / accounts.size());
        } else {
            vo = new StudyDurationVO();
            vo.setFinish(0);
            vo.setLateDate(null);
            vo.setMaxDate(null);
            vo.setMaxDuration(0l);
            vo.setProportion(0);
            vo.setTotalDuration(0l);
        }
        return vo;
    }

    @Override
    public Long todayStudyTime(String startDate, String endDate) {
        RequestContext context = ContextHolder.get();
        Long totalDuration = this.baseMapper.totalDuration(context.getAccountId(), Arrays.asList(context.getSiteId()), startDate, endDate);
        return totalDuration;
    }

    @Override
    public Map<Long, Long> getCourseStudyTime(CourseDashboardParam param) {
        Recorde recorde = new Recorde();
        recorde.setSiteId(param.getSiteId());
        recorde.setValid(1);
        EntityWrapper wrapper = new EntityWrapper(recorde);
        wrapper.between("create_time", param.getStartDate(), param.getEndDate());
        List<Recorde> recordes = this.selectList(wrapper);
        if (CollectionUtils.isEmpty(recordes)) {
            return null;
        }
        Map<Long, Long> map = new HashMap<>(recordes.size());
        recordes.forEach(a -> {
            if (map.containsKey(a.getAccountId())) {
                Long durationTime = map.get(a.getAccountId());
                Long durationTimeTem = 0L;
                Integer duration = a.getDuration();
                if (duration != null) {
                    durationTimeTem = Long.valueOf(duration.toString());
                    durationTime += durationTimeTem;
                }
                map.put(a.getAccountId(), durationTime);
            } else {
                Integer duration = a.getDuration();
                Long durationTime = 0L;
                if (duration != null) {
                    durationTime = Long.valueOf(duration.toString());
                }
                map.put(a.getAccountId(), durationTime);
            }
        });
        return map;
    }
}
