package com.yizhi.course.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.plugins.pagination.PageHelper;
import com.baomidou.mybatisplus.plugins.pagination.Pagination;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.course.application.domain.WechatInfo;
import com.yizhi.course.application.mapper.ReportMkCourseMapper;
import com.yizhi.course.application.service.IReportMkCourseService;
import com.yizhi.course.application.service.IWechatInfoService;
import com.yizhi.course.application.vo.MarketingCourseViewVo;
import com.yizhi.course.application.vo.ReportMarketingCourseAccountVo;
import com.yizhi.course.application.vo.ReportMarketingCourseDataChartsVo;
import com.yizhi.course.application.vo.ReportMarketingCourseVo;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountOrgsRepVO;
import com.yizhi.system.application.vo.AccountOrgsVO;
import com.yizhi.util.application.constant.MarketingCourseReportType;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.Period;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.time.temporal.WeekFields;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 17:18 2018/8/14
 * @Modified By
 */
@Service
//@Transactional
public class ReportMkCourseServiceImpl extends ServiceImpl<ReportMkCourseMapper, ReportMarketingCourseVo> implements IReportMkCourseService {

    private static Logger logger = LoggerFactory.getLogger(ReportMkCourseServiceImpl.class);

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private IWechatInfoService wechatInfoService;

    private static final String SPILT_STR = ",";
    /**
     * 图表数据获取
     * @param startDate
     * @param endDate
     * @param type
     * @param pageSize
     * @param pageNo
     * @return
     */
    @Override
    public Page<ReportMarketingCourseDataChartsVo> chartGroup(String startDate, String endDate,
                                                              Integer type, Integer pageSize,
                                                              Integer pageNo, Long cid, Long siteId, List<Long> orgs) {
        Page<ReportMarketingCourseDataChartsVo> pageData = new Page<>();
        // 开始时间
        LocalDate startLocalDate = LocalDate.parse(startDate, DateTimeFormatter.ISO_DATE);
        // 结束时间
        LocalDate endLocalDate = LocalDate.parse(endDate, DateTimeFormatter.ISO_DATE);

        List<String> retDataKey = getStringStream(type, startLocalDate, endLocalDate);
        //logger.info("****************统计图表出来的时间信息={}", JSON.toJSONString(retDataKey));
        endDate = endDate+" 23:59:59";
        //PageHelper.startPage(pageNo,pageSize);
        List<ReportMarketingCourseDataChartsVo> shareData = this.baseMapper.countShare(cid, siteId, orgs, startDate, endDate, type);
        //logger.info("****************统计图表出来的分享数据信息={}", JSON.toJSONString(shareData));
        Map<String,Integer> shareDataMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(shareData)){
            // 获取分享的次数
            shareDataMap = shareData.stream().collect(Collectors.toMap(key->key.getValue(),val->val.getTotalSendCount()));
        }

        List<ReportMarketingCourseDataChartsVo> readData = this.baseMapper.countRead(cid, siteId, orgs, startDate, endDate, type);
        //logger.info("****************统计图表出来的阅读数据信息={}", JSON.toJSONString(readData));
        Map<String,Integer> readDataMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(readData)){
            readDataMap = readData.stream().collect(Collectors.toMap(key->key.getValue(),val->val.getTotalRead()));
        }
        List<ReportMarketingCourseDataChartsVo> retDataList = new ArrayList<>();
        boolean isSpilt = retDataKey.get(0).indexOf(SPILT_STR)>=0;
        for(String key : retDataKey){
            ReportMarketingCourseDataChartsVo vo = new ReportMarketingCourseDataChartsVo();
            if(isSpilt){
                String[] keys=key.split(SPILT_STR);
                vo.setValue(keys[1]);
                Integer shareTem = shareDataMap.get(keys[0]);
                vo.setTotalSendCount(shareTem==null?0:shareTem);
                Integer readTem = readDataMap.get(keys[0]);
                vo.setTotalRead(readTem==null?0:readTem);
            }else{
                vo.setValue(key);
                Integer shareTem = shareDataMap.get(key);
                vo.setTotalSendCount(shareTem==null?0:shareTem);
                Integer readTem = readDataMap.get(key);
                vo.setTotalRead(readTem==null?0:readTem);
            }
            retDataList.add(vo);
        }
        //logger.info("****************统计图表出来的整合数据信息={}", JSON.toJSONString(retDataList));
        pageData.setRecords(retDataList);
        return pageData;

    }

    @Override
    public Page<ReportMarketingCourseVo> marketingCourseGroup(String startDate, String endDate,
                                                              String kwd, Integer pageSize, Integer pageNo,
                                                              Long cid, Long siteId, List<Long> orgs, boolean isPage) {
        Page<ReportMarketingCourseVo> voPage = new Page<>();
        Map<Long,ReportMarketingCourseVo> resultTemp = new HashMap<>();
        endDate = endDate+" 23:59:59";
        List<ReportMarketingCourseVo> shareData =
                this.baseMapper.marketingCourseShareGroup(cid,siteId,orgs,startDate,endDate,kwd);
        //logger.info("****************根据组织部门统计出来的分享信息={}", JSON.toJSONString(shareData));
        if(!CollectionUtils.isEmpty(shareData)){
            for(ReportMarketingCourseVo item : shareData){
                Long courseId= item.getId();
                if(courseId==null || courseId==0){
                    continue;
                }
                resultTemp.put(courseId,item);
            }
        }
        List<ReportMarketingCourseVo> readData =
                this.baseMapper.marketingCourseReadGroup(cid,siteId,orgs,startDate,endDate,kwd);
        //logger.info("****************根据组织部门统计出来的读取信息={}", JSON.toJSONString(readData));
        if(!CollectionUtils.isEmpty(readData)){
            for(ReportMarketingCourseVo item : readData){
                Long courseId= item.getId();
                if(courseId==null || courseId==0){
                    continue;
                }
                if(resultTemp.containsKey(courseId)){
                    ReportMarketingCourseVo old = resultTemp.get(courseId);
                    old.setTotalReadCount(item.getTotalReadCount());
                    old.setReadTimeSum(item.getReadTimeSum());
                    old.setReadTimeAvg(item.getReadTimeAvg());
                    resultTemp.put(courseId,old);
                    //resultTemp.remove(courseId);
                }else {
                    //item.setTotalSend(0);
                    //item.setTotalWxSendCount(0);
                    resultTemp.put(courseId,item);
                }
            }
        }
        // 查询课程打开次数
        List<ReportMarketingCourseVo> openData = this.baseMapper.marketingCourseOpenGroup(cid,siteId,orgs,startDate,endDate,kwd);
        if(!CollectionUtils.isEmpty(openData)){
            for(ReportMarketingCourseVo item : openData){
                Long courseId= item.getId();
                if(courseId==null || courseId==0){
                    continue;
                }
                if(resultTemp.containsKey(courseId)){
                    ReportMarketingCourseVo old = resultTemp.get(courseId);
                    old.setOpenCount(item.getOpenCount());
                    resultTemp.put(courseId,old);
                    //resultTemp.remove(courseId);
                }else {
                    resultTemp.put(courseId,item);
                }
            }
        }

        logger.info("****************根据组织部门统计出来的整合信息大小长度={}",resultTemp.size());
        int size = resultTemp.size();
        if(size>0){
            List<ReportMarketingCourseVo> result = null;
            if(isPage){
                result = resultTemp.entrySet().stream().map(obj -> obj.getValue()).
                        //sorted(Comparator.comparing(obj -> obj.getId(),Comparator.reverseOrder()));
                                sorted((obja,objb)->objb.getId().compareTo(obja.getId())).
                                skip((pageNo-1)*pageSize).limit(pageNo*pageSize).collect(Collectors.toList());

            }else {
                result = resultTemp.entrySet().stream().map(obj -> obj.getValue()).
                                sorted((obja,objb)->objb.getId().compareTo(obja.getId())).
                                collect(Collectors.toList());
            }
            voPage.setRecords(result);
        }
        if(isPage){
            // 总页数
            voPage.setTotal(size);
            // 每页条数
            voPage.setSize(pageSize);
            // 当前页
            voPage.setCurrent(pageNo);
        }

        return voPage;
    }

    @Override
    public Page<MarketingCourseViewVo> marketingCourseViewList(Long courseId, String nikeName,
                                                               Integer pageSize, Integer pageNo) {


        logger.info("marketingCourseViewList 处理开始; courseId= {}",courseId);
        Page<MarketingCourseViewVo> pageData = new Page<>();
        List<MarketingCourseViewVo> data = null;
        try {
            logger.info("marketingCourseViewList 处理判断是否分页; courseId= {}",courseId);
            if (pageSize == null || pageNo == null) {
                data = this.baseMapper.marketingCourseViewList(courseId, nikeName);
                logger.info("marketingCourseViewList 处理开始; 查询条数=", data == null ? 0 : data.size());
            } else {
                pageData.setCurrent(pageNo);
                pageData.setSize(pageSize);
                PageHelper.startPage(pageNo, pageSize);
                data = this.baseMapper.marketingCourseViewList(courseId, nikeName);
                Pagination pagination = PageHelper.getPagination();
                PageHelper.remove();
                pageData.setTotal(pagination.getTotal());

            }
            if (!CollectionUtils.isEmpty(data)) {
                Set<String> openIds = data.stream().map(MarketingCourseViewVo::getAccountId).collect(Collectors.toSet());
                if (!CollectionUtils.isEmpty(openIds)) {
                    EntityWrapper<WechatInfo> wechatInfo = QueryUtil.condition(new WechatInfo());
                    wechatInfo.in("open_id", openIds);
                    List<WechatInfo> wechatList = wechatInfoService.selectList(wechatInfo);
                    logger.info("marketingCourseViewList 处理中; 查询关联信息");
                    if (!CollectionUtils.isEmpty(wechatList)) {
                        Map<String, WechatInfo> wexinMap = wechatList.stream().collect(Collectors.toMap(key -> key.getOpenId(), val -> val));
                        for (MarketingCourseViewVo itemObj : data) {
                            if (wexinMap.containsKey(itemObj.getAccountId())) {
                                WechatInfo wechatInfoItem = wexinMap.get(itemObj.getAccountId());
                                itemObj.setAccountName(wechatInfoItem.getNikeName());
                                itemObj.setTel(wechatInfoItem.getTel());
                                itemObj.setAvatar(wechatInfoItem.getAvatar());
                            }
                        }
                    }
                }
            }
            pageData.setRecords(data);
            logger.info("marketingCourseViewList 处理结束; 查询条数=", data == null ? 0 : data.size());
        }catch (Exception e){
            logger.info("发生异常={}",e);
        }
        return pageData;
    }

    @Override
    public Page<ReportMarketingCourseAccountVo> marketingCourseAccountGroup(String startDate, String endDate,
                                                                            List<Long> orgKwd, List<Long> accountKwd,
                                                                            Integer pageSize, Integer pageNo,
                                                                            Long cid, Long siteId, List<Long> orgs,
                                                                            boolean isPage) {
        Page<ReportMarketingCourseAccountVo> voPage = new Page<>();
        Map<Long,ReportMarketingCourseAccountVo> resultTemp = new HashMap<>();
        endDate = endDate+" 23:59:59";
        List<ReportMarketingCourseAccountVo> shareData = this.baseMapper.mCAccountShareGroup(startDate, endDate,orgKwd,
                accountKwd, cid, siteId, orgs);
        if(!CollectionUtils.isEmpty(shareData)){
            for(ReportMarketingCourseAccountVo item : shareData){
                Long accountId= item.getAccountId();
                if(accountId==null || accountId==0){
                    continue;
                }
                resultTemp.put(accountId,item);
            }
        }
        //logger.info("****************根据用户统计出来的分享信息={}", JSON.toJSONString(shareData));
        List<ReportMarketingCourseAccountVo> readData = this.baseMapper.mCAccountReadGroup(startDate, endDate,orgKwd,
                accountKwd, cid, siteId, orgs);
        //logger.info("*****************根据用户统计出来的分享读取信息={}", JSON.toJSONString(readData));
        if(!CollectionUtils.isEmpty(readData)){
            for(ReportMarketingCourseAccountVo item : readData){
                Long accountId= item.getAccountId();
                if(accountId==null || accountId==0){
                    continue;
                }
                if(resultTemp.containsKey(accountId)){
                    ReportMarketingCourseAccountVo old = resultTemp.get(accountId);
                    old.setTotalReadCount(item.getTotalReadCount());
                    old.setReadTimeSum(item.getReadTimeSum());
                    old.setReadTimeAvg(item.getReadTimeAvg());
                    resultTemp.put(accountId,old);
                }else {
                    //item.setTotalSend(0);
                    //item.setTotalSendCount(0);
                    resultTemp.put(accountId,item);
                }
            }
        }

        // mCAccountOpenGroup
        List<ReportMarketingCourseAccountVo> openData = this.baseMapper.mCAccountOpenGroup(startDate, endDate,orgKwd,
                accountKwd, cid, siteId, orgs);
        if(!CollectionUtils.isEmpty(openData)){
            for(ReportMarketingCourseAccountVo item1 : openData){
                Long accountId= item1.getAccountId();
                if(accountId==null || accountId==0){
                    continue;
                }
                if(resultTemp.containsKey(accountId)){
                    ReportMarketingCourseAccountVo old = resultTemp.get(accountId);
                    old.setOpenCount(item1.getOpenCount());
                    resultTemp.put(accountId,old);
                }else {
                    resultTemp.put(accountId,item1);
                }
            }
        }

        logger.info("*****************根据用户统计出来的整合信息大小长度={}", resultTemp.size());
        int size = resultTemp.size();
        if(size>0){
            List<ReportMarketingCourseAccountVo> result;
            if(isPage){
                result = resultTemp.entrySet().stream().map(obj -> obj.getValue()).
                        sorted((obja,objb)->objb.getAccountId().compareTo(obja.getAccountId())).
                        skip((pageNo-1)*pageSize).limit(pageNo*pageSize).collect(Collectors.toList());
            }else{
                result = resultTemp.entrySet().stream().map(obj -> obj.getValue()).
                        sorted((obja,objb)->objb.getAccountId().compareTo(obja.getAccountId())).
                        collect(Collectors.toList());
            }
            //logger.info("****************根据用户统计出来的整合过滤信息={}", JSON.toJSONString(result));
            voPage.setRecords(result);
        }
        if(isPage){
            voPage.setTotal(size);//总页数
            voPage.setSize(pageSize);//每页条数
            voPage.setCurrent(pageNo);//当前页
        }

        return voPage;
    }

    /**
     * 下载统计
     * @param startDate
     * @param endDate
     * @param orgKwd
     * @param accountKwd
     * @param cid
     * @param siteId
     * @param orgs
     * @param business
     * @return
     */
    @Override
    public List<Object[]> downStatistics(String startDate,String endDate,
                                         List<Long> orgKwd, List<Long> accountKwd,
                                         Long cid, Long siteId,
                                         List<Long> orgs,
                                         String business) {
        endDate = endDate+" 23:59:59";
        List<Object[]> datas = new ArrayList<>();
        if(StringUtils.equals(MarketingCourseReportType.BY_COURSE_OPEN_EXPORT,business)){
            List<ReportMarketingCourseAccountVo> readData = this.baseMapper.downReadStatistics(startDate, endDate,orgKwd,
                    accountKwd, cid, siteId, orgs);
            if(!CollectionUtils.isEmpty(readData)){
                Set<Long> accountIds = readData.stream().map(item->item.getAccountId()).collect(Collectors.toSet());
                Map<Long, AccountOrgsVO> datamap = new HashMap<>();
                if(!CollectionUtils.isEmpty(accountIds)){
                    AccountOrgsRepVO findvo = new AccountOrgsRepVO();
                    findvo.setIds(new ArrayList<>(accountIds));
                    findvo.setIdsType(1);
                    findvo.setIsFindParent(true);
//                    findvo.setIsFindParent(false);
                    List<AccountOrgsVO> retData = accountClient.findCcountOrgs(findvo);
                    datamap = retData.stream().parallel().collect(Collectors.toMap(key->key.getId(),val->val));

                }
                //logger.info("datamap={}",datamap);
                for(ReportMarketingCourseAccountVo vo : readData){
                    Object[] data = new Object[8];
                    /*data[0] = vo.getAccountName();
                    data[1] = vo.getAccountFullName();*/
                    data[3] = vo.getCourseName();
                    data[4] = vo.getCourseCode();
                    data[5] = vo.getTotalReadCount();
                    if(datamap.containsKey(vo.getAccountId())){
                        AccountOrgsVO accountOrgsVO = datamap.get(vo.getAccountId());
                        data[0] = accountOrgsVO.getName();
                        data[1] =accountOrgsVO.getFullName();
                        data[2] = accountOrgsVO.getStatus();
                        data[6] = accountOrgsVO.getOrgName();
                        String orgNames = "";
                        if (!CollectionUtils.isEmpty(accountOrgsVO.getOrgNames())){
                            for (String s : accountOrgsVO.getOrgNames()){
                                if (StringUtils.isNotBlank(orgNames)){
                                    orgNames = orgNames + "/" + s;
                                }else {
                                    orgNames = orgNames + s;
                                }
                            }
                        }else {
                            orgNames = "--";
                        }
                        data[7] = orgNames;
                    }
                    datas.add(data);
                }
            }
        }else if(StringUtils.equals(MarketingCourseReportType.BY_USER_OPEN_EXPORT,business)){
            //titles = new String[]{"用户名","姓名","用户状态","课程名称","课程编码","打开时间","所在部门"};
            List<ReportMarketingCourseAccountVo> readData = this.baseMapper.downOpenStatistics(startDate, endDate,orgKwd,
                    accountKwd, cid, siteId, orgs);
            if(!CollectionUtils.isEmpty(readData)){
                Set<Long> accountIds = readData.stream().map(item->item.getAccountId()).collect(Collectors.toSet());
                Map<Long,AccountOrgsVO> datamap = new HashMap<>();
                if(!CollectionUtils.isEmpty(accountIds)){
                    AccountOrgsRepVO findvo = new AccountOrgsRepVO();
                    findvo.setIds(new ArrayList<>(accountIds));
                    findvo.setIdsType(1);
                    findvo.setIsFindParent(true);
//                    findvo.setIsFindParent(false);
                    List<AccountOrgsVO> retData = accountClient.findCcountOrgs(findvo);
                    datamap = retData.stream().parallel().collect(Collectors.toMap(key->key.getId(),val->val));

                }
                //logger.info("datamap={}",datamap);
                for(ReportMarketingCourseAccountVo vo : readData){
                    Object[] data = new Object[8];

                    data[3] = vo.getCourseName();
                    data[4] = vo.getCourseCode();
                    data[5] = vo.getDateTime();
                    if(datamap.containsKey(vo.getAccountId())){
                        AccountOrgsVO accountOrgsVO = datamap.get(vo.getAccountId());

                        data[0] = accountOrgsVO.getName();
                        data[1] =accountOrgsVO.getFullName();

                        data[2] = accountOrgsVO.getStatus();
                        data[6] = accountOrgsVO.getOrgName();
                        String orgNames = "";
                        if (!CollectionUtils.isEmpty(accountOrgsVO.getOrgNames())){
                            for (String s : accountOrgsVO.getOrgNames()){
                                if (StringUtils.isNotBlank(orgNames)){
                                    orgNames = orgNames + "/" + s;
                                }else {
                                    orgNames = orgNames + s;
                                }
                            }
                        }else {
                            orgNames = "--";
                        }
                        data[7] = orgNames;
                    }
                    datas.add(data);
                }
            }
        }else if(StringUtils.equals(MarketingCourseReportType.BY_USER_DETAIL_EXPORT,business)){
            List<ReportMarketingCourseAccountVo> readData = this.baseMapper.downReadDetailStatistics(startDate, endDate,orgKwd,
                    accountKwd, cid, siteId, orgs);
            if(!CollectionUtils.isEmpty(readData)){
                Set<Long> accountIds = readData.stream().map(item->item.getAccountId()).collect(Collectors.toSet());
                Map<Long,AccountOrgsVO> datamap = new HashMap<>();
                if(!CollectionUtils.isEmpty(accountIds)){
                    AccountOrgsRepVO findvo = new AccountOrgsRepVO();
                    findvo.setIds(new ArrayList<>(accountIds));
                    findvo.setIdsType(1);
                    findvo.setIsFindParent(true);
//                    findvo.setIsFindParent(false);
                    List<AccountOrgsVO> retData = accountClient.findCcountOrgs(findvo);
                    datamap = retData.stream().parallel().collect(Collectors.toMap(key->key.getId(),val->val));

                }
                //logger.info("datamap={}",datamap);
                for(ReportMarketingCourseAccountVo vo : readData){
                    Object[] data = new Object[8];
                    /*data[0] = vo.getAccountName();
                    data[1] = vo.getAccountFullName();*/
                    data[3] = vo.getCourseName();
                    data[4] = vo.getCourseCode();
                    data[5] = vo.getDateTime();
                    if(datamap.containsKey(vo.getAccountId())){
                        AccountOrgsVO accountOrgsVO = datamap.get(vo.getAccountId());

                        data[0] = accountOrgsVO.getName();
                        data[1] =accountOrgsVO.getFullName();

                        data[2] = accountOrgsVO.getStatus();
                        data[6] = accountOrgsVO.getOrgName();
                        String orgNames = "";
                        if (!CollectionUtils.isEmpty(accountOrgsVO.getOrgNames())){
                            for (String s : accountOrgsVO.getOrgNames()){
                                if (StringUtils.isNotBlank(orgNames)){
                                    orgNames = orgNames + "/" + s;
                                }else {
                                    orgNames = orgNames + s;
                                }
                            }
                        }else {
                            orgNames = "--";
                        }
                        data[7] = orgNames;
                    }
                    datas.add(data);
                }
            }
        }


        return datas;
    }

/*    public static void main(String[] args) {
        List<String> data = getStringStream(2,LocalDate.of(2019,1,1),LocalDate.of(2019,1,16));
        System.out.println(data);
    }*/

    public static List<String> getStringStream(Integer type, LocalDate startLocalDate, LocalDate endLocalDate) {
        long endNum;
        List<String> retDataKey;
        switch (type){
            case 2:
                endNum =0;
                LocalDate indexStart = null;
                if(! startLocalDate.getDayOfWeek().equals(DayOfWeek.MONDAY)){
                    indexStart = startLocalDate.with(TemporalAdjusters.next(DayOfWeek.MONDAY));
                    endNum++;
                }else{
                    indexStart = startLocalDate;
                }

                //endNum = ChronoUnit.WEEKS.between(indexStart,endLocalDate);
                long diffDay = ChronoUnit.DAYS.between(indexStart,endLocalDate);
                if(diffDay>=7){
                    endNum = endNum+Double.valueOf(Math.ceil(diffDay/7.00)).longValue();
                    if(endLocalDate.getDayOfWeek().equals(DayOfWeek.MONDAY)){
                        endNum++;
                    }
                }else if(diffDay>0){
                    endNum++;
                }else {
                    if(indexStart.equals(endLocalDate)){
                        endNum++;
                    }
                }

                endNum = endNum<=0?1:endNum;
                retDataKey = Stream.iterate(startLocalDate, nextDate->nextDate.with
                        (TemporalAdjusters.next(DayOfWeek.MONDAY))).limit(endNum)
                        //.map(item -> item.getYear()+(item.get(ChronoField.ALIGNED_WEEK_OF_YEAR)+SPILT_STR)+
                        // 占位和补位参考https://www.cnblogs.com/xrab/articles/5414160.html
                        .map(item -> item.getYear()+(String.format("%02d",item.get( WeekFields.ISO.weekOfWeekBasedYear()))+SPILT_STR)+
                                // mysql获取日期 http://www.w3school.com.cn/sql/func_date_format.asp
                                // java 获取周https://blog.csdn.net/sayWhat_sayHello/article/details/80589304
                                item.format(DateTimeFormatter.ofPattern("MM.dd"))+"-"+
                                item.with(TemporalAdjusters.next(DayOfWeek.SUNDAY)).
                                        format(DateTimeFormatter.ofPattern("MM.dd"))).collect(Collectors.toList());
                if(startLocalDate.getDayOfWeek().equals(DayOfWeek.SUNDAY)){
                    String lastVal = retDataKey.get(0);
                    lastVal = lastVal.substring(0,(lastVal.lastIndexOf("-")+1)).concat(startLocalDate.format(DateTimeFormatter.ofPattern("MM.dd")));
                    retDataKey.set(0,lastVal);
                }
                int index = retDataKey.size()-1;
                String lastVal = retDataKey.get(index);
                lastVal = lastVal.substring(0,(lastVal.lastIndexOf("-")+1)).concat(endLocalDate.format(DateTimeFormatter.ofPattern("MM.dd")));
                retDataKey.set(index,lastVal);
                break;
            case 3:
                endNum = Period.between(startLocalDate,endLocalDate).getMonths();
                endNum = endNum<=0?1:endNum+1;
                retDataKey = Stream.iterate(startLocalDate, nextDate->nextDate.plusMonths(1)).limit(endNum)
                        .map(item -> item.format(DateTimeFormatter.ofPattern("yyyyMM"))).collect(Collectors.toList());
                break;
            default:
                endNum = endLocalDate.toEpochDay()-startLocalDate.toEpochDay();
                endNum = endNum<=0?1:endNum+1;
                retDataKey = Stream.iterate(startLocalDate, nextDate->nextDate.plusDays(1)).limit(endNum)
                        .map(item -> item.format(DateTimeFormatter.ofPattern("yyyyMMdd"))).collect(Collectors.toList());
        }
        return retDataKey;
    }
}
