package com.yizhi.course.application.service.using;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.course.application.mapper.CourseStudentStatusMapper;
import com.yizhi.course.application.mapper.RecordeMapper;
import com.yizhi.system.application.system.remote.RemoteDaIndicatorSystemClient;
import com.yizhi.system.application.vo.domain.DaIndicatorCourseFinish;
import com.yizhi.system.application.vo.domain.DaIndicatorCourseTime;
import com.yizhi.util.application.num.NumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @ClassName RemoteDaIndicatorServiceUsing
 * @Description TODO
 * @Author shengchenglong
 * @DATE 2019-10-10 15:01
 * @Version 1.0
 */
@Service
public class RemoteDaIndicatorServiceUsing {

    @Autowired
    private CourseStudentStatusMapper courseStudentStatusMapper;
    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private RemoteDaIndicatorSystemClient remoteDaIndicatorSystemClient;
    @Autowired
    private RecordeMapper recordeMapper;

    /**
     * dashboard 课程完成数
     *
     * @param siteId
     * @param startDate
     * @param endDate
     * @param processTime
     * @return
     */
    public boolean courseFinish(long siteId, Date startDate, Date endDate, Date processTime) {
        int courseFinishInPreWeek = courseStudentStatusMapper.courseFinishInWeek(siteId, startDate, endDate);

        Date dateOfPrePreWeek = DateUtil.offset(startDate, DateField.DAY_OF_YEAR, -7);
        Date prePreStartDate = DateUtil.beginOfWeek(dateOfPrePreWeek);
        Date prePreEndDate = DateUtil.endOfWeek(dateOfPrePreWeek);

        int courseFinishInPrePreWeek = courseStudentStatusMapper.courseFinishInWeek(siteId, prePreStartDate, prePreEndDate);

        String weekChain = "-";

        if (courseFinishInPrePreWeek > 0) {
            weekChain = NumUtil.getChainPercentage(courseFinishInPreWeek, courseFinishInPrePreWeek, 0);
        }

        DaIndicatorCourseFinish daIndicatorCourseFinish = new DaIndicatorCourseFinish();
        daIndicatorCourseFinish.setEndDay(endDate);
        daIndicatorCourseFinish.setFinishCount(courseFinishInPrePreWeek);
        daIndicatorCourseFinish.setId(idGenerator.generate());
        daIndicatorCourseFinish.setProcessTime(processTime);
        daIndicatorCourseFinish.setSiteId(siteId);
        daIndicatorCourseFinish.setStartDay(startDate);
        daIndicatorCourseFinish.setWeekChain(weekChain);

        return remoteDaIndicatorSystemClient.insertCourseFinish(daIndicatorCourseFinish);
    }

    /**
     * dashboard 课程学习时长
     *
     * @param siteId
     * @param startDate
     * @param endDate
     * @param processTime
     * @return
     */
    public boolean courseTime(long siteId, Date startDate, Date endDate, Date processTime) {
        Long courseTimeInPreWeek = recordeMapper.courseTimeInWeek(siteId, startDate, endDate);

        Date dateOfPrePreWeek = DateUtil.offset(startDate, DateField.DAY_OF_YEAR, -7);
        Date prePreStartDate = DateUtil.beginOfWeek(dateOfPrePreWeek);
        Date prePreEndDate = DateUtil.endOfWeek(dateOfPrePreWeek);

        Long courseTimeInPrePreWeek = recordeMapper.courseTimeInWeek(siteId, prePreStartDate, prePreEndDate);

        String weekChain = "-";

        if (courseTimeInPrePreWeek > 0) {
            weekChain = NumUtil.getChainPercentage(courseTimeInPreWeek, courseTimeInPrePreWeek, 0);
        }

        DaIndicatorCourseTime record = new DaIndicatorCourseTime();
        record.setEndDay(endDate);
        record.setId(idGenerator.generate());
        record.setProcessTime(processTime);
        record.setSiteId(siteId);
        record.setStartDay(startDate);
        record.setTimeCount(addOne(courseTimeInPreWeek));
        record.setWeekChain(weekChain);

        return remoteDaIndicatorSystemClient.insertCourseTime(record);
    }

    /**
     * 学习时长只要带小数自动进一，
     * @param pre
     * @return
     */
    public static Integer addOne(Long pre) {
        Integer add = 0;
        if (pre > 0) {
            add = 1;
        }
        Integer integer = new Integer(String.valueOf(pre / 60 / 60));
        return integer + 1;
    }

}
