package com.yizhi.course.application.task;

import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.course.application.domain.CourseStudentStatus;
import com.yizhi.course.application.service.ICourseStudentStatusService;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 21:21 2018/11/14
 * @Modified By
 */
@Component
public class CourseStatusSyncAsync extends AbstractDefaultTask<String, Map<String,Object>> {

    private static final Logger logger = LoggerFactory.getLogger(CourseStatusSyncAsync.class);

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private ICourseStudentStatusService studentStatusService;

    @Override
    protected String execute(Map<String, Object> stringObjectMap) {

        String active = (String) stringObjectMap.get("active");

        List<CourseStudentStatus> data = studentStatusService.getTrainningMqInfo();

        logger.info("培训项目和课程数据同步：处理总条数={}",data.size());

        int index = 0;
        for(CourseStudentStatus item : data){
            index++;
            logger.info("培训项目和课程数据同步,发送消息参数 courseId={}, accountId={}, siteId={}",item.getCourseId(),item.getAccountId(),item.getSiteId());
            logger.info("处理第{}条数据",index);
            if(!"prod".equals(active)){
                continue;
            }
            try {
                cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE, new EventWrapper<>(
                        item.getCourseId(), // 课程id
                        TrainingProjectEvent.getInstanceCourse(
                                item.getCourseId(), // 课程id
                                item.getAccountId(), // 账号id
                                new Date(),
                                0L,//学习时长
                                true,//完成状态
                                item.getSiteId()//站点id
                        )
                ));
            } catch (Exception e) {
                logger.info("培训项目和课程数据同步，异常={}" ,e);
            }
        }

        return "ok";
    }
}
