package com.yizhi.course.application.task;

import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.service.IRecordeService;
import com.yizhi.course.application.util.JsonUtils;
import com.yizhi.course.application.vo.CourseStudyDurationRanking;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.function.BiConsumer;
import java.util.stream.Collectors;

/**
 * 生成前一天的排名
 */
@Component
public class CourseStudyHourRankingAsync extends AbstractDefaultTask<String, Map<String,Object>> {

    public static final String REDIS_KEY="course:ranking";
    @Autowired
    private ReportClient reportClient;

    @Autowired
    private IRecordeService recordeService;

    @Autowired
    private RedisCache redisCache;

    private Logger LOGGER = LoggerFactory.getLogger(CourseStudyHourRankingAsync.class);

    @Override
    protected String execute(Map<String, Object> stringObjectMap) {
        // 查询全部的站点信息
        //List<SiteVO> siteVOList = siteClient.findByCompanyId(null);
        // 遍历每个站点
        Long siteId = Long.valueOf(stringObjectMap.get("siteId").toString());
        String active = (String) stringObjectMap.get("active");
        // 查询站点下的用户信息
        ReportRangeAccountReqVO rangeAccountReqVO = new ReportRangeAccountReqVO();
        rangeAccountReqVO.setSiteId(siteId);
        rangeAccountReqVO.setIsParentOrg(false);
        rangeAccountReqVO.setEnabled(true);
        List<ReportAccountRespVO> accountData = reportClient.getRangeAccounts(rangeAccountReqVO);

        int sizeTmp = 0;
        if(!CollectionUtils.isEmpty(accountData)){
            sizeTmp = accountData.size();
        }
        LOGGER.info("站点{} 环境变量={} 查询用户数据={} 条",siteId,active,sizeTmp);

        if(CollectionUtils.isEmpty(accountData)){
            return "null";
        }
        // 查询网站某个站点的用户学习排名
        List<CourseStudyDurationRanking> rankingData = recordeService.studyHourRanking(siteId, active==null?"":active);
        Map<Long,List<CourseStudyDurationRanking>> groupData = null;
        if(!CollectionUtils.isEmpty(rankingData)){
            groupData = rankingData.stream().parallel().collect(Collectors.groupingBy(CourseStudyDurationRanking::getAccountId));
        }
        // 整合数据
        List<CourseStudyDurationRanking> allData = new ArrayList<>();

        List<CourseStudyDurationRanking> retData = new ArrayList<>();
        if(groupData == null){
            accountData.parallelStream()
                    //.skip(0) //0开始
                    .limit(100).forEach(
                            obj->{
                                setNewVo(retData, obj);
                            }
            );
            retData.sort(Comparator.comparing(CourseStudyDurationRanking::getAccountId));
            allData = retData;
        }else{

            for (ReportAccountRespVO accountRespVO :accountData) {
                Long userId = accountRespVO.getUserId();
                if(groupData.containsKey(userId)){
                    CourseStudyDurationRanking tmpData = groupData.get(userId).get(0);
                    tmpData.setProfilePicture(accountRespVO.getHeadPortrait());
                    tmpData.setOrgName(accountRespVO.getOrgName());
                    tmpData.setAccountFullName(accountRespVO.getUserFullName());
                    tmpData.setAccountName(accountRespVO.getUserName());
                    allData.add(tmpData);
                }else{
                    setNewVo(allData, accountRespVO);
                }
            }
            allData.sort(Comparator.comparing(CourseStudyDurationRanking::getDurationSum).reversed()
                    // 如果两个学习总时长一样，再按用户id升序排列
                    .thenComparing(CourseStudyDurationRanking::getAccountId));
            allData = allData.stream().limit(100).collect(Collectors.toList());
        }

        int size  = allData.size();
        for(int i=0 ;i<size ;i++ ){
            CourseStudyDurationRanking itemVo = allData.get(i);
            itemVo.setRanking(i+1);
        }

        //JsonUtils.json2JavaBeanList(JsonUtils.map2Json(allData), CourseStudyDurationRankingVO.class);
        redisCache.hset(REDIS_KEY,siteId.toString(), JsonUtils.map2Json(allData));

        return "OK";
    }

    private static void setNewVo(List<CourseStudyDurationRanking> allData, ReportAccountRespVO accountRespVO) {
        CourseStudyDurationRanking voNew = new CourseStudyDurationRanking();
        voNew.setAccountFullName(accountRespVO.getUserFullName());
        voNew.setAccountId(accountRespVO.getUserId());
        voNew.setAccountName(accountRespVO.getUserName());
        voNew.setDurationSum(0L);
        voNew.setOrgName(accountRespVO.getOrgName());
        voNew.setProfilePicture(accountRespVO.getHeadPortrait());
        allData.add(voNew);
    }


    /**
     *
     * @param <T>
     * @param startIndex 开始遍历的索引
     * @param elements 集合
     * @param action
     */
    public static <T> void forEach(int startIndex,Iterable<? extends T> elements, BiConsumer<Integer, ? super T> action) {
        Objects.requireNonNull(elements);
        Objects.requireNonNull(action);
        if(startIndex < 0) {
            startIndex = 0;
        }
        int index = 0;
        for (T element : elements) {
            index++;
            if(index <= startIndex) {
                continue;
            }

            action.accept(index-1, element);
        }
    }
    /*@Test
    public void test() {
        List<String> list = Arrays.asList("1","2", "3");
        ForEachUtils.forEach(0, list, (index, item) -> {
            log.info(index + " - " + item);
        });
    }
    @Test
    public void test() {
        List<String> list = Arrays.asList("x","y", "z");
        ForEachUtils.forEach(1, list, (index, item) -> {
            log.info(index + " - " + item);
        });
    }*/
}
