package com.yizhi.course.application.task;

import com.alibaba.fastjson.JSON;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.course.application.service.IReportMkCourseService;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.util.application.constant.MarketingCourseReportType;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author: XieHaijun
 * @Description:
 * @Date: Created in 16:05 2018/8/27
 * @Modified By
 */
@Component
public class MarketingCourseExportXlsxAsync extends AbstractDefaultTask<String, Map<String,Object>> {

    private static final Logger logger = LoggerFactory.getLogger(MarketingCourseExportXlsxAsync.class);

    @Autowired
    private IReportMkCourseService reportMkCourseService;

    @Autowired
    private OrganizationClient organizationClient;

    @Autowired
    private AccountClient accountClient;

    @Override
    protected String execute(Map<String, Object> param) {

        RequestContext requestContext = (RequestContext)param.get("requestContext");
        ContextHolder.set(requestContext);
        // logger.info("########################################报表统计导出开始,上下文信息={}", JSON.toJSONString(requestContext));
        String business = (String) param.get("business");// 获取业务标识
        String startDate = (String) param.get("startDate");
        String endDate = (String) param.get("endDate");

        String[] titles = null; // 标题字段列表
        String exportName = (String) param.get("exportName");
        int cloumSize = 0;
        String exportDesc = "统计周期："+LocalDate.parse(startDate, DateTimeFormatter.ISO_DATE).
                format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))+"～"+
                LocalDate.parse(endDate, DateTimeFormatter.ISO_DATE).
                        format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));

        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter=DateTimeFormatter.ofPattern("yyyyMMddHHmmssSSS");
        Date submitTime = Date.from(now.atZone(ZoneId.systemDefault()).toInstant());
        Long taskId = submitTime.getTime();//任务标识
        //String serialNo = UUID.randomUUID().toString();//任务编号
        String serialNo = (String) param.get("serialNo");
        Long submitterId = requestContext.getAccountId();
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();
        String taskName = null;
        TaskContext taskContext = null;

        List<Object[]> exportBodyData = new ArrayList<>();

        List<Long> orgKwds= null;
        String orgKwd = (String) param.get("orgKwd");
        if(StringUtils.isNotBlank(orgKwd)){
            orgKwds = organizationClient.findOrgIdByNameOrCode(orgKwd,companyId);
        }

        List<Long> accountKwds = null;
        String accountKwd=(String) param.get("accountKwd");
        // 获取用户关键字对应的用户id集合
        if(StringUtils.isNotBlank(accountKwd)){
            accountKwds = accountClient.findAIdsByKeyword(accountKwd,companyId);
        }

        Long cid = requestContext.getCompanyId();
        //Long siteId = requestContext.getSiteId();
        List<Long> orgs = requestContext.getOrgIds();

        exportBodyData = reportMkCourseService.downStatistics(startDate, endDate, orgKwds,
                accountKwds, cid, siteId, orgs, business);

        taskName = exportName+"_"+serialNo;

        // 异步日志开始工作
        taskContext = new TaskContext(taskId,serialNo,taskName,submitterId,submitTime,siteId,companyId);
        working(taskContext);

        if(StringUtils.equals(MarketingCourseReportType.BY_COURSE_OPEN_EXPORT,business)){
            titles = new String[]{"用户名","姓名","用户状态","课程名称","课程编码","阅读次数","所在部门","所在组织架构"};
        }else if(StringUtils.equals(MarketingCourseReportType.BY_USER_OPEN_EXPORT,business)){
            titles = new String[]{"用户名","姓名","用户状态","课程名称","课程编码","打开时间","所在部门","所在组织架构"};
        }else if(StringUtils.equals(MarketingCourseReportType.BY_USER_DETAIL_EXPORT,business)){
            titles = new String[]{"用户名","姓名","用户状态","课程名称","课程编码","阅读时间","所在部门","所在组织架构"};
        }
        cloumSize = titles.length;
        // 生成excel
        String fileName = taskName+".xlsx";

        logger.info("导出数据={},fileName={}", JSON.toJSONString(exportBodyData),fileName);
        logger.info("titles={},exportName={}",titles,exportName);
        logger.info("cloumSize={},exportDesc={}",cloumSize,exportDesc);
        String upLoadUrl = createExcel(exportBodyData, titles, exportName, cloumSize, exportDesc,fileName);

        success(taskContext,"成功", upLoadUrl);
        logger.info("########################################报表统计导出完成,下载url="+upLoadUrl);
        return "ok";
    }

/*    public static void main(String[] args) {
        List<Object[]> exportBodyData = new ArrayList<>();
        Object[] data1 = new Object[]{"heqq@kmelearning.com", "", "正常", "5步搞定银行服务", "dy2018101905", 6, "银行业务中心"};
        exportBodyData.add(data1);
        String[] titles = new String[]{"用户名","姓名","用户状态","打开课程数","有效转发课程数","阅读次数","所在部门"};
        int cloumSize = titles.length;
        String exportName = "导出顶上大标题";
        String exportDesc = "导出的小标题说明";
        String fileName = "测试"+".xlsx";
        String upLoadUrl = createExcel(exportBodyData, titles, exportName, cloumSize, exportDesc,fileName);
    }*/
    /**
     * 生成excel
     * @param exportBodyData 导出的主体数据
     * @param titles 标题头
     * @param exportName 导出顶上大标题
     * @param cloumSize 列大小
     * @param exportDesc 导出的小标题说明
     */
    public static String createExcel(List<Object[]> exportBodyData, String[] titles,
                                     String exportName, int cloumSize, String exportDesc,
                                     String fileName) {

        String requestPath= FileConstant.SAVE_PATH;
        //String requestPath = "D:\\data";
        File fileDir=new File(requestPath);
        if(!fileDir.exists()) {
            fileDir.mkdir();
        }
        //String fileDir= "D:\\data";
        String filePath = fileDir+File.separator+fileName;
        String upLoadUrl = "";
        File file = new File(filePath);
        try(//HSSFWorkbook workbook = new HSSFWorkbook();
            FileOutputStream os = new FileOutputStream(filePath);
            //XSSFWorkbook workbook1 = new XSSFWorkbook(os);
            //XSSFWorkbook workbook1 = new XSSFWorkbook(new FileInputStream(new File(filePath)));
            SXSSFWorkbook workbook = new SXSSFWorkbook(new XSSFWorkbook(), 100);){
            // 创建第一个sheet页
            Sheet sheet = workbook.createSheet(exportName);
            //sheet.setDefaultColumnWidth(20);//设置sheet页默认列宽度设置成100个字符宽度

            int index = 0;
            if(StringUtils.isNotBlank(exportName)){
                // 设置单元格的高度，其实就是设置行的高度
                // HeightInPoints的单位是点，而Height的单位是1/20个点，所以Height的值永远是HeightInPoints的20倍
                Row row1 = sheet.createRow(index);
                //row.setHeight(Short.valueOf("400"));//20*200
                Cell exportNameCell = row1.createCell(0);
                exportNameCell.setCellValue(exportName);
                CellStyle exportNameStye = workbook.createCellStyle();
                Font headfont = workbook.createFont();
                //设置字体大小
                headfont.setFontHeightInPoints((short) 18);
                // //粗体显示
                headfont.setBold(true);
                exportNameStye.setFont(headfont);
                exportNameStye.setAlignment(HorizontalAlignment.CENTER);//居中
                exportNameStye.setVerticalAlignment(VerticalAlignment.CENTER);//设置垂直居中
                exportNameCell.setCellStyle(exportNameStye);

                //sheet.createRow(index).setRowStyle();
                //合并列
                CellRangeAddress region=new CellRangeAddress(index, index, 0, cloumSize-1);
                sheet.addMergedRegion(region);
                index++;
            }
            if(StringUtils.isNotBlank(exportDesc)){
                Cell exportDescCell = sheet.createRow(index).createCell(0);
                exportDescCell.setCellValue(exportDesc);
                CellStyle exportDescStye = workbook.createCellStyle();
                Font headfont = workbook.createFont();
                //设置字体大小
                headfont.setFontHeightInPoints((short) 14);
                // //粗体显示
                headfont.setBold(true);
                exportDescStye.setFont(headfont);
                exportDescStye.setAlignment(HorizontalAlignment.LEFT);
                exportDescStye.setVerticalAlignment(VerticalAlignment.CENTER);//设置垂直居中
                exportDescCell.setCellStyle(exportDescStye);
                //sheet.createRow(index).setRowStyle();
                //合并列
                CellRangeAddress region=new CellRangeAddress(index, index, 0, cloumSize-1);
                sheet.addMergedRegion(region);
                index++;
            }
            if(titles!=null){
                Row titleRow = sheet.createRow(index);
                CellStyle titlesStye = workbook.createCellStyle();
                Font headfont = workbook.createFont();
                //设置字体大小
                headfont.setFontHeightInPoints((short) 12);
                // //粗体显示
                headfont.setBold(true);
                titlesStye.setFont(headfont);
                titlesStye.setAlignment(HorizontalAlignment.CENTER);
                titlesStye.setVerticalAlignment(VerticalAlignment.CENTER);
                //titleRow.setRowStyle(titlesStye);
                for(int k =0;k<titles.length;k++){
                    //sheet.autoSizeColumn(k, true);// 列自适应宽度
                    Cell titleCell = titleRow.createCell(k);
                    titleCell.setCellStyle(titlesStye);
                    titleCell.setCellValue((String)titles[k]);
                    sheet.setColumnWidth(k,20 * 256);
                }

                index++;
            }
            for(int rownum = 0; rownum < exportBodyData.size(); rownum++){
                Row row = sheet.createRow(index);
                Object[] itemData = exportBodyData.get(rownum);
                int cellnumSum = itemData.length;


                for(int cellnum = 0; cellnum < cellnumSum; cellnum++){
                    //sheet.setColumnWidth(cellnum,100 * 256);
                    Cell cell = row.createCell(cellnum);
                    Object valObj = itemData[cellnum];
                    String val = valObj == null?" ":valObj.toString();
                    cell.setCellValue(val);
                }
                index++;
            }
            //ByteArrayOutputStream os = new ByteArrayOutputStream();
            workbook.write(os);
            //阿里云返回url
            upLoadUrl = OssUpload.upload(filePath, fileName);
            file.delete();
            //success(taskContext,"成功", upLoadUrl);
            //data = os.toByteArray();
            logger.info("文档生成，上传地址upLoadUrl={}",upLoadUrl);
        }catch (IOException e){

            logger.info("生成excel发送异常{}",e);
            return null;
        }
        return upLoadUrl;
    }

}
