package com.yizhi.course.application.task;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.course.application.enums.MaterialTypeEnum;
import com.yizhi.course.application.mapper.CourseMapper;
import com.yizhi.course.application.service.IMaterialService;
import com.yizhi.course.application.vo.MaterialExportVO;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: MaterialListExportAsync
 * @author: zjl
 * @date: 2021/1/29  11:39
 */
@Component
public class MaterialListExportAsync extends AbstractDefaultTask<String, Map<String,Object>> {

    private static final Logger logger = LoggerFactory.getLogger(MaterialListExportAsync.class);

    @Autowired
    private CourseMapper courseMapper;

    @Autowired
    private IMaterialService materialService;

    @Override
    protected String execute(Map<String, Object> arg0) {

        Long accountId=(Long) arg0.get("accountId");
        Long siteId=(Long) arg0.get("siteId");
        Long companyId=(Long) arg0.get("companyId");
        Long taskId=(Long) arg0.get("taskId");
        Date submitTime=(Date) arg0.get("submitTime");
        String serialNo=(String) arg0.get("serialNo");
        String taskName=(String) arg0.get("taskName");
        String name=(String) arg0.get("name");
        Long classifyId=(Long) arg0.get("classifyId");
        Integer shelves=(Integer) arg0.get("shelves");
        Integer type=(Integer) arg0.get("type");
        Integer courseType=(Integer) arg0.get("courseType");

        /**
         * 走异步任务
         */
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
        working(taskContext);


        List<Long> courseIds = courseMapper.courseIdlistExport(name, classifyId, shelves, companyId, siteId, null, null, type, courseType);
        if (CollectionUtils.isEmpty(courseIds)) {
            success(taskContext, "未找到数据", "");
        }

        List<MaterialExportVO> voList = materialService.getMaterialExportVOList(courseIds);

        String upLoadUrl=null;
        String requestPath= "D:\\dev_soft";//FileConstant.SAVE_PATH;
        File fileDir=new File(requestPath);
        if(!fileDir.exists()) {
            fileDir.mkdir();
        }
        try {
            //excel生成：excel-sheet-row-cell
            HSSFWorkbook workbook = new HSSFWorkbook();
            HSSFSheet sheet = workbook.createSheet("课程素材清单"  + DateUtil.format(submitTime, "yyyyMMddHHmmss"));
            // 在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short,
            // 创建第1行
            //创建单元格格式
            HSSFCellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.LEFT);//创建居左格式
            HSSFRow row = sheet.createRow((int) 0);
            //合并单元格
            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 9); // 起始行, 终止行, 起始列, 终止列
            sheet.addMergedRegion(cra);
            HSSFCell cell = row.createCell((short) 0);
            cell.setCellValue("课程素材清单" + "-" + DateUtil.format(submitTime, "yyyyMMddHHmmss"));


            row = sheet.createRow((int) 1);
            row.createCell((short) 0).setCellValue("课程名称");
            row.createCell((short) 1).setCellValue("课程编码");
            row.createCell((short) 2).setCellValue("课程类型");
            row.createCell((short) 3).setCellValue("课程状态");
            row.createCell((short) 4).setCellValue("章节名称");
            row.createCell((short) 5).setCellValue("素材/测验类型");
            row.createCell((short) 6).setCellValue("素材/测验编码");
            row.createCell((short) 7).setCellValue("素材/测验名称");
            row.createCell((short) 8).setCellValue("素材时长");
            row.createCell((short) 9).setCellValue("素材/测验积分");

            int index = 2;
            int s = 0;
            for (MaterialExportVO vo : voList) {
                if (vo == null) {
                    continue;
                }
                row = sheet.createRow((int) index);
                cell = row.createCell((short) 0);
                if (vo.getCourseName() != null) {
                    cell.setCellValue(vo.getCourseName());
                }
                cell = row.createCell((short) 1);
                if (vo.getCourseCode() != null) {
                    cell.setCellValue(vo.getCourseCode());
                }
                cell = row.createCell((short) 2);
                if (vo.getCourseType() != null) {
                    cell.setCellValue(vo.getCourseType() == 0 ? "一般课程" : "scorm课程");
                }
                cell = row.createCell((short) 3);
                if (vo.getShelves() != null) {
                    cell.setCellValue(vo.getShelves() == 0 ? "已下架" : (vo.getShelves() == 1 ? "已上架" : "草稿"));
                }
                cell = row.createCell((short) 4);
                if (vo.getChapterName() != null) {
                    cell.setCellValue(vo.getChapterName());
                }
                cell = row.createCell((short) 5);
                if (vo.getMaterialType() != null) {
                    cell.setCellValue(MaterialTypeEnum.relation.get(vo.getMaterialType()));
                }
                cell = row.createCell((short) 6);
                if (vo.getMaterialCode() != null) {
                    cell.setCellValue(vo.getMaterialCode());
                }
                cell = row.createCell((short) 7);
                if (vo.getMaterialName() != null) {
                    cell.setCellValue(vo.getMaterialName());
                }
                cell = row.createCell((short) 8);
                if (vo.getMaterialDuration() != null) {
                    cell.setCellValue(vo.getMaterialDuration());
                }
                cell = row.createCell((short) 9);
                if (vo.getScore() != null) {
                    cell.setCellValue(vo.getScore());
                }
                index++;
                s++;
            }
            StringBuffer fileNameSb = new StringBuffer().append("课程素材清单").append("-").append(DateUtil.format(submitTime, "yyyyMMddHHmmss")).append(".xls");
            String fileName = fileNameSb.toString();
            String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                workbook.write(os);
                //阿里云返回url
                upLoadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", upLoadUrl);
            } catch (Exception e1) {
                logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误", e1);
                fail(taskContext, "写入过程中发生错误");

            } finally {
                if (os != null) {
                    os.close();
                }
                if (workbook != null) {
                    workbook.close();
                }
                if (file != null) {
                    file.delete();
                }
            }
        } catch (Exception e) {
            logger.error("创建Excel的过程中发生错误", e);
            fail(taskContext, "签到统计导出过程中发生错误，请查看日志");
        }

        return upLoadUrl;
    }
}
