package com.yizhi.course.application.util;

import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.course.application.domain.Course;
import com.yizhi.message.application.enums.EvenType;
import com.yizhi.message.application.enums.RelationType;
import com.yizhi.message.application.vo.MessageRemindVo;
import com.yizhi.message.application.vo.TaskVo;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class CourseEvenSendMessage {


    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    private Logger logger = LoggerFactory.getLogger(CourseEvenSendMessage.class);


    /**
     * 触发事件,
     *
     * @param course    业务参数对象
     * @param accountId
     */
    public void evenSendMessage(Course course, Long accountId, EvenType evenType) {


        if (course != null) {
            com.yizhi.core.application.vo.TaskVo taskVo = new com.yizhi.core.application.vo.TaskVo();
            taskVo.setTaskName(course.getName());

            MessageRemindVo vo = new MessageRemindVo();
            vo.setTaskVo(taskVo);
            vo.setVisibleRange(course.getScope());
            vo.setMessageId(evenType.getKey());
            vo.setMessageType(3);
            vo.setRelationId(course.getId());
            vo.setRelationType(RelationType.KC.getKey());
            vo.setSendType(1);
            //定时任务默认五分钟执行一次
            vo.setSendTime(DateUtils.addMinutes(new Date(), 0));
            if (accountId != null) {
                vo.setAccountId(accountId);
            }
            RequestContext requestContext = new RequestContext();
            requestContext.setCompanyId(course.getCompanyId());
            requestContext.setSiteId(course.getSiteId());
            requestContext.setAccountId(course.getCreateById());
            requestContext.setAccountName(course.getCreateByName());
            vo.setRequestContext(requestContext);
            try {
                //临时取消触发功能
//                cloudEventPublisher.publish(Constans.MESSAGE_QUEUE, new EventWrapper<MessageRemindVo>(null, vo));
//                logger.error("发送消息成功=====================");
            } catch (Exception e) {
                e.printStackTrace();
                logger.error("发送消息失败=====================", e);
            }

        }
    }
}
